/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.client.internal.BuildClient");

(function() {
	
var CACHE_DURATION_DEFAULT = 300000;		// 5 minutes
var CACHE_DURATION_QUEUE = 120000;			// 2 minutes
var CACHE_DURATION_DEFINITION = 3600000;	// 1 hour

var CACHE_GC_INTERVAL = 600000;				// 10 minutes

var BuildRoot = net.jazz.ajax._contextRoot + "/resource/virtual/build/";
var enc = encodeURIComponent;

var cache = {};
setInterval(function() {
	var time = new Date().getTime();
	for (var i in cache) {
		if (cache[i].expires && cache[i].expires < time) {
			delete cache[i];
		}
	}
},CACHE_GC_INTERVAL);
/*
 * Query interface for builds.
 */
com.ibm.team.build.web.client.internal.BuildClient = {
	getResults: function(args) {
		var url = this._getResultsURL(args);
		return this._doExecuteGet(url, args, CACHE_DURATION_DEFAULT);
	},
	flushResults: function(args) {
		var url = this._getResultsURL(args);
		delete cache[url];
	},
	_getResultsURL: function(args) {
		return BuildRoot + "results?definitionUUID=" + enc(args.definitionItemId) + "&profile=REDUCED";
	},
	
	getResultRecords: function(args) {
		var url = BuildRoot + "resultrecords?";
		if (args.definitionItemId) {
			url += "definitionUUID=" + enc(args.definitionItemId);
		} else if (args.uuids) { 
			url += "uuid=" + args.uuids.join("&uuid=");
		}
		url += "&profile=REDUCED";
		return this._doExecuteGet(url, args, CACHE_DURATION_DEFAULT);
	},
	flushResultRecordsByUUID: function(uuid) {
		for (var i in cache) {
			if (i.indexOf("resultrecords") != -1 && i.indexOf(uuid) != -1) {
				delete cache[i];
			}
		}
	},
	
	getEngineStatusRecords: function(args) {
		var url = BuildRoot + "enginestatusrecords?projectAreaUUID=" + enc(args.projectAreaItemId) + "&profile=REDUCED";
		return this._doExecuteGet(url, args, CACHE_DURATION_DEFAULT);
	},
	
	getQueue: function(args) {
		var url = this._getQueueURL(args);
		return this._doExecuteGet(url, args, CACHE_DURATION_QUEUE);
	},
	flushQueue: function(args) {
		var url = this._getQueueURL(args);
		delete cache[url];
	},
	_getQueueURL: function(args) {
		return BuildRoot + "queue?projectAreaUUID=" + enc(args.projectAreaItemId) + "&profile=REDUCED";
	},
	
	getDefinitionStatusRecords: function(args) {
		var url = BuildRoot + "definitionstatusrecords?projectAreaUUID=" + enc(args.projectAreaItemId) + "&includeChildTeamAreas=true&profile=REDUCED";
		return this._doExecuteGet(url, args, CACHE_DURATION_DEFAULT);
	},
	
	getDefinition: function(args) {
		var url = BuildRoot + "definition/" + enc(args.definitionItemId);
		return this._doExecuteGet(url, args, CACHE_DURATION_DEFINITION);
	},
	
	getResultPresentation: function(args){
		var url = BuildRoot +"resultpresentation/" + enc(args.resultItemId);
		if (args.types) {
			url += "?type=" + args.types.join("&type=");
		}
		return this._doExecuteGet(url, args);
	}, 
	
	getCompilationSummary: function(args){
	 	var url = BuildRoot +"compile/" + enc(args.resultItemId) + "/summary";
		return this._doExecuteGet(url, args);		
	},
	
	getJUnitSummary: function(args){
	 	var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/summary";
		return this._doExecuteGet(url, args);		
	},
	
	getCompilationContributions: function(args) {
	 	var url = BuildRoot + "compile/" + enc(args.resultItemId) + "/contributions";
		return this._doExecuteGet(url, args);
	},
	 
	getCompilationInfo: function(args){
	 	var url = BuildRoot +"compile/" + enc(args.resultItemId) + "/components";
		return this._doExecuteGet(url, args);
	},

	getCompilationErrors: function(args){
		var url = BuildRoot + "compile/contributions/" + enc(args.contributionItemId) + "/errordata";
		return this._doExecuteGet(url, args);
	},

	getCompilationWarnings: function(args){
		var url = BuildRoot + "compile/contributions/" + enc(args.contributionItemId) + "/warningdata";
		return this._doExecuteGet(url, args);
	},
	 
	getJUnitInfo: function(args){
		var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/components";
		return this._doExecuteGet(url, args);
	},
	 
	getPackages: function(args){
		var url = BuildRoot +"compile/" + enc(args.resultItemId) + "/packages?component=" + enc(args.componentName);
		return this._doExecuteGet(url, args);
	}, 
	
	getChildrenOfPackage: function(args){
		var url = BuildRoot +"compile/" + enc(args.resultItemId) + "/packages/" + enc(args.packageItemId) + "/children";
		return this._doExecuteGet(url, args);		
	},
	
	getSuites: function(args ){
		var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/testsuites?component=" + enc(args.componentName);
		return this._doExecuteGet(url, args);
	},
	
	getSuiteClasses: function(args){
		var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/testsuites/" + enc(args.suiteItemId) + "/testclasses";		
		return this._doExecuteGet(url, args);		
	},
	
	getTestCases: function(args){
		var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/testclasses/" + enc(args.classItemId);		
		return this._doExecuteGet(url, args);		
	},
	
	getErrorContent: function(args){
		var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/testclasses/" + enc(args.classItemId) + "/testcases/" + enc(args.caseName) + "/errordata";		
		return this._doExecuteGet(url, args);		
	},
	
	getFailureContent: function(args){
		var url = BuildRoot +"junit/" + enc(args.resultItemId) + "/testclasses/" + enc(args.classItemId) + "/testcases/" + enc(args.caseName) + "/failuredata";		
		return this._doExecuteGet(url, args);		
	},
	
	deleteBuildResult: function(args){
		var url = BuildRoot +"result/" + enc(args.resultItemId);
		return this._doExecuteDelete(url, args);		
	},
	
	updateBuildState: function(args){
		var modArgs = dojo.mixin({
			putData: dojo.toJson({buildState: args.buildState})
		}, args);
		delete modArgs.buildState;
		
		var url = BuildRoot +"result/" + enc(args.resultItemId) + "/buildstate";
		return this._doExecutePut(url, modArgs);
	},
	//send execution request for modification of tags and delete allowed.
	updateBuildProperties: function(args){
		var url = BuildRoot +"result/" + enc(args.resultItemId);
		var modArgs = dojo.mixin({
			putData: dojo.toJson(
			{
				tags: args.tags,
				deleteAllowed: args.deleteAllowed
			})
		}, args);
		return this._doExecutePut(url, modArgs);
	},
	requestBuild: function(args) {
		var url = BuildRoot + "requests";
		return this._doExecutePost(url,args);
	},
	
	_doExecuteGet: function(url, args, cacheDuration){
		var currentTime, deferred;
		if (cacheDuration && cache[url]) {
			if ((!cache[url].expires || cache[url].expires > (currentTime = new Date().getTime())) &&
						(!args.refreshTime || cache[url].requested > args.refreshTime)) {
				deferred = cache[url].deferred;
			} else {
				delete cache[url];
			}
		}
		if (!deferred) {
			var xhrArgs = {
				url: url,
				headers: {
					"accept": "text/json"
				},
				handleAs: args.handleAs || "json"
			};
			deferred = jazz.client.xhrGet(xhrArgs);
			if (cacheDuration) {
				cache[url] = { deferred: deferred, requested: (currentTime || new Date().getTime()) };
				deferred.addCallback(function(response) {
					if (cache[url]) {
						cache[url].expires = new Date().getTime() + cacheDuration;
					}
				});
				// do not cache failed requests
				deferred.addErrback(function(response) {
					delete cache[url];
				});
			}
		}
		deferred.addErrback(args.errback);
		deferred.addCallback(args.callback);
		return deferred;
	},
	
	_doExecutePost: function(url,args) {
		var xhrArgs = {
			url: url,
			headers: {
				"accept": "text/json",
				"content-type": "text/json;charset=UTF-8"
			},
			handleAs: args.handleAs || "json",
			postData: args.postData
		};
		var deferred = jazz.client.xhrPost(xhrArgs);
		deferred.addErrback(args.errback);
		deferred.addCallback(args.callback);
		return deferred;
	},

	_doExecuteDelete: function(url,args){
		var xhrArgs = {
			url: url,
			headers: {
				"accept": "text/json"
			},
			handleAs: args.handleAs || "json"
		};
		var deferred = jazz.client.xhrDelete(xhrArgs);
		deferred.addErrback(args.errback);
		deferred.addCallback(args.callback);
		return deferred;
	},

	_doExecutePut: function(url,args){
		var xhrArgs = {
			url: url,
			headers: {
				"accept": "text/json",
				"content-type": "text/json"
			},
			handleAs: args.handleAs || "json",
			putData: args.putData
		};
		var deferred = jazz.client.xhrPut(xhrArgs);
		deferred.addErrback(args.errback);
		deferred.addCallback(args.callback);
		return deferred;
	}
};
	
})();
