/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.viewlets.web.client.internal.PlansClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.apt.web.client.util");
dojo.require("jazz.client.xhr");

(function() {

var SCRIPT_DELIMITER		= "\r\n\r\n--a1a1bf4aa1b34a2db01610882541df6b----\r\n\r\n"; //$NON-NLS-1$
var PROVIDED_UNIT_REGEXP	= /dojo\.provide\(\"([^\"]+)\"\)/;

var scriptsLoaded= false;
jazz.client.xhrGet({
    url: (net.jazz.ajax._dataServerRoot || net.jazz.ajax._contextRoot) + "/service/com.ibm.team.apt.internal.service.scriptprovider.IScriptProviderService?scope=dashboardViewlets",
	handleAs: "text", //$NON-NLS-1$
	handle: function(response, ioArgs){
		var scripts= String(response).split(SCRIPT_DELIMITER);
		for (var i= 0; i < scripts.length; i++) {
			var match= PROVIDED_UNIT_REGEXP.exec(scripts[i]);
			if (match && dojo._loadedModules[match[1]]) {
				// do not load scripts twice
				continue;
			}
			eval(scripts[i]);
		}					

		scriptsLoaded= true;
	},
	error: function() {
		// let initializing the plan fail rather than keeping it in an endless wait
		scriptsLoaded= true;
	}
});

	
/*
 * Aliases
 */
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var PlanUtil = com.ibm.team.apt.web.client.util;

/*
 * Constants
 */
var _APT_SERVICE_URI = "com.ibm.team.apt.internal.service.rest.IPlanRestService";
var SERVICE_ID = "com.ibm.team.apt.viewlets.internal.service.IPlansService";

com.ibm.team.apt.viewlets.web.client.internal.PlansClient = {
	
	/* function */ getScriptsLoadedCondition: function() {
		return function() {
			return scriptsLoaded;
		};
	},

	/* void */ getDevelopmentLines: function(/* string */ itemId, /* string */ itemScope, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {itemId: itemId, itemScope: itemScope};
		this._invoke(SERVICE_ID, "getDevelopmentLines", params, srh, false, tracker);
	},

	/* void */ getCurrentIteration: function(/* string */ lineItemId, /* string */ teamItemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		if (!this._iterationsMap)
			this._iterationsMap = new Object();
		var serviceObject = {
			self: this,
			success: function (dto) {
				this.self._iterationsMap[dto.itemId] = dto;
				srh.doCallback(dto);
			},
			failure: function (error) {
				srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		var params = {lineItemId: lineItemId, teamItemId: teamItemId};
		this._invoke(SERVICE_ID, "getCurrentIteration", params, srh2, false, tracker);
	},

	/* void */ getIteration: function(/* string */ itemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		if (!this._iterationsMap)
			this._iterationsMap = new Object();
		if (this._iterationsMap[itemId]) {
			if (this._iterationsMap[itemId] == "fetching") {
				if (!this._iterationsCallbackQueue)
					this._iterationsCallbackQueue = new Object();
				if (!this._iterationsCallbackQueue[itemId])
					this._iterationsCallbackQueue[itemId] = [];
				this._iterationsCallbackQueue[itemId].push(srh);
			} else
				srh.doCallback(this._iterationsMap[itemId]);
			return;
		} else
			this._iterationsMap[itemId] = "fetching";
		var serviceObject = {
			self: this,
			success: function (dto) {
				this.self._iterationsMap[itemId] = dto;
				if (this.self._iterationsCallbackQueue && this.self._iterationsCallbackQueue[itemId]) {
					for (var i = 0; i < this.self._iterationsCallbackQueue[itemId].length; i++)
						this.self._iterationsCallbackQueue[itemId][i].doCallback(dto);
				}
				srh.doCallback(dto);
			},
			failure: function (error) {
				srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._invoke(SERVICE_ID, "getIteration", {itemId: itemId}, srh2, false, tracker);
	},

	/* void */ getDevelopmentLineById: function(/* string */ id, /* string */ projectAreaItemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		this._invoke(SERVICE_ID, "getDevelopmentLineById", {id: id, projectAreaItemId: projectAreaItemId}, srh, false, tracker);
	},

	// expose the APT services with our own client so we can change project area
	/* void */ searchIterationPlans: function(params, srh, /* ServiceTracker */ tracker) {
		this._invoke(_APT_SERVICE_URI, "getIterationPlanSearchResults", params, srh, false, tracker);
	},
	
	/* string */ getProjectAreaByTeam: function(itemId, srh, tracker) {
		if (!this.projectAreasMap)
			this.projectAreasMap = {};
		if (this.projectAreasMap[itemId])
			return this.projectAreasMap[itemId];
		var serviceObject = {
			self: this,
			success: function(result) {
				this.self.projectAreasMap[itemId] = result;
				if (srh)
					srh.doCallback(result);
			},
			failure: function(error) {
				if (srh)
					srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._invoke(SERVICE_ID, "getProjectAreaByTeam", {itemId: itemId}, srh2, false, tracker);
	},

	/*
	 * Invokes a service call.
	 */
	/* void */ _invoke: function(/* string */ serviceId, /* string */ method, /* object */ params, /* ServiceResponseHandler */ srh, /* boolean */ post, /* ServiceTracker */ tracker) {
		var req = new ServiceRequest(serviceId, method, params);
		if (tracker) {
			tracker.register(req, srh);
		}
		TeamServerClient.invokeService(req, srh);

		/*var request = {
		var request = {
			url: URL_PREFIX + method,
			content: params,
			handleAs: "json",
			headers: {
				accept: "text/json"
			},
			handle: function(response, ioArgs) {
				if (response instanceof Error) {
					console.error(response);
					srh.doErrorCallback(response);
				}
				else {
					var returnValue= response["soapenv:Body"].response.returnValue;
					result= null;
					if (returnValue.values) {          // array of EMF objects 
						result= returnValue.values;
					} else if (returnValue.value) {    // single EMF object
						result= returnValue.value;
					} else if (returnValue.dataArgs) { // data query
						result= _demarshallDataQuery(returnValue.dataArgs);
					}
					srh.doCallback(result);
				}
			}
		};
		if (post) {
			jazz.client.xhrPost(request);
		}
		else {
			jazz.client.xhrGet(request);
		}*/
	}
};

})();