/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.internal.ResourcePlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$


(function() { // module container

var PlanningUtil			= com.ibm.team.apt.web.client.util;
var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

var SERVICE_URI				= "com.ibm.team.apt.internal.service.rest.IResourcePlanningRestService"; //$NON-NLS-1$

var Assert					= com.ibm.team.apt.web.ui.internal.common.Assert;

com.ibm.team.apt.web.client.internal.ResourcePlanningClient= {

	// ---- API ------------------------------------------------------------------------------------------------------------	
	
	/**
	 * Asynchronous fetch of all absences of user with given contributorId (UUID)
	 */
	fetchAbsences: function(responseHandler, contributorId) {
		var serviceArguments= {};
		serviceArguments["contributorId"]= contributorId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getAbsences", serviceArguments); //$NON-NLS-1$
	},
	
	fetchWorkEnvironmentInfo: function(responseHandler, contributorId) {
		var serviceArguments= {};
		serviceArguments["contributorId"]= contributorId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getWorkEnvironment", serviceArguments); //$NON-NLS-1$
	},
	
	fetchTimeZoneData: function(responseHandler, contributorId) {
		var serviceArguments= {};

		this._invokePlanService(responseHandler, "getTimeZones", serviceArguments); //$NON-NLS-1$
	},
	
	fetchRegionData: function(responseHandler, contributorId) {
		var serviceArguments= {};

		this._invokePlanService(responseHandler, "getRegions", serviceArguments); //$NON-NLS-1$
	},
	
	postAbsences: function(responseHandler, contributorId, addedAbsences, deletedAbsencesId, modifiedAbsences) {
		var serviceArguments= {};
		serviceArguments["contributorId"]= contributorId; //$NON-NLS-1$
		serviceArguments["deletedAbsencesIds"]= deletedAbsencesId; //$NON-NLS-1$
		serviceArguments["addedAbsences"]= addedAbsences; //$NON-NLS-1$
		serviceArguments["modifiedAbsences"]= modifiedAbsences; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "postAbsences", serviceArguments); //$NON-NLS-1$
	},
	
	postWorkEnvironment: function(responseHandler, contributorId, regionId, timeZoneId, workDays, assignments) {
		var serviceArguments= {};
		serviceArguments["contributorId"]= contributorId; //$NON-NLS-1$
		serviceArguments["regionId"]= regionId ? regionId : "-1"; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["timeZoneId"]= timeZoneId ? timeZoneId : "-1"; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["workDays"]= workDays; //$NON-NLS-1$
		serviceArguments["assignments"]= assignments; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "postWorkEnvironment", serviceArguments); //$NON-NLS-1$
	}, 
		
	// ---- implementation -------------------------------------------------------------------------------------------------

	/**
	 * Invokes the requested method on the APT REST service.
	 */
	_invokePlanService: function(responseHandler, method, serviceArguments) {
		var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(SERVICE_URI, method, serviceArguments);
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, responseHandler);
	},

	__sentinel: null // terminates this class definition
};

})();