/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.workitem.ui.CqWorkItemsViewlet");

dojo.require("com.ibm.rational.cm.web.viewlets.ui.PagedTableViewlet");

dojo.require("jazz.util.format");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.internal.Constants");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.rational.cm.web._base");

(function(){

var messages;
var PREF_WORKITEM_QUERY = "workItemQuery";
var PREF_SHOW_NO_LINKS = "showNoLinks";
var PREF_ROWS = "rowsPerPage";
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var FormatUtil = jazz.util.format;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;
var Constants = com.ibm.team.workitem.web.client.internal.Constants;
var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var IMG_URL_CLOSED_OVERLAY = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/work-items/images/wi_complete_ovr.gif");
var IMG_URL_IN_PROGRESS_OVERLAY = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/work-items/images/wi_in_progress_ovr.gif");

var cq = com.ibm.rational.cm.web;
var cqLinkType = "com.ibm.team.workitem.linktype.relatedChangeManagement";
// use load API since somehow CSS module is not loaded by static dojo["require"]
cq.loadModuleSync("com.ibm.rational.cm.web.ui.view.common.Section");

/*
 * Converts filter object in queryDTO to jsonExpression format that can be passed to getResultSet
 */
var STR_ATTR_EXPRESSIONS = "attributeExpressions", STR_TERM_EXPRESSIONS = "termExpressions";
var STR_OP = "operator", STR_ATTR = "attribute", STR_ATTR_ID = "attributeId";
var STR_VALUES = "values", STR_VARS = "variables";
var getJsonExpression = function(obj){
	var jsonExpression = {};
	for(var key in obj){
		var val = obj[key];
		switch(key){
			case STR_ATTR_EXPRESSIONS:
			case STR_TERM_EXPRESSIONS:
				jsonExpression[key] = dojo.map(val, function(item){
					return getJsonExpression(item);
				});
				break;
			case STR_OP:
				jsonExpression[key] = (dojo.isObject(val)) ? val.id : val;
				break;
			case STR_ATTR:
				jsonExpression[STR_ATTR_ID] = val.id;
				break;
			case STR_VALUES:
				if(dojo.isArray(val) && val.length > 0){
					jsonExpression[key] = dojo.map(val, function(item){
						return item.id;
					});
				}else{
					jsonExpression[key] = [];
				}
				break;
			case STR_VARS:
				if(dojo.isArray(val) && val.length > 0){
					jsonExpression[key] = dojo.map(val, function(item){
						return {
							id: item.id,
							"arguments": item["arguments"]
						};
					})
				}else{
					jsonExpression[key] = [];
				}
				break;
		}
	}
	if(jsonExpression[STR_ATTR_ID]){
		if(!jsonExpression[STR_VALUES]){
			jsonExpression[STR_VALUES] = [];
		}
		if(!jsonExpression[STR_VARS]){
			jsonExpression[STR_VARS] = [];
		}
	}else{
		if(!jsonExpression[STR_ATTR_EXPRESSIONS]){
			jsonExpression[STR_ATTR_EXPRESSIONS] = [];
		}
		if(!jsonExpression[STR_TERM_EXPRESSIONS]){
			jsonExpression[STR_TERM_EXPRESSIONS] = [];
		}
		jsonExpression.similarityExpressions = [];
	}
	return jsonExpression;
}

dojo.declare(
	"com.ibm.rational.cm.web.workitem.ui.CqWorkItemsViewlet",
	com.ibm.rational.cm.web.viewlets.ui.PagedTableViewlet,
{
	contentClass: "com-ibm-rcm-CqWorkItemsViewlet",	
	
	constructor: function(){
		this._preferenceProviders = {};
		this._queryDefs = {};
		this._rowWidgets = {};
	},
	
	postCreate: function(){
		messages = this.messages;
		this.inherited(arguments);
	},

	/*
	 * Override PagedTableViewlet
	 * need to override, otherwise "fetch" won't be called when saving preferences
	 */
	getResultCount: function(){
		var queryItemId = this.getPreference(PREF_WORKITEM_QUERY);
		if(queryItemId == "-1" || !queryItemId || queryItemId.length === 0){
			return -1;
		}else{
			return (this._totalResults >= 0) ? this._totalResults : -1;
		}
	},
	
	/*
	 * Override PagedTableViewlet
	 */
	getItemsPerPage: function(){
		return this.getPreference(PREF_ROWS);
	},

	/*
	 * Override PagedTableViewlet
	 */
	refresh: function(){
		delete this._totalResults;
		this.inherited(arguments);
	},

	/*
	 * Overrides PagedTableViewlet and called from onInit & refresh
	 * Passed ServiceResponseHandler is ignored since we use our own XHR handler
	 * Added doPrompt argument to original method in PagedTableViewlet
	 */
	fetch: function(srh, start, count, entries, details){
		var queryItemId = this.getPreference(PREF_WORKITEM_QUERY);
		this._setLoading(false);
		var site = this.getSite();
		site.setLoading(false);
		if(queryItemId == "-1" || !queryItemId || queryItemId.length === 0){
			this._renderNoResults();
			site.doneRefresh();
		}else{
			var jsonExpression = this._queryDefs[queryItemId];
			if(jsonExpression){ // use cached value
				this.executeQuery(this._generateQueryParams(jsonExpression, start, count), srh, entries, details);
			}else{
				this.getQueryDefinition(queryItemId, srh, start, count, entries, details);
			}
		}
	},
	
	_generateQueryParams: function(jsonExpression, start, count){
		var linkAttrId = "link:"+cqLinkType+":target";
		var cloneObj = dojo.clone(jsonExpression);

		// if showNoLinks preference is false (default), add filter for showing only linked items
		if(!this.getPreference(PREF_SHOW_NO_LINKS)){
			var obj = {};
			obj[STR_ATTR_ID] = linkAttrId;
			obj[STR_OP] = "link.exists";
			obj[STR_VALUES] = [];
			obj[STR_VARS] = [];
			cloneObj[STR_ATTR_EXPRESSIONS].push(obj);
		}
				
		return {
			columnIdentifiers: [
				"workItemType", // without this column, iconUrl won't be returned in resultset response
				"id",
				"summary",
				linkAttrId
			],
			jsonExpression: dojo.toJson(cloneObj),
			maxResults: count,
			startIndex: start
		}
	},
	
	getQueryDefinition: function(queryItemId, srh, start, count, entries, details){
		// Create a reference to this object to use in the service object
		var self = this;

		var site = this.getSite();
		var setLoading = function(/* boolean */ loading){
			self._setLoading(loading);
			site.setLoading(loading);
		}
		
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(dto){
				setLoading(false);
				var jsonExpression = self._queryDefs[queryItemId] = getJsonExpression(dto.expression);
				self.executeQuery(self._generateQueryParams(jsonExpression, start, count), srh, entries, details);
			},
			// Service failure
			failure: function(errorObject) {
				setLoading(false);
				alert("Error getting query definitions!: "+errorObject);
			}
		};
		// Create the handler
		setLoading(true);
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		QueryClient.getEditableQuery(handler, queryItemId);
	},
	
	executeQuery: function(args, srh, entries, details){
		var self = this;
		var site = this.getSite();
		var setLoading = function(/* boolean */ loading){
			self._setLoading(loading);
			site.setLoading(loading);
		}
		var serviceObject = {
			self: this,
			success: function(resultSet){
				setLoading(false);
				if (!this.self.isDestroyed()) {
					if (resultSet && resultSet.totalCount > 0) {
						this.self._resultHeaderIndex = {};
						this.self._resultHeaders = resultSet.headers;
						for (var i = 0; i < resultSet.headers.length; i++) 
							this.self._resultHeaderIndex[resultSet.headers[i].attributeId] = i;
						if (this.self._totalResults != resultSet.totalCount) {
							this.self._totalResults = resultSet.totalCount;
							this.self._totalResultsChanged();
						}else{
this.self.updateTotalPageFooter();
						}
						this.self._token = resultSet.token;
						if (resultSet.rows) {
							var dtos = resultSet.rows;
							var result = new Object();
							if (entries) 
								result.entries = dtos;
							if (details) 
								result.details = dtos;
							srh.doCallback(result);
						}
					}
					else {
						this.self._totalResults = 0;
						this.self._totalResultsChanged();
						srh.doCallback();
					}
				}
			},
			failure: function(error){
				setLoading(false);
				// 404 means query from settings area does not exist
				// 400 means defaultQuery from template does not exist
				if(error.status == 404 || error.status == 400){
					this.self._deletedQuery = true;
					this.self._totalResults = 0;
					this.self.updateTotalPageFooter();
					this.self.getSite().setSecondaryTitle("");
					srh.doCallback();
				}
				else {
					srh.doErrorCallback(error);
				}
			}
		};
		setLoading(true);
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		QueryClient.getResultSet(null, args, handler);
	},

	/*
	 * Overrides PagedTableViewlet
	 * returns an array of TD elements
	 */
	renderEntry: function(/* object */ resultDTO, /* object */ resultDetailsDTO, /* boolean */ zoomed){
		var doc = dojo.doc;
		var columns = [];

		// add twistie for work items with links
		var sectionCell = doc.createElement("td");
		dojo.addClass(sectionCell, "sectionCell");
		var linksContainer;
		var self = this;
		dojo.forEach(resultDTO.info, function(obj){
			if(obj.item){
				dojo.forEach(obj.item, function(linkInfo){
					if(!linksContainer){
						linksContainer = doc.createElement("div");
						if(!self._rowWidgets[resultDTO.id]){
							self._rowWidgets[resultDTO.id] = {};
						}
						var section = self._rowWidgets[resultDTO.id].section = new com.ibm.rational.cm.web.ui.view.common.Section({
							collapsed: true,
							contentNode: linksContainer
						});
						sectionCell.appendChild(section.domNode);
					}
					var linkWrapper = doc.createElement("div");
					linkWrapper.style.paddingTop = "2px";
					linkWrapper.style.paddingBottom = "2px";
					var recordIcon = doc.createElement("img");
					dojo.attr(recordIcon, {
						src: cq.getIconUrl("RECORD"),
						width: 16,
						height: 16,
						style: {
							verticalAlign: "text-bottom"
						}
					});
					linkWrapper.appendChild(recordIcon);
					linkWrapper.appendChild(doc.createTextNode(" "));
					var cqRecordLink = doc.createElement("a");
					cqRecordLink.href = cq.convertResourceURLtoPresentationURL("RECORD", linkInfo.url);
					cqRecordLink.innerHTML = linkInfo.comment;
					linkWrapper.appendChild(cqRecordLink);
					linksContainer.appendChild(linkWrapper);
				});
			}
		});
		columns.push(sectionCell);
		
		var titleColSpan = 2;
		
		var href = WorkItemUtil.getViewWorkItemUri(resultDTO.id);
		
		var iconCell = doc.createElement("td");
		var iconContainer = doc.createElement("div");
		iconContainer.className = "iconContainer";
		
		iconCell.className = "iconCell";
		var icon = doc.createElement("img");
		dojo.attr(icon, {
			src: resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.TYPE]],
			width: 16,
			height: 16,
			style: {
				marginRight: "0.2em"
			}
		});
		iconContainer.appendChild(icon);
		
		// add overlay if available/needed
		if (resultDTO.stateGroup) {
			var src;
			switch(resultDTO.stateGroup){
				case Constants.CLOSED_STATES:	
					src = IMG_URL_CLOSED_OVERLAY;
					dojo.addClass(icon, "closed");
					break;
				case Constants.IN_PROGRESS_STATES:	
					src = IMG_URL_IN_PROGRESS_OVERLAY;
					break;
			}
			if(src){
				var overlayIcon = doc.createElement("img");
				overlayIcon.className = "overlayIcon";
				overlayIcon.src = src;
				iconContainer.appendChild(overlayIcon);
			}
		}
		iconCell.appendChild(iconContainer);
		columns.push(iconCell);
/*		
		if (this.getPreference("secondIcon") != "none") {
			titleColSpan = 1;
			var secondIconCell = doc.createElement("td");
			secondIconCell.className = "iconCell";
			var secondIcon = doc.createElement("img");
			secondIcon.width = 16;
			secondIcon.height = 16;
			secondIcon.style.marginRight = "0.2em";
			console.debug(resultDTO);
			if (this.getPreference("secondIcon") == "priority"){
				var secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.PRIORITY]];
				var secondIconName = dojo.string.substitute(messages.priority,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.PRIORITY]]]);
			}
			else if (this.getPreference("secondIcon") == "severity") {
				var secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.SEVERITY]];
				var secondIconName = dojo.string.substitute(messages.severity,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.SEVERITY]]]);
			}
			else if (this.getPreference("secondIcon") == "status") {
				var secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.STATE]];
				var secondIconName = dojo.string.substitute(messages.status,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.STATE]]]);
			}
			else if (this.getPreference("secondIcon") == "resolution") {
				var secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.RESOLUTION]];
				var secondIconName = dojo.string.substitute(messages.resolution,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.RESOLUTION]]]);
			}
			if (secondIconSrc) {
				secondIcon.src = secondIconSrc;
				secondIcon.title = secondIconName;
				secondIconCell.appendChild(secondIcon);
			}
			columns.push(secondIconCell);
		}
*/		
		var titleCell = doc.createElement("td");
		titleCell.className = "titleCell";
		titleCell.colSpan = titleColSpan;
		var titleWrapper = doc.createElement("div");
		titleWrapper.className = "titleWrapper";
		var titleLink = doc.createElement("a");
		titleLink.href = href;
		titleLink.innerHTML = resultDTO.id + ": " + FormatUtil.insertWordBreaks(resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.SUMMARY]], 20, true);
/*
		new jazz.ui.HoverPopup({around: titleLink, createContents: dojo.hitch(this,function(hp) {
			var node = doc.createElement("div");
			node.className = "com-ibm-team-workitem-viewlets-web-ui-internal-WorkItemsViewlet com-ibm-team-workitem-viewlets-web-ui-internal-WorkItemsViewletTooltip";
			node.appendChild(this._getDetailsWidget(resultDTO).domNode)
			return node;
		}), isEnabled: dojo.hitch(this,function(hp) {
			return !this.isZoomed();
		})});
		*/
		titleWrapper.appendChild(titleLink);
		titleCell.appendChild(titleWrapper);

		// add links if links exist
		if(linksContainer){
			titleCell.appendChild(linksContainer);
		}

		columns.push(titleCell);
		
		return columns;
	},
	
	renderDetails: function(/* object */ detailsData){
		var td = dojo.doc.createElement("td");
		td.colSpan = 4;
		return [ td ];
	},
	
	clearRows: function(){
		for(var id in this._rowWidgets){
			this._rowWidgets[id].section.destroy();
		}
	},
	
	/*
	 * Override PagedTableViewlet
	 * show different messages depending on conditions
	 */
	getNoResultsMessage: function(){
		var queryItemId = this.getPreference(PREF_WORKITEM_QUERY);
		if(queryItemId == "-1" || !queryItemId || queryItemId.length === 0){ // no query specified
			return this.getNoQueryMessage();
		}else{
			return this.getNoHitsMessage();
		}
	},
	
	/*
	 * Override
	 */
	getNoQueryMessage: function(){
		if(!this._noQueryMsg){
			this._noQueryMsg = this.messages.NoWorkItemQuery; // override
		}
		return this.inherited(arguments);
	},
	
	getPreferenceProvider: function(/* string */ preferenceId){
		if(!this._preferenceProviders[preferenceId]){
			if(preferenceId == PREF_WORKITEM_QUERY){
				dojo["require"]("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");
				this._preferenceProviders[preferenceId] = new com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider({ prefId: preferenceId });
			}
		}
		return this._preferenceProviders[preferenceId];
	},
	
	/*
	 * This will get called after preference has been saved (not preference value changes)
	 */
	settingsChanged: function(oldSettings, newSettings){
		var updateTitle = oldSettings.userTitle == true && newSettings.userTitle == false;
		var site = this.getSite();
		if(oldSettings.preferences[PREF_WORKITEM_QUERY] != newSettings.preferences[PREF_WORKITEM_QUERY]){
			this.reset(true);
			updateTitle = true;
		}else if(oldSettings.preferences[PREF_SHOW_NO_LINKS] != newSettings.preferences[PREF_SHOW_NO_LINKS]){
			this.reset(true);
			site.refresh();
		}else if(oldSettings.preferences[PREF_ROWS] != newSettings.preferences[PREF_ROWS]){
			this.reset(false);
		}
		// getTextLabel is not CustomPreferenceProvider API
		// but is provided by the WorkItemQueryPreferenceProvider
		if(updateTitle){
			this.getSite().setTitle(this.getPreferenceProvider(PREF_WORKITEM_QUERY).getTextLabel(newSettings.preferences[PREF_WORKITEM_QUERY]), true);
		}
	}
});

})();