/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.scm.web.ui.internal.AdminViewer");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.reports.scm.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.reports.scm.web.ui.internal", "ReportMessages");

(function() { // module container

var ReportsClient = com.ibm.team.reports.scm.web.client.internal.ReportsClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

dojo.declare("com.ibm.team.reports.scm.web.ui.internal.AdminViewer", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.scm.web", "ui/internal/templates/AdminViewer.html"),
						
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.scm.web.ui.internal", "ReportMessages");
			this.streams = null;
		},
		
		postCreate: function() {
			com.ibm.team.reports.scm.web.ui.internal.AdminViewer.superclass.postCreate.apply(this);
			this.connect(this.saveChanges, "onsubmit", this._saveChangesHandler);
			this.connect(this.cancelChanges, "onsubmit", this._cancelChangesHandler);
			this._getStreams();
		},
		
		_saveChangesHandler: function(e) {
			e.preventDefault();
			var args = { streamItemId: [], streamName: [], streamEnabled: [] };
			if (this.streams != null) {
				for (var i = 0; i < this.streams.length; i++) {
					var item = this.streams[i];
					if (item != null) {
						var id = item.id;
						var input = document.getElementById(id);
						if (input == null) {
							continue;
						}
						if (input.checked != item.enabled) {
							args.streamItemId.push(id);
							args.streamName.push(item.name);
							args.streamEnabled.push(input.checked);
						}
					}
				}
			}
			if (args.streamItemId.length > 0) {
				var handler = new ServiceResponseHandler(
						this,
						"_handleSaveStreams",
						"_remoteError");
				this._working();
				ReportsClient.postSaveStreams(handler, args);
			}
		},
		
		_cancelChangesHandler: function(e) {
			e.preventDefault();
			this._loadStreams();
		},
		
		_getStreams : function() {
			var handler = new ServiceResponseHandler(
				this,
				"_handleGetStreams",
				"_remoteError");
			this._working();
			ReportsClient.getStreams(handler);
		},
		
		_handleGetStreams : function(result) {
			this.streams = result;
			this._loadStreams();
			this._done();
		},
		
		_loadStreams : function() {
			var parent = this.streamsDiv;
			this._removeChildren(parent);
			if (this.streams != null) {
				for (var i = 0; i < this.streams.length; i++) {
					var item = this.streams[i];
					if (item != null) {
						var input = document.createElement("input");
						input.type = "checkbox";
						input.id = item.id;
						input.checked = item.enabled;
						parent.appendChild(input);
						var text = document.createTextNode(" " + item.name);
						parent.appendChild(text);
						var br = document.createElement("br");
						parent.appendChild(br);
					}
				}
			}
		},
		
		_handleSaveStreams : function(result) {
			if ((result != null) && (result.length > 0)) {
				var error = [];
				error.push(this.messages.streamsFailedToBeSaved);
				error.push("\n");
				for (var i = 0; i < result.length; i++) {
					var item = result[i];
					if (item != null) {
						error.push(item.name);
						error.push(": ");
						error.push(item.errorMessage);
						error.push("\n");
					}
				}
				alert(error.join(""));
			}
			this._getStreams();
		},
		
		_removeChildren: function(element) {
			while (element.firstChild) {
			    element.removeChild(element.firstChild);
			}
		},
		
		_working: function() {
			this._displayStatus(this.messages.requestSubmitted, this.messages.waitingForResponse);
		},
		
		_noop: function() {
		},
		
		_done: function() {
			this._displayStatus(this.messages.responseReceived, this.messages.lastRequestSucceeded);
		},
		
		_remoteError: function() {
			this._displayStatus(this.messages.errorInResponse, this.messages.lastRequestFailed);
		},
		
		_displayStatus: function(line1, line2) {
			var text1 = document.createTextNode(line1);
			var br = document.createElement("br");
			var text2 = document.createTextNode(line2);
			this._removeChildren(this.statusDiv);
			this.statusDiv.appendChild(text1);
			this.statusDiv.appendChild(br);
			this.statusDiv.appendChild(text2);
		}
	}
);

})();