/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.BuildQueryColumn");

dojo.require("com.ibm.team.build.web.ui.internal.utils.ResultUtil");
dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.require("jazz.util.date");
dojo.require("jazz.util.html");
dojo.require("jazz.app.ResourceLink");

dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {
	
var ResultUtil = com.ibm.team.build.web.ui.internal.utils.ResultUtil;
var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryColumn", null,
{      
	getCellElement:function(result){
		var td = document.createElement("td");
		td.className = this.getClassName(result);
		td.innerHTML = this.getInnerHtml(result);
		return td;
	},
	
	getInnerHtml: function(result){
		return "";
	},
	
	getClassName: function(result){
		return "";
	}
    
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryIconColumn", com.ibm.team.build.web.ui.internal.BuildQueryColumn, {
	getCellElement: function(result) {
		var td = document.createElement("td");
		td.className = "iconCell";
		var div = document.createElement("div");
		div.className = this.getClassName(result);
		dojo.addClass(div, "statusIcon");
		td.appendChild(div);
		return td;
	}
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryStatusColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryIconColumn,
{
	getClassName: function(result) {
        if (result.buildResult.buildState == "COMPLETED" || result.buildResult.buildState == "IN_PROGRESS" ) {
	        switch (result.buildResult.buildStatus) {
	            case "ERROR":
	                return "error-status"; //$NON-NLS-1$
	            case "WARNING":
	                return "warning-status"; //$NON-NLS-1$
	            default:
	                return "ok-status"; //$NON-NLS-1$
	        }
        }
		return ""; //$NON-NLS-1$
	}  
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryStateColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryIconColumn,
{        
	getClassName: function(result){
		switch(result.buildResult.buildState) {
			case "IN_PROGRESS":
				return result.buildResult.personalBuild ? "inProgressPersonal" : "running";
			case "COMPLETED":
				return result.buildResult.personalBuild ? "completedPersonal" : "completed";
			case "INCOMPLETE":
				return result.buildResult.personalBuild ? "incompletePersonal" : "incomplete";
			case "CANCELED":
				return result.buildResult.personalBuild ? "cancelledPersonal" : "cancelled";
			case "NOT_STARTED":
				return result.buildResult.personalBuild ? "pendingPersonal" : "pending";	
		}	
		return "incomplete-state";
	}   
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryLabelColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{   
	includeDefId: false,
	
	constructor: function(args) {
		if (args && args.includeDefId)
			this.includeDefId = args.includeDefId;
	},
	
	getCellElement:function(result){
		var td = document.createElement("td");
		td.className = this.getClassName(result);
		var label = ResultUtil.getLabel(result, this.includeDefId, true);
		var uri = getUrlRoot() + net.jazz.ajax._contextRoot + "/resource/itemOid/com.ibm.team.build.BuildResult/" + result.buildResult.itemId;
		var presUri = BuildUtil.getResultHref(result.buildResult.itemId);
		var rl = new jazz.app.ResourceLink({uri: uri, presentationUri: presUri, label: label, retainLinkText: true});
		td.appendChild(rl.domNode);
		return td;
	}
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryProgressColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{     

	getInnerHtml: function(result){
		var progressStr = this._getStateText(result);
		return progressStr;
	},
	
	_getStateText: function(input) {
		switch(input.buildResult.buildState) {
			case "IN_PROGRESS": 
				if(input.percentComplete != -1){
					var progress = dojo.string.substitute(messages.percentComplete,[ input.percentComplete.toString()]);
					if(input.currentBuildActivities && input.currentBuildActivities.length > 0) {
						if (input.currentBuildActivities.length == 1)
							progress = dojo.string.substitute(messages.progressAndActivity,[ progress, jazz.util.html.escape(input.currentBuildActivities[0].label)]);
						else
							progress = dojo.string.substitute(messages.progressAndActivity,[ progress, dojo.string.substitute(messages.multipleCurrentActivities,[input.currentBuildActivities.length])]);						
					}
					return progress;
				}
				return messages.unknown;
			case "COMPLETED":
				return messages.completed
			case "INCOMPLETE":
				return messages.abandoned
			case "CANCELED":
				return messages.canceled
			case "NOT_STARTED":
				return messages.pending
		}
		return messages.unknown;
	}
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryEstimatedTimeColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{       

	getInnerHtml: function(result){
		if(result.buildResult.buildState == "IN_PROGRESS"){		
			return this._getEstimatedTime(result);			
		}
		return "&nbsp;";
	},
	
	_getEstimatedTime: function(result){
		// use a space because empty cells don't get borders in IE
		var estimated = "&nbsp;";
		if(result.buildAverageData){
			var avTime = result.buildAverageData.averageBuildTimeTaken 
			if(!isNaN(avTime)){
				var start = result.buildResult.buildStartTime;
				if(start && !isNaN(start)){
					var startDate = new Date(start);
					var avEndTime = dojo.date.add(startDate, "millisecond", avTime);
					estimated = jazz.util.date.formatDif(avEndTime, false, true);
					if(dojo.date.compare(avEndTime) < 0)
						estimated = dojo.string.substitute(messages.overdue,[ estimated]);			
				}
			}
		}
		return estimated;
	}   
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryStartTimeColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{       

	getInnerHtml: function(result){
		// use a space because empty cells don't get borders in IE
		var startTime = "&nbsp;";
		if(result.buildResult.buildState != "NOT_STARTED" && result.buildResult.buildState != "CANCELED"){
			var start = result.buildResult.buildStartTime;
			startTime = new Date(start);
			startTime = dojo.date.locale.format(startTime, {formatLength: "medium" }); //$NON-NLS-1$
		}
		return startTime;
	} 
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryTimeInQueueColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{
	getInnerHtml: function(result){
		// use a space because empty cells don't get borders in IE
		var timeInQueue = "&nbsp;";
		if(result.buildResult.buildState == "NOT_STARTED"){
			var timeCreate=result.buildRequests[0].created;
			timeCreate=dojo.date.stamp.fromISOString(timeCreate);
			if(timeCreate){ 
				timeInQueue=jazz.util.date.formatDuration(new Date()-timeCreate);
			}
			
		}
		return timeInQueue;
	}
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryDurationColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{        

	getInnerHtml: function(result){
		// use a space because empty cells don't get borders in IE
		var duration = "&nbsp;";
		if(result.buildResult.buildState != "NOT_STARTED" && result.buildResult.buildState !="CANCELED"){
			duration = ResultUtil.getDuration(result);
		}		
		return duration;
	}
	 
});

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueryTagsColumn",
 com.ibm.team.build.web.ui.internal.BuildQueryColumn,
{        
	getInnerHtml: function(result){
		// if no tags, return a space because empty cells don't get borders in IE
		return jazz.util.html.escape(result.buildResult.tags) || "&nbsp;";
	}
});

var urlRoot;
var getUrlRoot = function() {
	if (!urlRoot) {
		var winUrl = new dojo._Url(window.location.href);
		urlRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" );
	}
	return urlRoot;
}

})();

