/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.DefinitionPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.Table");
dojo.require("com.ibm.team.build.web.ui.internal.BuildQueryColumn");
dojo.require("com.ibm.team.build.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function(){
	
var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var Table = com.ibm.team.build.web.ui.internal.utils.Table;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var BuildQueryStatusColumn = com.ibm.team.build.web.ui.internal.BuildQueryStatusColumn;
var BuildQueryStateColumn = com.ibm.team.build.web.ui.internal.BuildQueryStateColumn;
var BuildQueryLabelColumn = com.ibm.team.build.web.ui.internal.BuildQueryLabelColumn;
var BuildQueryProgressColumn = com.ibm.team.build.web.ui.internal.BuildQueryProgressColumn;
var BuildQueryEstimatedTimeColumn = com.ibm.team.build.web.ui.internal.BuildQueryEstimatedTimeColumn;
var BuildQueryStartTimeColumn = com.ibm.team.build.web.ui.internal.BuildQueryStartTimeColumn;
var BuildQueryDurationColumn = com.ibm.team.build.web.ui.internal.BuildQueryDurationColumn;
var BuildQueryTagsColumn = com.ibm.team.build.web.ui.internal.BuildQueryTagsColumn;
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
var TABLE_SPECS = {
		headings: [{label: "", width:"1%"},
				   {label: "", width:"1%"},
				   {label: messages.labelLabel, width:"28%"},
				   {label: messages.labelProgress, width:"15%"},
				   {label: messages.labelTags,width:"10%"},
				   {label: messages.labelStart, width:"22%"},
				   {label: messages.labelDuration, width:"22%"},
				   {label: messages.labelEstimated, width:"1%"}
				  ],
		style: com.ibm.team.build.web.ui.internal.utils.table.Styles.GOLD
	};
var CHUNK_SIZE = 15;

dojo.declare("com.ibm.team.build.web.ui.internal.DefinitionPage",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/ListPage.html"),
	
	widgetsInTemplate: true,
	
	postCreate: function() {
		dojo.addClass(this.domNode, "BuildQuery");
		this.populateHeader();
		this._doQuery();
	},
	
	refresh: function(e) {
		if (e)
			e.preventDefault();
		PlatformUI.getWorkbench().showMessage(messages.loadingDef);
		this._refreshTime = new Date().getTime();
		this._doQuery();
		this.updateTitle();
	},
	
	populateHeader: function() {
		this.metanav.populateBreadcrumbs();
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.requestBuild), {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/rquebuild.gif"),
			label: messages.request
		});
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.refresh), {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/refresh.gif"),
			label: messages.refresh
		});
		this.updateTitle();
	},
	updateTitle: function() {
		var callback = dojo.hitch(this,this._handleBuildDefCallback);
		var args = {definitionItemId: this.definitionId, callback: callback, errback: callback};
		if (this._refreshTime)
			args.refreshTime = this._refreshTime;
		BuildClient.getDefinition(args);
	},
	requestBuild: function() {
		location.href = BuildUtil.getRequestBuildHref(this.definitionId);
	},
	_handleBuildDefCallback: function(response){
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			ErrorUtil.handleErrorResponse(response, messages.definitionNotFound);
		}
		else {
			this.definition = response;
			this.metanav.setTitle(dojo.string.substitute(messages.buildsHeading,[jazz.util.html.escape(response.id)]));
			PlatformUI.getWorkbench().setTitle(dojo.string.substitute(messages.buildsTitle,[ response.id]));
			this._flushPendingRequests();
		}
	},
	
	_doQuery: function() {
		var callback = dojo.hitch(this,this._handleBuildResultsCallback);
		var args = {definitionItemId: this.definitionId, callback: callback, errback: callback};
		if (this._refreshTime)
			args.refreshTime = this._refreshTime;
		BuildClient.getResults(args);
	},
	
	_handleBuildResultsCallback: function(response){
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			PlatformUI.getWorkbench().hideMessageArea();
			ErrorUtil.handleErrorResponse(response);
		}
		else {
			response.sort(function(a,b) {
				return b.buildStartTime - a.buildStartTime;
			});
			this.content.innerHTML = "";

			if (!response || !response.length) {
				var div = document.createElement("div");
				div.className = "row none-text";
				div.innerHTML = messages.noResultsFound;
				this.content.appendChild(div);
				PlatformUI.getWorkbench().hideMessageArea();
			}
			else {
				var table = new Table(TABLE_SPECS);
				this._table = table;
				if(!this._columns){ //Create column objects only once
					this._columns = [ new BuildQueryStatusColumn(),
						 			  new BuildQueryStateColumn(), 
									  new BuildQueryLabelColumn(), 
									  new BuildQueryProgressColumn(), 
									  new BuildQueryTagsColumn(),
									  new BuildQueryStartTimeColumn(),
									  new BuildQueryDurationColumn(), 
									  new BuildQueryEstimatedTimeColumn()
									];
				}
				
				this._fetchChunk(response, 0, table);
			}
		}
	},
	
	_fetchChunk: function(results, startIndex, table) {
		var resultsToFetch = results.slice(startIndex, startIndex + CHUNK_SIZE);
		if (resultsToFetch.length) {
			var uuids = [];
			for (var j = 0; j < resultsToFetch.length; ++j) {
				uuids.push(resultsToFetch[j].itemId);
			} 
			var callback = dojo.hitch(this, 
				function(response) { 
					this._handleBuildResultRecordsCallback(response, results, startIndex, table); 
				}
			);
			var args = {uuids: uuids, callback: callback, errback: callback};
			if (this._refreshTime)
				args.refreshTime = this._refreshTime;
			BuildClient.getResultRecords(args);
		} else {
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},
	
	_handleBuildResultRecordsCallback: function(response, results, startIndex, table){
		// check whether response is stale due to widget disposal or a refresh
		if (!this.domNode || this._table != table || !response)
			return;
		if (response instanceof Error) {
			PlatformUI.getWorkbench().hideMessageArea();
			ErrorUtil.handleErrorResponse(response);
		} else if (!response.length) {
			// not an error response, but something went wrong if length is 0, ignore the call
			return;
		} else {
			for (var i = 0; i < response.length; i++) {
				this._createResultTable(table.tbody, response[i]);
			}
			if (startIndex == 0) {
				// only append the table when the first chunk comes back, 
				// to avoid an empty table and to reduce jumpiness
				this.content.appendChild(table.domNode);
			}
			
			this._fetchChunk(results, startIndex + CHUNK_SIZE, table);
		}
	},
		
	/* 
	 * 1) When there are no results for the given definition, the NoResults message will not be shown within a table, since 
	 * the table headings don't have much value in this case.
	 * 2) For a build in-progress,
	 * Expectation: "{some percentage}%" or "{some percentage}% ({current activity})" for progress column if there is 
	 * a current activity, and an accurate value for duration column.  The Estimated Completion column is filled in with
	 * a relative time for when it estimates the build to complete, based on how far along it is in the build, and the 
	 * average time collected for past builds.  To see this in action, try the Build Queue drop-down of the Builds view
	 * on an active server like jazzdev. In-progress build can be abandoned.
	 * 3) For a pending build (one that hasn't yet started),
	 * Expectation: the Progress column should show 'Pending'; the status column should show the pending icon , the other 
	 * columns should be blank. Pending build can be cancelled.
	 * 4) For a cancelled build,  
	 * Expectation: the Progress column should show 'Cancelled'; the status column should show the cancelled icon , the other 
	 * columns should be blank.
	 * 5) For the start time column, we want to ensure that it shows times relative to the user's locale and time zone.  
	 * That is, if you requests a build on jazzdev at 9am Beijing time, and the build starts immediately, the start time 
	 * should show 9am, not 9pm (Eastern time). 
	 */		
	_createResultTable: function(parent, result){
		var resultRow = document.createElement("tr");
		for (var i = 0; i < this._columns.length; i++) {
		  resultRow.appendChild(this._columns[i].getCellElement(result));
		}
		parent.appendChild(resultRow);
	},
	
	getArtifactInfo: function(srh) {
		if (!this.definition) {
			if (!this._pendingInfoRequests)
				this._pendingInfoRequests = [];
			this._pendingInfoRequests.push(srh);
		}
		else
			srh.doCallback(this._getInfo());
	},
	
	_flushPendingRequests: function() {
		if (this._pendingInfoRequests) {
			var info = this._getInfo();
			for (var i = 0; i < this._pendingInfoRequests.length; i++)
				this._pendingInfoRequests[i].doCallback(info);
		}
		this._pendingInfoRequests = null;
	},
	
	_getInfo: function() {
		if (!this.definition)
			return null;
		return {
			itemId: this.definitionId,
			title: this.definition.id,
			iconUrl: dojo.moduleUrl("com.ibm.team.build.web", "graphics/common/icons/build_type_obj.gif"),
			href: BuildUtil.getDefinitionHref(this.definitionId)
		};
		
	}

});
	
})();
