/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.LeftNavSearch");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("jazz.ui.SearchBox");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases
 */
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var SearchBox = jazz.ui.SearchBox;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var SCOPE_ALL = "all";
var SCOPE_TEAMS = "teams";
var SCOPE_SHARED = "contributors";

var _messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

/*
 * The quick search section at the top of the left nav.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.LeftNavSearch",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/LeftNavSearch.html"),

	constructor: function(/* object */ args) {
		this._dashboardPage = args.dashboardPage;
	},
	
	postCreate: function() {
		var searchBox = new SearchBox({
			scopes: [
				{id: SCOPE_ALL, label: _messages.leftSearchScopeAll, icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/search-all.gif")},
				{id: SCOPE_TEAMS, label: _messages.leftSearchScopeAllTeams, icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/search-team.gif")},
				{id: SCOPE_SHARED, label: _messages.leftSearchScopeAllShared, icon: dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/leftnav/icons/search-contributor.gif")}
			],
			onUpdate: dojo.hitch(this, this._onUpdate),
			tooltip: _messages.leftSearchTooltip,
			defaultText: _messages.leftSearchDefaultText,
			noResultsLabel: _messages.leftSearchNoResultsLabel
		});
		this.domNode.insertBefore(searchBox.domNode, this.domNode.firstChild);
	},
	
	_onUpdate: function(/* string */ query, /* string */ scope, /* function */ callback, /* function */ errorCallback) {
		// call search service
		var serviceObj = {
			_self: this,
			_query: query,
			_callback: callback,
			_errorCallback: errorCallback,
			success: function(/* ResponseDTO */ responseDTO){
				if (responseDTO.status == "OK") {
					var results = this._self._createResults(responseDTO.data, this._query);
					this._callback(results);
				}
				else {
					this._errorCallback(responseDTO.msg);
				}
			},
			failure: function(/* Error */ error) {
				this._errorCallback(error.message);
			}
		};
		var projectAreaItemId = getCurrentProjectArea().itemId;
		var includeTeams = (scope != SCOPE_SHARED);
		var includeContributors = (scope != SCOPE_TEAMS);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.search(query, projectAreaItemId, includeTeams, includeContributors, srh);
	},
	
	/* array */ _createResults: function(/* SearchResultsDTO */ resultsDTO, /* string */ query) {
		var results = [];
		var teams = resultsDTO.teams;
		var shared = resultsDTO.shared;
		if ((teams && teams.length > 0) || (shared && shared.length > 0)) {
			if (teams) {
				for (var i = 0; i < teams.length; i++) {
					results.push(this._createResult(teams[i], DashboardConstants.SCOPE_TEAM_AREA, query));
				}
			}
			if (shared) {
				var usersGroup;
				if (teams && teams.length > 0) {
					usersGroup = {label: dojo.string.substitute(_messages.leftSearchUsers, [ shared.length ]), children: []};
					results.push(usersGroup);
				}
				for (var i=0;i<shared.length;i++) {
					var result = this._createResult(shared[i], DashboardConstants.SCOPE_CONTRIBUTOR, query);
					if (usersGroup) {
						usersGroup.children.push(result);
					}
					else {
						results.push(result);
					}
				}
			}
		}
		return results;
	},

	/* object */ _createResult: function(/* SearchResultDTO */ resultDTO, /* string */ scope, /* string */ query) {
		var handler = {
			_self: this,
			_resultDTO: resultDTO,
			_scope: scope,
			handle: function() {
				if (this._self._dashboardPage.navigateAwayEvent(false)) {
					location.href = DashboardUtil.getDashboardHref(this._scope, eval(this._resultDTO.id));
				}
			}
		};
		var label = this._formatLabel(resultDTO.name, query);
		var result = { label: label, handler: dojo.hitch(handler, handler.handle)};
		if (resultDTO.children) {
			var children = [];
			for (var i=0;i<resultDTO.children.length;i++) {
				children.push(this._createResult(resultDTO.children[i], scope, query));
			}
			result.children = children;
		}
		return result;
	},
	
	_formatLabel: function(/* string */ label, /* string */ query) {
		var index = label.toLowerCase().indexOf(query.toLowerCase());
		if (index != -1) {
			return label.substring(0, index) + "<b>" + label.substring(index, index + query.length) + "</b>" + label.substring(index + query.length);
		}
		else {
			return "<span class=\"com-ibm-team-dashboard-web-ui-internal-LeftNavSearch-groupItem\">" + label + "</span>"
		}
		return label;
	}
});

})();
