/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit.form.Button");
dojo.require("dijit.Editor");

dojo.require("dijit._editor.plugins.EnterKeyHandling");
dojo.require("dijit._editor.plugins.TextColor");
dojo.require("dijit._editor.plugins.LinkDialog");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");

dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StaticMessages");

(function() {

/*
 * Aliases 
 */
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

/*
 * TODOs
 */
// TODO: MP: Enhancement: Allow editor and text editor to be zoomable
// TODO: MP: Bug: IE6 Resize handle area looks bad
// TODO: MP: Bug: IE6 Absolute overlay height 100% does not work - need fixed height

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/StaticViewlet.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {},

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return StaticViewlet
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StaticMessages");
		
	},
		
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	init: function() {
		
		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();
		
		// Constants //------------------------------------------------------------>
		
		this._STATE_HTML = "html";
		this._INITIAL_HTML = 
			"<div style=\"margin:5px\">\n   " +
			this.messages.msgDefineContentHere +
			"\n</div>";
		
		// Instance Variables //--------------------------------------------------->

		this._viewletID = this._static._instanceCount;

		// Widget variables
		this._buttonRichText = null;
		this._buttonHTML = null;
		this._editor = null;
		this._editorIFrame = null;
		
		// Resize variables
		this._resizeActionInProgress = false;
		this._dragStartYPosition = 0;
		this._resizeMouseUpHandle = null;
		this._resizeMouseMoveHandle = null;
		this._resizeMouseOutHandle = null;
		
		// Status variables
		this._viewletMode = this._static._MODE_DISPLAY;
		
		// Load the display HTML
		this._loadDisplayArea();
		// Add double-click listener
		// Disabled 
		// this._createListenerDoubleClick();	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.StaticViewlet";
		
		this._static._MINIMUM_EDITOR_HEIGHT = 150; // pixels
		
		this._static._MODE_DISPLAY = 0;
		this._static._MODE_RICHTEXT = 1;
		this._static._MODE_HTML = 2;
		this._static._MODE_MESSAGE = 3;
		
		this._static._RICH_TEXT_EDITOR_ID = "viewlet_rich-text-editor_";
		
	},	
	
	_doEdit: function(event) {
		dojo.stopEvent(event);
		this.getSite().editContents();
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
	 * @overrides super
 	 */		
	validateEditedContent: function() {
		// Get the edited contents
		var html = new String(this._textArea.value);
		// Check for insecure content
		if (jazz.util.html.hasScript(html)) {
			alert(this.messages.errorScript);
			return false;
		}
		return true;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides super
 	 */		
	preSaveState: function() {
		// Save button was pressed in edit mode
		// Update the display HTML using the contents of the editor
		this._updateDisplayHTML();
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides super
 	 */		
	update: function() {
		// Determine whether we are entering or leaving edit mode
		if (this.isEditing()) {
			// Enter
			// Load the editor area
			this._loadEditorArea();
		} else {
			// Leave
			// Load the display HTML
			this._loadDisplayArea();
		}
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_loadEditorArea: function() {
		if (this._buttonRichText == null) {
			// Create the edit area tabs	
			this._createEditAreaTabs();			
		}	
		// Add content to editor area
		this._updateEditorAreaContent();	

		// Show the editor area
		this._updateVisibility(this._editorArea, true);
		// Show the editor area tabs
		this._updateVisibility(this._editorAreaTabs, true);
		// Hide the rich text editor area
		this._updateVisibility(this._richTextEditorArea, false);		
		// Hide the display area
		this._updateVisibility(this._displayArea, false);
		// Hide the message area
		this._updateVisibility(this._messageArea, false);
		
		// Set HTML mode
		this._viewletMode = this._static._MODE_HTML;
		// Toggle buttons
		this._buttonRichText.setDisabled(false);
		this._buttonHTML.setDisabled(true);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */          
	_createEditAreaTabs: function() {
		// Create the resize handle
		this._createResizeHandle();
		// Create the rich text tab 
		this._createButtonRichText();
		// Create the html tab	
		this._createButtonHTML();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */          
	_createResizeHandle: function() {
		// Connect listener
		this.connect(this._editorAreaDragHandle, "onmousedown", this._handleEventDragStart);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */          
	_handleEventDragStart: function(event) {
		// Prevent default events and text selection
		event.preventDefault();
		event.cancelBubble = true;
		dojo.stopEvent(event);
		// Update flag
		this._resizeActionInProgress = true;
		this._dragStartYPosition = event.pageY;
		// Test
		// console.debug(event);
		// Connect listeners
		if (this._resizeMouseMoveHandle == null) {
			this._resizeMouseMoveHandle = this.connect(document.body, "onmousemove", this._handleEventMove);
		}
		if (this._resizeMouseUpHandle == null) {
			this._resizeMouseUpHandle = this.connect(document.body, "onmouseup", this._handleEventDragStop);
		}
		if (this._resizeMouseOutHandle == null) {
			this._resizeMouseOutHandle = this.connect(document.body, "onmouseout", this._handleEventMouseOut);
		}
		// Show overlay
		this._updateVisibility(this._richTextEditorOverlay, true);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */          
	_handleEventMove: function(event) {
		// Prevent default events and text selection
		event.preventDefault();
		event.cancelBubble = true;
		dojo.stopEvent(event);
		// Determine if we are dragging
		if (this._resizeActionInProgress == false) {
			return;
		}
		// Test
		// console.debug(event);
		// Get current position
		var dragCurrentYPosition = event.pageY;
		// Calculate the difference between the start and current postions
		var positionDelta = dragCurrentYPosition - this._dragStartYPosition;
		
		// Determine which editor we are operating on
		var editorNode = null;
		if (this._viewletMode == this._static._MODE_HTML) {
			editorNode = this._textArea;
		} else {
			editorNode = this._editorIFrame;
		}
		// Get the current height of the editor
		var currentEditorHeight = editorNode.clientHeight;
		// Test
		// console.debug(currentEditorHeight);
		// console.debug(positionDelta);
		// Add the 
		var newEditorHeight = currentEditorHeight + positionDelta;
		// Enforce minimum height
		if (newEditorHeight < this._static._MINIMUM_EDITOR_HEIGHT) {
			return;
		}
		// Set new height
		editorNode.style.height = newEditorHeight + "px";
		// Reset starting point
		this._dragStartYPosition = dragCurrentYPosition;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */          
	_handleEventMouseOut: function(event) {
		// Prevent default events and text selection
		event.preventDefault();
		event.cancelBubble = true;
		dojo.stopEvent(event);		
		// Stop the drag if the mouse goes outside the document
		if (dojo.isIE) {
			// IE
			if (event.toElement == null) {
				this._handleEventDragStop(event);
			}
		} else {
			// FF
			if (event.relatedTarget == null) {
				this._handleEventDragStop(event);
			}
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */          
	_handleEventDragStop: function(event) {
		// Prevent default events and text selection
		event.preventDefault();
		event.cancelBubble = true;
		dojo.stopEvent(event);
		// Update flag
		this._resizeActionInProgress = false;
		// Test
		// console.debug(event);
		// Disconnect listeners
		if (this._resizeMouseMoveHandle != null) {
			this.disconnect(this._resizeMouseMoveHandle);
			this._resizeMouseMoveHandle = null;
		}
		if (this._resizeMouseUpHandle != null) {
			this.disconnect(this._resizeMouseUpHandle);	
			this._resizeMouseUpHandle = null;
		}
		if (this._resizeMouseOutHandle != null) {
			this.disconnect(this._resizeMouseOutHandle);	
			this._resizeMouseOutHandle = null;
		}
		// Hide overlay
		this._updateVisibility(this._richTextEditorOverlay, false);
	},	
		
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */          
	_createButtonRichText: function() {
		// Create a reference to this object for the call back
		var self = this;
		// Create the button parameters
		var parameters = {
			label: this.messages.msgRichText,
			iconClass: "richtext-icon",
			onClick: function() {
				self._handleEventButtonClickedRichText();
			}
		};
  		// Create the button
		this._buttonRichText = new dijit.form.Button(
			parameters
		);
		// Get the attach point
		var node = this._editorAreaTabs; 
  		// Add the button to the attach point
		node.appendChild(this._buttonRichText.domNode);		
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_handleEventButtonClickedRichText: function() {
		// Toggle buttons
		this._buttonRichText.setDisabled(true);
		this._buttonHTML.setDisabled(false);
		// Lazy load editor
   		if (this._editor == null) {
   			// Create the Editor
   			this._createEditor();		
   		}	   	   
		// Update editor contents   
		this._editor.setValue(this._textArea.value);
		// Hide the editor area
		this._updateVisibility(this._editorArea, false);
		// Show the rich text editor area
		this._updateVisibility(this._richTextEditorArea, true);
		// Hide the message area
		this._updateVisibility(this._messageArea, false);
		
		// Set rich text mode
		this._viewletMode = this._static._MODE_RICHTEXT;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */          
	_createButtonHTML: function() {
		// Create a reference to this object for the call back
		var self = this;
		// Create the button parameters
		var parameters = {
			label: this.messages.msgHTML,
			iconClass: "html-icon",
			onClick: function() {
				self._handleEventButtonClickedHTML();
			}
		};
  		// Create the button
		this._buttonHTML = new dijit.form.Button(
			parameters
		);
		// Get the attach point
		var node = this._editorAreaTabs; 
  		// Add the button to the attach point
		node.appendChild(this._buttonHTML.domNode);		
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_handleEventButtonClickedHTML: function() {
		// Toggle buttons
		this._buttonRichText.setDisabled(false);
		this._buttonHTML.setDisabled(true);
		// Update text area contents
		this._textArea.value = this._editor.getValue(false);
		// Show the editor area
		this._updateVisibility(this._editorArea, true);
		// Hide the rich text editor area
		this._updateVisibility(this._richTextEditorArea, false);
		// Hide the message area
		this._updateVisibility(this._messageArea, false);
		
		// Set HTML mode
		this._viewletMode = this._static._MODE_HTML;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */          
	_createEditor: function() {
         // Get the attach point
         var node = this._richTextEditorArea; 
         // Add the editor to the attach point
         this._editor = new dijit.Editor({
         	id: this._getRichTextEditorID(),
         	height: "150px",
  			plugins: [ 
				'dijit._editor.plugins.EnterKeyHandling',
				'undo',
  				'redo',
				'|',
				'bold',
				'italic',
				'underline',
				'strikethrough',
				'|',
				'foreColor',
				'hiliteColor',
				'|',
				'justifyLeft',
				'justifyRight',
				'justifyCenter',
				'justifyFull',
  				'|',
				'removeFormat',
				'|',
				'insertOrderedList',
				'insertUnorderedList',
				'indent',
				'outdent',
				'|',
				'createLink',
				'unlink',
				'|',
				'insertImage'
			]         	
         }, node);
		 // Get the IFrame node to control the editor height later on
		 this._editorIFrame = this._parseEditorIFrame(node);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return String
 	 */	
	_getRichTextEditorID: function() {
		return this._static._RICH_TEXT_EDITOR_ID + this._viewletID;	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return Element
 	 */          
	_parseEditorIFrame: function(parent) {
		// Child 1: DIV: Toolbar
		// Child 2: DIV: Editor
		// 		Child 2.1: IFRAME 
		// Toolbar
		var firstChild = parent.firstChild;
		if (firstChild == null) {
			return;
		}
		// Editor
		var secondChild = firstChild.nextSibling;
		if (secondChild == null) {
			return;
		}
		// IFrame
		return secondChild.firstChild;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return <none>
 	 */	
	_updateVisibility: function(node, show) {
		if (show == true) {
			// Show
			node.style.display = "block";
		} else {
			// Hide
			node.style.display = "none";
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_updateDisplayHTML: function() {
		var html;
		if (this._viewletMode == this._static._MODE_RICHTEXT) {
			html = this._editor.getValue(false);
		} else {
			html = this._textArea.value;
		}	
		// Save the display HTML
		this.getViewletState().set(this._STATE_HTML, html);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_createListenerDoubleClick: function() {
		var node = this._displayArea;
		// Create a reference to this object to use in the service object
		var self = this;
		// Add the listener
		node.ondblclick = function() {
			self._container.startEditing();
		};
	},

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_loadDisplayArea: function() {
		// Add content to display area
		this._updateDisplayAreaContent();		
		
		// Hide the editor area
		this._updateVisibility(this._editorArea, false);
		// Hide the rich text editor area
		this._updateVisibility(this._richTextEditorArea, false);		
		// Hide the editor area tabs
		this._updateVisibility(this._editorAreaTabs, false);
		
		if (this.isEmpty) {
			// Hide the message area
			this._updateVisibility(this._messageArea, true);
			// Add default content
			this._addActionEditContents();			
			
			this._updateVisibility(this._displayArea, false);
			// Set message mode
			this._viewletMode = this._static._MODE_MESSAGE;
		} else {
			// Show the display area
			this._updateVisibility(this._displayArea, true);
			this._updateVisibility(this._messageArea, false);
			// Set display mode
			this._viewletMode = this._static._MODE_DISPLAY;
		}
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addActionEditContents: function() {

		// Remove previous content (if any)
		this._removeAllChildren(this._messageArea);

		if (this.getSite().isDashboardEditing()) {
			// The user has permission to edit the Dashboard
			// Add action
			var links = I18nUtil.addMessage(this._messageArea, this.messages.msgNoContentDefinedAction);
			var link = links[0];
			link.href = "javascript:void(0);";
			this.connect(link, "onclick", this._doEdit);		

		} else {
			// The user does not have the permission to edit the Dashboard
			// Do not add action
			var fallbackMessage = document.createTextNode(this.messages.msgNoContentDefined);
			this._messageArea.appendChild(fallbackMessage);			
		}
		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_updateDisplayAreaContent: function() {
		// Get the attach point
		var node = this._displayArea;
		// Remove previous content
		this._removeAllChildren(node);
		// Create a wrapping div
		var div = document.createElement("div");
		div.className = "wrapper";
		// Set the content to be the html
		var html = this.getViewletState().get(this._STATE_HTML);
		if (html == null) {
			html = "";
		} else {
			html = new String(html);
		}
		div.innerHTML = html;
		if (html.length > 0) {
			this.isEmpty = false;	
		} else {
			this.isEmpty = true;
		}
		// Add the wrapping div to the node
		node.appendChild(div);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_updateEditorAreaContent: function() {
		// Get the attach point
		var node = this._textArea;
		// Get the saved HTML contents
		var html = this.getViewletState().get(this._STATE_HTML);
		// Declare some initial content if there is no html defined
		if (html == null) {
			html = this._INITIAL_HTML;
		}
		// Load the saved HTML contents into the editor area
		node.value = html;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Node node
 	 * @return <none>
 	 */	
	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.firstChild != null) {
			node.removeChild(node.firstChild);
		}
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @return <none>
 	 * @overrides super
 	 */	
	refresh: function() {
		this.getSite().doneRefresh();
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}	

});


})();

