/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;

dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.DeferredLoadReadStore", dojo.data.ItemFileReadStore, { //$NON-NLS-1$
	constructor: function(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	isItemLoaded: function(item){
		var type = this.getValue(item, "type"); //$NON-NLS-1$
		if (type === ProcessConstants.SERVICE_PROVIDER_NOT_LOADED_TYPE) {
			return false;
		}
		return true;
	},
	
	/**
	 * Overload of ItemFileReadStore#loadItem.
	 * See Read#loadItem for details on parameter etc.
	 */
	loadItem: function(keywordArgs){
		var item = keywordArgs.item;
		this._assertIsItem(item);
		
		var serviceProviderCatalogNode;
		var serviceProviderCatalogId = this.getValue(item, "serviceProviderCatalogId"); //$NON-NLS-1$
		this.fetch({
			query: {
				id: "*" + serviceProviderCatalogId //$NON-NLS-1$
			},
			queryOptions: {
				deep: true
			},
			onItem: function(catalog, request){
				serviceProviderCatalogNode = catalog;
			}
		});
		var serviceProviderResource = this.getValue(serviceProviderCatalogNode, "serviceProviderCatalogURL"); //$NON-NLS-1$
		var deferred = jazz.client.xhrGet({
			url: serviceProviderResource,
			headers: {
				"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: dojo.hitch(this, function(serviceProviderCatalogXML){
				this._populateServiceProviders(serviceProviderCatalogXML, item, serviceProviderCatalogNode, keywordArgs);
			}),
			error: dojo.hitch(this, function(error){
				item.name = [this.messages.DeferredLoadReadStore_Error];
				item.type = ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE;
				//the default error handler dumps to the console
				if (keywordArgs.onError) {
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onError.call(scope, error);
				}
				if (keywordArgs.onItem) {
					scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onItem.call(scope, serviceProviderCatalogNode);
				}
			})
		});
	},
	
	_populateServiceProviders: function(serviceProviderCatalogXml, stubItem, parentServiceProviderCatalogTreeNode, keywordArgs){
		if (!serviceProviderCatalogXml) {
			return;
		}
		var serviceProviders = [];
		
		var children = serviceProviderCatalogXml.childNodes;
		var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
		children = serviceProviderCatalog.childNodes; //children of ServiceProviderCatalog
		var entries = UIHelper.getNodes(children, "entry", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
		var providers = [];
		var providerCatalogs = [];
		dojo.forEach(entries, function(entry){
			var serviceProvider = UIHelper.getNode(entry.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			if (serviceProvider) {
				providers.push(serviceProvider);
			}
			var serviceProviderCatalog = UIHelper.getNode(entry.childNodes, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			if (serviceProviderCatalog) {
				providerCatalogs.push(serviceProviderCatalog);
			}
		});
		
		var firstChild = true;
		dojo.forEach(providers, function(serviceProvider){
			var serviceProviderTreeNode;
			if (firstChild) {
				serviceProviderTreeNode = stubItem;
			} else {
				serviceProviderTreeNode = {};
			}
			UIHelper.configureServiceProviderTreeNode(serviceProviderTreeNode, serviceProvider);
			
			if (!firstChild) {
				parentServiceProviderCatalogTreeNode.children.push(serviceProviderTreeNode);
				this._arrayOfAllItems.push(serviceProviderTreeNode);
			} else {
				firstChild = false;
			}
			
			serviceProviderTreeNode[this._storeRefPropName] = this;
			serviceProviderTreeNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in should be the item
		}, this);
		
		dojo.forEach(providerCatalogs, function(serviceProviderCatalog){
			var serviceProviderCatalogNode = {};
			var stubNode = {};
			UIHelper.configureServiceProviderCatalogTreeNode(serviceProviderCatalogNode, stubNode, this.messages.DeferredLoadReadStore_NoServiceProvidersSpecified, serviceProviderCatalog);
			
			this._arrayOfAllItems.push(serviceProviderCatalogNode);
			serviceProviderCatalogNode[this._storeRefPropName] = this;
			serviceProviderCatalogNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in should be the item
			parentServiceProviderCatalogTreeNode.children.push(serviceProviderCatalogNode);
			
			this._arrayOfAllItems.push(stubNode);
			stubNode[this._storeRefPropName] = this;
			stubNode[this._itemNumPropName] = (this._arrayOfAllItems.length - 1); //Last one pushed in should be the item
			serviceProviders.push(serviceProviderCatalogNode);
		}, this);
		
		this._arrayOfAllItems[parentServiceProviderCatalogTreeNode[this._itemNumPropName]] = parentServiceProviderCatalogTreeNode;
		
		if (serviceProviders.length > 0) {
			serviceProviders.sort(function(provider1, provider2){
				return com.ibm.team.process.web.client.internal.ProcessClient.stringCompare(provider1.name, provider2.name);
			});
		} else {
			serviceProviders = [];
			var serviceProviderTreeNode = {};
			serviceProviderTreeNode.name = [this.messages.DeferredLoadReadStore_NoServiceProviders];
			serviceProviderTreeNode.id = [this.messages.DeferredLoadReadStore_NoServiceProviders + new Date().getTime()];
			serviceProviderTreeNode.type = [ProcessConstants.SERVICE_PROVIDER_ERROR_TYPE];
			serviceProviders.push(serviceProviderTreeNode);
		}
		if (keywordArgs.onItem) {
			var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
			keywordArgs.onItem.call(scope, parentServiceProviderCatalogTreeNode);
		}
	}
});
