/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor"); //$NON-NLS-1$
	dojo.require("dojox.string.BidiComplex"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor"); //$NON-NLS-1$
	dojo.require("jazz.app.ResourceLink"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
	var ResourceLink = jazz.app.ResourceLink;
	var PROJECT_AREA_EDITOR = "com.ibm.team.process.editProjectArea"; //$NON-NLS-1$
	var TEAM_AREA_EDITOR = "com.ibm.team.process.editTeamArea"; //$NON-NLS-1$
	var MSG_COOKIE_NAME = "com.ibm.team.process.editTeamArea#saveMsg"; //$NON-NLS-1$
	var TEAM_INVITATION_COOKIE_NAME = "com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor#teamInvite"; //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor", com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TeamAreaEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		initialize: function(args){
			this.context = ProcessConstants.EditorContext.TEAM;
			if (!this.isNew) {
				this.itemId = args.itemId;
			}
			this.projectAreaItemId = args.projectAreaItemId;
			this.parentItemId = args.parentItemId;
			this.parentName = args.parentName;
			this.parentType = args.parentType;
			this._teamAreaIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/teamAreaIcon.png"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		// called from AbstractProcessAreaEditor.postCreate 
		customConnect: function(){
		
			this.connect(this.name, "onchange", function(){ //$NON-NLS-1$
				if (this.hierarchyNameDisplay != null) {
					dojox.data.dom.removeChildren(this.hierarchyNameDisplay);
					if (this.name.value == ProcessConstants.EMPTY) {
						this.hierarchyNameDisplay.appendChild(document.createTextNode(this.messages.TeamAreaEditor_1));
					} else {
						this.hierarchyNameDisplay.appendChild(document.createTextNode(this.name.value));
					}
				}
			});
			
			if (!this.isNew) {
				this.createChildButton.style.display = ProcessConstants.EMPTY;
				this.connect(this.createChildButton, "onclick", function(e){ //$NON-NLS-1$
					dojo.stopEvent(e);
					if (!this._handleNavigate()) {
						document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=new&parentItemId=" + this.itemId + "&projectAreaItemId=" + this.projectAreaItemId + "&parentType=TEAM"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					}
				});
			}
		},
		
		customRefresh: function(){
			dojox.data.dom.removeChildren(this.processHierarchy);
			this.processHierarchy.appendChild(document.createTextNode(this.messages.TeamAreaEditor_2));
			dojox.data.dom.removeChildren(this.developmentLineDiv);
			this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_3));
			
			if (this.isNew) {
				this.name.value = this.messages.TeamAreaEditor_4;
				this.populateDevelopmentLineSelector();
				
				if (this.parentType != null && this.parentItemId != null) {
					var srh = new ServiceResponseHandler(this, "populateProcessHierarchy", "handleTeamAreaHierarchyFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
					if (this.parentType == "TEAM") { //$NON-NLS-1$
						ProcessClient.getHierarchyForTeamArea({
							uuid: this.parentItemId
						}, srh);
					} else if (this.parentType == "PROJECT") { //$NON-NLS-1$
						ProcessClient.getProjectHierarchy({
							uuid: this.parentItemId
						}, srh);
					}
				}
				setTimeout(dojo.hitch(this.name, function(){
					try {
						this.select();
					} catch (error) {
						// If the text can't be selected...ignore the error
					}
				}), 250); //$NON-NLS-1$
			} else {
				srh = new ServiceResponseHandler(this, "populateProcessAreaData", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getTeamAreaByUUID({
					uuid: this.itemId
				}, srh);
			}
		},
		
		clearState: function(refreshClicked){
			this.inherited(arguments);
			this.selectedMembers = {};
			this.selectedAdmins = {};
			this.parentProcessArea = null;
			dojox.data.dom.removeChildren(this.processHierarchy);
			dojox.data.dom.removeChildren(this.developmentLineDiv);
		},
		
		getFetchErrorMessage: function(error){
			if (error.status == ProcessConstants.SC_FORBIDDEN) {
				return this.messages.TeamAreaEditor_ErrorNoReadAccess;
			} else {
				return this.messages.TeamAreaEditor_7;
			}
		},
		
		getPrimaryFieldLabel: function(){
			return this.messages.TeamAreaEditor_8;
		},
		
		populateProcessAreaData: function(data){
			this.inherited(arguments);
			this.projectAreaName = data.processHierarchy.name;
			
			// TODO I don't like this....need to force a refresh after
			//we are sure that the new data has been async loaded
			if (this.permissionEditor) {
				this.permissionEditor.refresh(); //update the roles
			}
			
			var teamArray = this.getAncestorsAsArray(data.processHierarchy);
			teamArray.shift();
			this.teamAreaPath = dojo.map(teamArray, "return '/'.concat(item.name)").join(ProcessConstants.EMPTY); //$NON-NLS-1$
			this.populateProcessHierarchy(data.processHierarchy);
			this.populateDevelopmentLine(data.developmentLine);
		},
		
		createInvitationPropertiesString: function(user, repoLink){
			return dojo.string.substitute("teamRepository=${0}\nuserId=${1}\nuserName=${2}\nprojectAreaName=${3}\nteamAreaPath=${4}", [repoLink, user.userId, user.name, this.projectAreaName, this.teamAreaPath]); //$NON-NLS-1$
		},
		
		getInvitationLink: function(host){
			return dojo.string.substitute("${0}/web/projects/${1}", [host, this.projectAreaName]); //$NON-NLS-1$;
		},
		
		customTeamInvitationParams: function(params){
			params.teamAreaUuid = this.itemId;
		},
		
		populateDevelopmentLine: function(data){
			dojox.data.dom.removeChildren(this.developmentLineDiv);
			if (data != null && data.label != null) {
				var label = data.label;
				if (data.projectTimeline == true) {
					label += " " + this.messages.Timeline_DecoratorText; //$NON-NLS-1$
				}
				this.developmentLineDiv.appendChild(document.createTextNode(label));
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_17));
			}
		},
		
		populateDevelopmentLineSelector: function(){
			dojox.data.dom.removeChildren(this.developmentLineDiv);
			if (this.parentProcessArea != null) {
				if (this.parentProcessArea.type == "projectArea") { //$NON-NLS-1$
					var srh = new ServiceResponseHandler(this, "developmentLineSelectorCallback", "handleProjectDevLineQueryError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getProjectDevelopmentLines({
						uuid: this.parentProcessArea.itemId
					}, srh);
				} else if (this.parentProcessArea.type == "teamArea") { //$NON-NLS-1$
					srh = new ServiceResponseHandler(this, "developmentLineDisplayCallback", "handleTeamDevLineQueryError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getTeamAreaByUUID({
						uuid: this.parentProcessArea.itemId
					}, srh);
				}
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_18));
			}
		},
		
		handleProjectDevLineQueryError: function(error){
			this.handleServerResponseError(error, this.messages.TeamAreaEditor_19);
		},
		
		handleTeamDevLineQueryError: function(error){
			this.handleServerResponseError(error, this.messages.TeamAreaEditor_20);
		},
		
		developmentLineSelectorCallback: function(data){
			dojox.data.dom.removeChildren(this.developmentLineDiv);
			if (data != null && data.length > 0) {
				var select = document.createElement("select"); //$NON-NLS-1$
				this.developmentLineSelector = select;
				for (var i = 0; i < data.length; i++) {
					var option = new Option(data[i].label, data[i].itemId);
					if (data[i].summary != null && data[i].summary != ProcessConstants.EMPTY) {
						option.description = data[i].summary;
					} else if (data[i].description != null && data[i].description != ProcessConstants.EMPTY) {
						option.description = data[i].description;
					} else {
						option.description = this.messages.TeamAreaEditor_21;
					}
					if (dojo.isIE) {
						select.options[select.options.length] = option;
					} else {
						select.appendChild(option);
					}
				}
				this.developmentLineDiv.appendChild(select);
			} else {
				this.developmentLineDiv.appendChild(document.createTextNode(this.messages.TeamAreaEditor_22));
			}
		},
		
		developmentLineDisplayCallback: function(data){
			this.populateDevelopmentLine(data.developmentLine);
		},
		
		populateProcessHierarchy: function(project){
			this._project = project;
			dojox.data.dom.removeChildren(this.processHierarchy);
			this.showBreadcrumbs();
			var div = document.createElement(ProcessConstants.DIV);
			var link = document.createElement("a"); //$NON-NLS-1$
			link.tabIndex = 3;		
			link.href = this._getServerUrl() + net.jazz.ajax._contextRoot + "/process/project-areas/" + this._project.itemId;//$NON-NLS-1$
			presentationUri = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + this._project.itemId; //$NON-NLS-1$ //$NON-NLS-2$
			var rl = new ResourceLink( {
				'presentationUri': presentationUri //$NON-NLS-1$ 
			}, link);
			link.appendChild(document.createTextNode(this._project.name));
			div.appendChild(link);
			div.className = "hierarchyElement"; //$NON-NLS-1$
			this.processHierarchy.appendChild(div);
			var isNewParent = false;
			if (this.isNew && this._project.itemId === this.parentItemId) {
				this.parentProcessArea = {
					name: this._project.name,
					itemId: this._project.itemId,
					type: "projectArea" //$NON-NLS-1$
				};
				isNewParent = true;
			}
			if (this._project.children != null && this._project.children.length > 0) {
				this.populateRemainingHierarchy(this._project.children, 1, isNewParent);
			} else {
				if (isNewParent) {
					this.populateRemainingHierarchy([], 1, true);
				}
			}
			this.populateDevelopmentLineSelector();
		},
		
		populateRemainingHierarchy: function(teams, indent, includeNewTeam){
			teams.sort(function(a, b){
				return a.name > b.name ? 1 : a.name < b.name ? -1 : 0;
			});
			dojo.forEach(teams, dojo.hitch(this, function(team){
				var div = document.createElement(ProcessConstants.DIV);
				var link = document.createElement("a"); //$NON-NLS-1$
				var openEditorUri = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + team.itemId + "&projectAreaItemId=" + this._project.itemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				//show a rich hover for other team areas (not the one open in this editor)
				if (this.itemId !== team.itemId) {
					link.href = this._getServerUrl() + net.jazz.ajax._contextRoot + "/process/project-areas/" + this.projectAreaItemId + "/team-areas/" + team.itemId; //$NON-NLS-1$ //$NON-NLS-2$  
					var rl = new ResourceLink({
						'presentationUri': openEditorUri//$NON-NLS-1$
					}, link);
				} else {
					link.href = openEditorUri;
				}
				
				link.tabIndex = 3;
				link.appendChild(document.createTextNode(team.name));
				div.appendChild(link);
				div.className = "hierarchyElement"; //$NON-NLS-1$
				if (!this.isNew && this.itemId === team.itemId) {
					div.className += " selected"; //$NON-NLS-1$
					this.hierarchyNameDisplay = div;
				}
				div.style.paddingLeft = (15 * indent) + "px"; //$NON-NLS-1$
				this.processHierarchy.appendChild(div);
				var isNewParent = false;
				if (this.isNew && team.itemId === this.parentItemId) {
					this.parentProcessArea = {
						name: team.name,
						itemId: team.itemId,
						type: "teamArea" //$NON-NLS-1$
					};
					isNewParent = true;
				}
				if (team.children != null && team.children.length > 0) {
					this.populateRemainingHierarchy(team.children, indent + 1, isNewParent);
				} else {
					if (isNewParent) {
						this.populateRemainingHierarchy([], indent + 1, true);
					}
				}
			}));
			if (includeNewTeam) {
				var div = document.createElement(ProcessConstants.DIV);
				this.hierarchyNameDisplay = div;
				div.className = "hierarchyElement selected"; //$NON-NLS-1$
				div.style.paddingLeft = (15 * indent) + "px"; //$NON-NLS-1$
				div.appendChild(document.createTextNode(this.messages.TeamAreaEditor_23));
				this.processHierarchy.appendChild(div);
			}
		},
		
		// returns an array representing the process hierarchy directly above this team area
		getAncestorsAsArray: function(processArea){
			var foundParentOfNewTeam = this.isNew && this.parentItemId === processArea.itemId;
			var foundThisTeam = !this.isNew && this.itemId === processArea.itemId;
			var hasChildren = processArea.children && processArea.children.length > 0;
			
			// Return an array containing this process area so it can bubble up recursively
			// The recursive callers will insert their process area at the front of this array so
			// we get a complete ancestry array in the end 
			if (foundParentOfNewTeam || foundThisTeam) {
				return [processArea];
			} else if (hasChildren) {
				for (var i = 0; i < processArea.children.length; i++) {
					var curLine = this.getAncestorsAsArray(processArea.children[i]);
					// If the current child team exists along the ancestry path, we know this element
					// also belongs along the ancestry path.  Add the current processArea to the front
					// of the ancestry path and bubble up to the next recursive caller.
					if (curLine) {
						curLine.unshift(processArea);
						return curLine;
					}
				}
			} else {
				// We didn't find what we were looking for down this branch.
				// Return null so the recursive callers will ignore this path.
				return null;
			}
		},
		
		customShowBreadcrumbs: function(){
			var hierarchy = this.getAncestorsAsArray(this._project);
			for (var i = 0; i < hierarchy.length; i++) {
				// Don't include the current element in the breadcrumb hierarchy
				if (!this.isNew && i === hierarchy.length - 1) {
					continue;
				}
				var link = document.createElement("a"); //$NON-NLS-1$
				if (i === 0) {
					link.href = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + hierarchy[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					link.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + hierarchy[i].itemId + "&projectAreaItemId=" + this.projectAreaItemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
				link.appendChild(document.createTextNode(dojox.string.BidiComplex.createDisplayString(hierarchy[i].name), "URL")); //$NON-NLS-1$
				this.breadcrumbs.appendChild(link);
				var span = document.createElement("span"); //$NON-NLS-1$
				span.appendChild(document.createTextNode(">")); //$NON-NLS-1$
				this.breadcrumbs.appendChild(span);
			}
		},
		
		customChangesPending: function(){
			return false;
		},
		
		handleSave: function(e){
			dojo.stopEvent(e);
			this.saveButton.disabled = true;
			var saveObj = {
				itemId: this.itemId
			};
			if (!this.isNew) {
				saveObj.stateId = this.stateId;
			}
			for (var i = 0; i < this.inputFields.length; i++) {
				if (this.inputFields[i].changePending) {
					saveObj[this.inputFields[i].name] = this.inputFields[i].value;
				}
			}
			saveObj.jsonMembers = dojo.toJson(this.pendingMemberChanges);
			saveObj.jsonAdmins = dojo.toJson(this.pendingAdminChanges);
			if (this.isNew && this.parentProcessArea != null) {
				saveObj.jsonParentProcessArea = dojo.toJson(this.parentProcessArea);
			}
			if (this.developmentLineSelector != null && this.developmentLineSelector.value != null) {
				saveObj.developmentLineUuid = this.developmentLineSelector.value;
			}
			
			this.generateChangeParameters(saveObj);
			this.generateRoleChanges(saveObj);
			
			
			var msg = this.validateRequired(saveObj);
			if (msg != null && msg != ProcessConstants.EMPTY) {
				this.displayErrorMessage(msg);
			} else {
				var srh = new ServiceResponseHandler(this, "handleSaveResponse", "handleSaveError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.postTeamArea(saveObj, srh);
				this.displayPendingMessage(this.messages.TeamAreaEditor_26);
			}
		},
		
		validateRequired: function(saveData){
			if (saveData.name == ProcessConstants.EMPTY || (this.isNew && saveData.name == null)) {
				this.togglePrimaryField(ProcessConstants.EditorMode.EDIT);
				return this.messages.TeamAreaEditor_27;
			}
			if (this.isNew && (saveData.jsonParentProcessArea == null || saveData.jsonParentProcessArea == ProcessConstants.EMPTY)) {
				return this.messages.TeamAreaEditor_28;
			}
		},
		
		handleSaveResponse: function(response){
			if (this.pendingMemberChanges.add != null && this.pendingMemberChanges.add.length > 0) {
				var currentUser = getAuthenticatedContributor();
				var newMemberIds = dojo.filter(this.pendingMemberChanges.add, function(memberId){
					return memberId != currentUser.itemId;
				});
				if (newMemberIds.length > 0) {
					if (this.isNew && response) {
						// in this case we'll be destroying the page and creating a new one
						// so we have to save the member ids in a cookie
						dojo.cookie(TEAM_INVITATION_COOKIE_NAME, dojo.toJson(newMemberIds));
					} else {
						// not destroying the page, so call directly
						this.chooseTeamInvitees(newMemberIds);
					}
				}
			}
			
			if (this.isNew && response) {
				dojo.cookie(MSG_COOKIE_NAME, this.messages.TeamAreaEditor_29);
				this._unregisterHandlers();
				document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + response + "&projectAreaItemId=" + this.projectAreaItemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			} else {
				this.refresh(false);
				this.displaySuccessMessage(this.messages.TeamAreaEditor_30, true);
				dojo.forEach(this._saveParticipants, function(saveParticipant){
					saveParticipant.saved(response);
				});
			}
		},
		
		getSaveErrorMessage: function(err){
			return this.messages.TeamAreaEditor_31;
		},
		
		enableSaveButton: function(isDirty){
			if (isDirty && this._canSave()) {
				this.saveButton.disabled = false;
			} else {
				this.saveButton.disabled = true;
			}
		},
		
		_getProcessAreaStore: function(){
			var args = {
				editorContext: ProcessConstants.EditorContext.TEAM,
				processAreaItemId: this.itemId,
				errorContext: this,
				errorCallback: this._handleCreateStoreError
			};
			return new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore(args);
		}
		
	});
	
})();
