﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class HorizontalBarChart extends BarChart {
	
	public function HorizontalBarChart() {
		super();
	}
	
	public function draw(parentMC:MovieClip):Void {

		super.draw(parentMC);		
		
		this.grid = new HorizontalBarGrid(this.width, this.height, this.startX, this.startY);
		this.grid.draw(this.mc);
		
		this._createBarShapes();
		this._drawBarSeries();
	}
	
	private function _createBarShapes():Void {

		var model:ChartModel = ChartEngine.getInstance().getModel();
		
		for (var index:Number = 0; index < model.parameterCount; index++) {
			
			var outlineColour = ColourUtility.darkenColour(model.parameterColours[index], 
														   Chart.RGB_DARKEN_PERCENTAGE);
			
			this.shapes[index] = new HorizontalBarShape(outlineColour, 
											 			model.parameterColours[index],
													  	model.parameterNames[index], 											  
													  	model.parameterValues[index],
													  	true);
		}
	}	
	
	private function _drawBarSeries() {
		
		var singleBarArea:Number = this.grid.getGridData().graphAreaHeight / this.shapes.length;
		var barWidth:Number = singleBarArea / 2;
		
		if (barWidth > BarShape.MAX_BAR_THICKNESS) {
			barWidth = BarShape.MAX_BAR_THICKNESS;
		}
		
		var barSpaceWidth:Number = (singleBarArea - barWidth) / 2;
		var startY:Number = this.grid.getGridData().startY;		
		var nextY = startY + barSpaceWidth;
		
		for (var barIndex:Number = 0; barIndex < this.shapes.length; barIndex++) {
			
			var bar:BarShape = this.shapes[barIndex];
			var x1:Number = this.grid.getGridData().xZeroPosition;
			var x2:Number = this.grid.getXCoordinate(bar.getValue());
			var y1:Number = nextY;
			var y2:Number = nextY + barWidth;
			
			bar.draw(this.mc, x1, x2, y1, y2);
			
			nextY = nextY + barSpaceWidth + barWidth + barSpaceWidth;
			
		}		
	}	
	
	public function destroy():Void {
		super.destroy();
	}		
	
}