﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

class PieShape extends ChartShape {

	private var _centerX:Number;
	
	private var _centerY:Number;
	
	private var _startAngle:Number;
	
	private var _arcAngle:Number;
	
	private var _radius:Number;
	
	private var _jumpOffsetX:Number;

	private var _jumpOffsetY:Number;
	
	private var _jumped:Boolean;
	
	private var _step:Number;
	
	private var _midX:Number;
	
	private var _midY:Number;

	private var _midAngle:Number;

	private var _hoverMcID:String;

	private var _animateJumpStep:Number;
	
	private var _maxAnimateJumpOffset:Number;
	
	private var _hoverMc:MovieClip;
	
	// Number of radians in a full circle
	private static var _TWO_PIE:Number = 2 * Math.PI;
	
	private static var _MAX_ANIMATE_JUMP_OFFSET_RATIO:Number = 6;
	
	private static var _ANIMATE_JUMP_STEP_RATIO:Number = 3;
	
	
	public function PieShape(outlineColour:Number, fillColour:Number, name:String, value:Number, animate:Boolean) {
		super(outlineColour, fillColour, name, value, animate);
		
		this._jumpOffsetX = 0;
		this._jumpOffsetY = 0;
		
		this._jumped = false;
		
		this._step = 0;
		
		this._hoverMcID = "hover_mc_" + ChartShape._instanceCount;
	}

	public function draw(parentMC:MovieClip, startX:Number, startY:Number, startAngle:Number, arc:Number, radius:Number) {
		
		// Set globals
		this.mc = parentMC.createEmptyMovieClip(this.id, parentMC.getNextHighestDepth());
		this._hoverMc = parentMC.createEmptyMovieClip(this._hoverMcID, parentMC.getNextHighestDepth());
		
		// Make the hover movie clip completely invisible
		// The hover movie clip's only purpose is to trigger stable (stationary shape object) mouse over events
		this._hoverMc._alpha = 0;
		
		this._centerX = startX;
		this._centerY = startY;
		this._startAngle = (startAngle / PieChart.FULL_CIRCLE_DEGREES) * -1;
		this._arcAngle = (arc / PieChart.FULL_CIRCLE_DEGREES) * -1;
		this._radius = radius;
		
		this.mc.beginFill(this.fillColour);
		this._hoverMc.beginFill(0xFFFFFF);
		
		// TODO: Magic number of outline thickness
		this.mc.lineStyle(1, this.outlineColour);
		this._hoverMc.lineStyle(1, 0xFFFFFF);
		
		// TODO: Revisit - use radians instead?
		this._midAngle = this._startAngle + (this._arcAngle / 2);
		this._midX = this._centerX + Math.cos(this._midAngle * PieShape._TWO_PIE) * this._radius;
		this._midY = this._centerY + Math.sin(this._midAngle * PieShape._TWO_PIE) * this._radius;
		// TODO: Consider calculating this once only in pie chart object (values the same for all pie shapes)
		this._maxAnimateJumpOffset = Math.round(this._radius / PieShape._MAX_ANIMATE_JUMP_OFFSET_RATIO);
		this._animateJumpStep = Math.round(this._maxAnimateJumpOffset / PieShape._ANIMATE_JUMP_STEP_RATIO);
		
		// TODO: Code clean-up

		this.mc.moveTo(this._centerX, this._centerY);
		this._hoverMc.moveTo(this._centerX, this._centerY);

		// Flash uses 8 segments per circle, to match that, we draw in a maximum
		// of 45 degree segments. First we calculate how many segments are needed
		// for our arc.
		var segments:Number = Math.ceil(Math.abs(arc) / 45);
		// Calculate the sweep of each segment.
		var segmentAngle:Number = arc / segments;
		// The math requires radians rather than degrees. To convert from degrees
		// use the formula (degrees/180)*Math.PI to get radians.
		var theta:Number = -(segmentAngle / PieChart.HALF_CIRCLE_DEGREES) * Math.PI;
		// Convert angle startAngle to radians
		var angle:Number = -(startAngle / PieChart.HALF_CIRCLE_DEGREES) * Math.PI;
		// Draw the curve in segments no larger than 45 degrees.
		if (segments > 0) {
			// Draw a line from the center to the start of the curve
			var ax:Number = this._centerX + Math.cos(startAngle / PieChart.HALF_CIRCLE_DEGREES * Math.PI) * radius;
			var ay:Number = this._centerY + Math.sin(-startAngle / PieChart.HALF_CIRCLE_DEGREES * Math.PI) * radius;
			
			this.mc.lineTo(ax, ay);
			this._hoverMc.lineTo(ax, ay);
			
			// Loop for drawing curve segments
			var angleMid:Number;
			var bx:Number;			
			var by:Number;			
			var cx:Number;			
			var cy:Number;			
			
			for (var i = 0; i < segments; i++) {
				angle += theta;
				angleMid = angle - (theta / 2);
				bx = this._centerX + Math.cos(angle) * radius;
				by = this._centerY + Math.sin(angle) * radius;
				cx = this._centerX + Math.cos(angleMid) * (radius / Math.cos(theta / 2));
				cy = this._centerY + Math.sin(angleMid) * (radius / Math.cos(theta / 2));
				
				this.mc.curveTo(cx, cy, bx, by);
				this._hoverMc.curveTo(cx, cy, bx, by);
			}
			
			// Close the wedge by drawing a line to the center
			this.mc.lineTo(this._centerX, this._centerY);
			this._hoverMc.lineTo(this._centerX, this._centerY);
		}
		
		this.mc.endFill();
		this._hoverMc.endFill();
		
		// Event handlers
		this._addEventHandlers();	
	}

	private function _addEventHandlers():Void {
		// TODO: Generalize with bar shape?
		var context = this;

		this._hoverMc.onRollOver = function() {
			context.mc._alpha = 60;
			context._animateJumpOut();
			context.tooltip.show();
		};
		this._hoverMc.onRollOut = function() {
			context.mc._alpha = 100;
			context._animateJumpIn();
			context.tooltip.hide();
		};	
		
		var model:ChartModel = ChartEngine.getInstance().getModel();
		if (model.onClickHandler) {
			this._hoverMc.onPress = function() {
				context.doDrillIn();
			};
		}
	}
	
	public function doRollOver():Void {
		// TODO: Generalize further
		this._animateJumpOut();		
		this.tooltip.setPosition(this._midX, this._midY);
		super.doRollOver();
	}
	
	public function doRollOut():Void {
		super.doRollOut();
		this._animateJumpIn();		
	}
	
	private function _animateJumpIn():Void {
		
		if (this.animate == false) {
			return;
		} else if (this._jumped == false) {
			return;
		}		
		
		this._jumped = false;
		
		var context = this;

	    this.mc.onEnterFrame = function() {
	    	
	    	context._step = context._step - context._animateJumpStep;
	        
	        context._calculateJumpOffsets(context._step);
	        
	        this._x = context._jumpOffsetX;
			this._y = context._jumpOffsetY;
	        
	        if (context._step <= 0) {
	            delete this.onEnterFrame;
				this._x = 0;
				this._y = 0;
	        }
	    };		
		
	}

	private function _animateJumpOut():Void {
		
		if (this.animate == false) {
			return;
		} else if (this._jumped) {
			return;
		}
		
		this._jumped = true;
		this._step = 0;

		var context = this;

	    this.mc.onEnterFrame = function() {

	    	context._step = context._step + context._animateJumpStep;
	        
	        context._calculateJumpOffsets(context._step);
	        
	        this._x = context._jumpOffsetX;
			this._y = context._jumpOffsetY;
	        
	        if (context._step >= context._maxAnimateJumpOffset) {
	            delete this.onEnterFrame;
	        }
	    };
		
	}	
	
	private function _calculateJumpOffsets(step:Number):Void {

		var nextX = this._centerX + Math.cos(this._midAngle * PieShape._TWO_PIE) * (this._radius + step);
		var nextY = this._centerY + Math.sin(this._midAngle * PieShape._TWO_PIE) * (this._radius + step);		
		
		this._jumpOffsetX = nextX - this._midX;
		this._jumpOffsetY = nextY - this._midY;
	}
	
	public function destroy():Void {
		
		delete this._centerX;
		delete this._centerY;
		delete this._startAngle;
		delete this._arcAngle;
		delete this._radius;
		delete this._jumpOffsetX;
		delete this._jumpOffsetY;
		delete this._jumped;
		delete this._step;
		delete this._midX;
		delete this._midY;
		delete this._midAngle;
		delete this._hoverMcID;
		delete this._animateJumpStep;
		delete this._maxAnimateJumpOffset;
		
		// Hover movie clip
		if (this._hoverMc) {
			this._hoverMc.removeMovieClip();
			delete this._hoverMc;
		}		
		
		super.destroy();
	}			
	
}