/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.module.WorkItemCreationWidget");

// declare dependencies on other Dojo modules
dojo.require("com.ibm.team.workitem.web.ui.internal.module.AbstractChooserWidget");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

(function() {

    
var InlineWorkItemEditor = com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor;
var WorkItemProxyFactory = com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory;
var WorkItemAttributes= com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;

var ITEM_MESSAGE_CREATE= "oslc_cm.create";

var OSCL_ATTRIB_TYPE= "dc:type";
var OSCL_ATTRIB_SUMMARY= "dc:name";
var OSCL_ATTRIB_DESCRIPION= "dc:description";

// declare and implement the widget "class"
dojo.declare("com.ibm.team.workitem.web.ui.internal.module.WorkItemCreationWidget", [ com.ibm.team.workitem.web.ui.internal.module.AbstractChooserWidget ], {

_workingCopy: null,

_openDialog: function(currentProjectArea) {
	var workItemTypeId= this._dialogArguments[OSCL_ATTRIB_TYPE];
	if (!workItemTypeId) {
		this._fetchWorkItemTypes(currentProjectArea);
	} else {
	    this._createWorkingCopy(currentProjectArea, workItemTypeId);
	}
},

_fetchWorkItemTypes: function(currentProjectArea) {
	var self= this;
	var handlerObj= {
		success: function(/* UIItemSetDTO */ valueSet) {
			self._createWorkingCopy(currentProjectArea, valueSet[0].uiItems[0].id);
		},
		failure: function(errorObj) {
			self._handleError(self.messages.errorCannotFetchWorkItemTypes);
		}
	};
	
	var handler= new ServiceResponseHandler(handlerObj, "success", "failure");
	WorkItemClient.getAllValues(handler, { newWorkItem: true, ids: [ WorkItemAttributes.TYPE ] });
},

_createWorkingCopy: function(currentProjectArea, workItemTypeId) {
    var self= this;
	var workingCopy= WorkItemProxyFactory.getWorkItemProxy({createIfNeeded: true});
	workingCopy.incrementUseCount();
	workingCopy.initialize({
		fetchEditorPresentation: false,
		fetchInlinePresentation: false,
		type: workItemTypeId,
		onSuccess: function() {
			self._openDialog2(workingCopy, currentProjectArea);
		}, 
		onError: function() {
			self._handleError(self.messages.errorCannotCreateWorkingCopy);
		}
	});
},

_openDialog2: function(workingCopy, currentProjectArea) {
	this._workingCopy= workingCopy;
	
	var initialSummary= this._dialogArguments[OSCL_ATTRIB_SUMMARY];
	if (initialSummary)
        this._setWorkingCopyValue(workingCopy, WorkItemAttributes.SUMMARY, initialSummary);
		
	var initialDescription= this._dialogArguments[OSCL_ATTRIB_DESCRIPION];
	if (initialDescription)
        this._setWorkingCopyValue(workingCopy, WorkItemAttributes.DESCRIPTION, initialDescription);

	var bodyDiv = document.createElement("div");
	bodyDiv.className= "com-ibm-team-workitem-web-ui-internal-module-WorkItemCreationWidget";

    var contentParams = {
		workItemId: workingCopy.getId(),
		parentWidget: this,
        projectArea : this._projectArea,
        isTrimless: true,
        isShowingBottomSaveButtons: false,
        isShowingTopSaveButtons: false,
        isShowingTopCloseButton: false
    };
    var content = new InlineWorkItemEditor(contentParams);
    dojo.style(content.domNode, "marginLeft", "0em");
    dojo.style(content.domNode, "width", "100%");
    dojo.style(content.domNode, "height", "475px");
    dojo.style(content.domNode, "overflow", "auto");
    dojo.style(content.domNode, "position", "relative");
    
    bodyDiv.appendChild(content.domNode);
    
	var buttonDiv = document.createElement("div");
	buttonDiv.className = "buttonSection";
	
	var okButton = document.createElement("button");
	okButton.className = "Button";
	okButton.tabOrder = "5";
	okButton.disabled = "disabled";
	okButton.appendChild(document.createTextNode(this.messages.buttonOK));
	this.okButton = okButton;
	
	this.connect(okButton, "onclick", dojo.hitch(content, content._handleBottomOkClicked));

	var cancelButton = document.createElement("button");
	cancelButton.className = "Button";
	cancelButton.tabOrder = "6";
	cancelButton.appendChild(document.createTextNode(this.messages.buttonCancel));

	this.connect(cancelButton, "onclick", dojo.hitch(content, content._handleBottomCancelClicked));
	
	buttonDiv.appendChild(okButton);
	buttonDiv.appendChild(cancelButton);
	bodyDiv.appendChild(buttonDiv);
			
	 this._setContentWidget(bodyDiv);
},

_setWorkingCopyValue: function(workingCopy, attributeId, newValue) {
    workingCopy.setValue({
        caller: self,
        path: ["attributes", attributeId, "content"],
        savedAttributePath: ["attributes", attributeId, "content"],
        attributeId: attributeId,
        value: newValue
    });
},

// called by the InlineWorkItemEditor
hidePreviewEditor: function() {
	var newItems= null;

	if (!this._workingCopy.isNewWorkItem()) {
		newItems= [ { 
			"id": this._workingCopy.getId(), 
			"summary" : this._workingCopy.getSummaryContent()
		} ];
	}
	
	this._close(newItems, ITEM_MESSAGE_CREATE);
},

// called by the InlineWorkItemEditor
setSaveEnabled: function(parentParms, isDirty) {
	if (this.okButton) {
		this.okButton.disabled= isDirty ? "" : "disabled";
	}
},

_close: function() {
	if (this._workingCopy) {
	    this._workingCopy.decrementUseCount();
	    this._workingCopy= null;
	}

    this.inherited(arguments); // Call superclass method...  
}


	
});

})();
