/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.page.PlanningPageController"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.QueryDescriptor"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.SearchResult"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.ServiceCallController"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var isGuest				= com.ibm.team.repository.web.client.session.isGuest;

var Assert				= com.ibm.team.apt.web.ui.internal.common.Assert;
var PlatformUI			= net.jazz.ajax.ui.PlatformUI;
var ActionRegistry		= null; // initialize later
var ResourceRegistry	= null;


var PlanningUtil		= com.ibm.team.apt.web.client.util;
var PlanningClient		= com.ibm.team.apt.web.client.internal.PlanningClient;
var ResponseHandler		= com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler;
var QueryDescriptor		= com.ibm.team.apt.web.ui.internal.common.QueryDescriptor;
var SearchResult		= com.ibm.team.apt.web.ui.internal.common.SearchResult;

var RESULTPAGE_SIZE		= 50;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * Main application controller method for Jazz iteration planning web ui. Modules
 * *outside* of the com.ibm.team.apt.web component should *not* access
 * methods in this controller directly; rather external modules should update
 * the browser's URIs using URIs returned from
 * <code>com.ibm.team.apt.web.client.util</code> helper methods.
 */
dojo.declare("com.ibm.team.apt.web.ui.internal.page.PlanningPageController", //$NON-NLS-1$ 
			  com.ibm.team.apt.web.ui.internal.common.ServiceCallController, {
			  	
	_currentAction: null,
	_currentActionArgs: null,
	_currentSearchResults: null,

	// ---- API ------------------------------------------------------------------------------------------------------------

	constructor: function(aptView) {
		this._view = aptView;
		// TODO we should remove old query identifiers from this map
		this._searchTokens= {};
		this._searchResults= {};
	},

	registerCommands: function() {
		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ResourceRegistry = PlatformUI.getWorkbench().getResourceRegistry();

		// register actions
		ActionRegistry.registerAction("com.ibm.team.apt._showDefaultView", this, "_showDefaultView"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.team.apt.search", this, "search"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.team.apt.viewPlan", this, "viewPlan"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.team.apt.createPlan", this, "createPlan"); //$NON-NLS-1$ //$NON-NLS-2$
		ActionRegistry.registerAction("com.ibm.team.apt.editPlan", this, "editPlan"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	// ---- action handlers ------------------------------------------------------------------------------------------------
	
	_showDefaultView: function(args, forceDefaultQuery) {
		var action= this._currentAction;
		var args= this._currentActionArgs;

		// avoid endless redirect when in a bad state
		this._currentAction= null;
		this._currentActionArgs= null;

		if (action == null) {
			action= this.search;
			args= [ { }, true];
			
			var queryToRun= PlanningUtil.QUERY_CURRENTPLANS;
			if (!isGuest() && !forceDefaultQuery) {
				queryToRun= PlanningUtil.QUERY_MYPLANS;
				args.push(dojo.hitch(this, function() { 
					this._currentAction= null;
					this._currentActionArgs= null;
					this._showDefaultView(null, true);
				}));
			}
			
			args[0][PlanningUtil.QUERYTYPE_PREDEFINED]= queryToRun;
		}

		action.apply(this, args);
	},

	/**
	 * 
	 * @param {Object} args
	 */
	search: function(args, forceShowResultPage, showNoResultFn) {
		var keywords= null;
		var predefinedQueryId= null;
		var custom= null;
		
		if (dojo.exists("keywords", args)) { //$NON-NLS-1$
			keywords= args["keywords"]; //$NON-NLS-1$
		}
		if (dojo.exists("predef", args)) { //$NON-NLS-1$
			if (dojo.indexOf(PlanningUtil.PREDEFINED_QUERIES, args["predef"]) != -1) { //$NON-NLS-1$
				predefinedQueryId= args["predef"]; //$NON-NLS-1$
			}
		} else if (dojo.exists("relationshipType", args) && dojo.exists("relatedItemId", args)) { //$NON-NLS-1$ //$NON-NLS-2$
			if (dojo.indexOf(PlanningUtil.RELATIONSHIPS_QUERIES, args["relationshipType"]) != -1) { //$NON-NLS-1$
				custom= {
					relationshipType: args["relationshipType"], //$NON-NLS-1$
					relatedItemId: args["relatedItemId"] //$NON-NLS-1$
				};
				
				custom.currentOnly= args["currentOnly"] == "true"; //$NON-NLS-1$ //$NON-NLS-2$
			}
		}

		if (keywords != null || predefinedQueryId != null || custom != null) {
			var querySequenceId= null;
			var resultIndex= 0;
			if (dojo.exists("tq", args)) { //$NON-NLS-1$
				querySequenceId= args["tq"]; //$NON-NLS-1$
			}
			
			if (dojo.exists("index", args)) { //$NON-NLS-1$
				resultIndex= parseInt(args["index"]); //$NON-NLS-1$
			}
			
			var queryDescriptor= new QueryDescriptor(keywords, predefinedQueryId, custom, querySequenceId);
			
			var showSingleResultFn= forceShowResultPage ? null : dojo.hitch(this, this._showSingleResult);
			this._doHandleSearchRequest(queryDescriptor, resultIndex, dojo.hitch(this, this._showResultPage), showSingleResultFn, showNoResultFn);
			
			this._currentAction= this.search;
			this._currentActionArgs= arguments;
		} else {
			console.debug("incomprehensible search parameters, redirecting to default view! " + dojo.toJson(args)); //$NON-NLS-1$
			this._showDefaultView();
		}
	},

	createPlan: function(args){
		var _self= this;
		var serviceCallback= {
				_success: function(result) {
				_self._view.showCreatePlan(result);
			},
			_error: function(error) {
				_self.remoteError(error);
			}
		};
		var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
		responseHandler.setLabel(MSGS.MONITOR_LOAD_PLAN, this);		
		PlanningClient.fetchNewPlan(responseHandler);
	},
	
	editPlan: function(args) {
		var _self= this;
		var serviceCallback= {
				_success: function(result) {
				_self._view.showEditPlan(result);
			},
			_error: function(error) {
				_self.remoteError(error);
			}
		};
		var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
		responseHandler.setLabel(MSGS.MONITOR_LOAD_PLAN, this);		
		PlanningClient.fetchEditPlan(responseHandler, args.id);
	},
	
	savePlan: function(args){
		var _self= this;
		var serviceCallback= {
			_success: function(result) {
				_self._view._validateRecentPlans(); 
				location.href= PlanningUtil.getPlanViewURI(result.plan.itemId, {ignoreCache: true});
			},
			_error: function(error) {
				_self.remoteError(error);
			}
		};
		var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
		responseHandler.setLabel(MSGS.MONITOR_SAVE_PLAN, this);		
		PlanningClient.postModifyPlan(responseHandler, args);
	},
	
	deletePlan: function(args) {
		var _self= this;
		var serviceCallback= {
			_success: function(result) {
				_self._view._validateRecentPlans(); 
			},
			_error: function(error) {
				_self.remoteError(error);
			}
		};
		
		var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
		responseHandler.setLabel(MSGS.MONITOR_DELETE_PLAN, this);		
		PlanningClient.deletePlan(responseHandler, args);
	},
	
	viewPlan: function(args) {
		if (dojo.exists("id", args)) { //$NON-NLS-1$
			var iterationPlanId= args["id"]; //$NON-NLS-1$

			if (!this._ensureSearchResultsAvailable(iterationPlanId, args)) {
				// the next or previous search result entry is not yet available but requrired by the meta nav
				// the _ensureSearchResultsAvailable method is fetching the data right now and will
				// call this method again.
				return;
			}

			if (!dojo.exists("page", args)) { //$NON-NLS-1$
				args["page"]= "overview"; //$NON-NLS-1$ //$NON-NLS-2$
			}			 

			if (!this._view.showCachedIterationPlan(iterationPlanId, args, this._currentSearchResults) || dojo.exists("ignoreCache", args)) { //$NON-NLS-1$
				var _self= this;
				var serviceCallback= {
					_success: function(result) {
						_self._view.showIterationPlan(result, args, _self._currentSearchResults);
					},
					_error: function(error) {
						_self.remoteError(error);
					}
				};
				
				var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
				responseHandler.setLabel(MSGS.MONITOR_LOAD_PLAN, this);		
				
				PlanningClient.fetchIterationPlan(responseHandler, iterationPlanId);
			}

			this._currentAction= this.viewPlan;
			this._currentActionArgs= arguments;
		} else {
			console.debug("incomprehensible plan view parameters, redirecting to default view! " + dojo.toJson(args)); //$NON-NLS-1$
			this._showDefaultView();
		}
	},
	
	refreshPlan: function(args) {
		if (dojo.exists("id", args)) { //$NON-NLS-1$
			var iterationPlanId= args["id"]; //$NON-NLS-1$

			if (!dojo.exists("page", args)) { //$NON-NLS-1$
				args["page"]= "overview"; //$NON-NLS-1$ //$NON-NLS-2$
			}			 

			var _self= this;
			var serviceCallback= {
				_success: function(result) {
					_self._view.showIterationPlan(result, args, _self._currentSearchResults);
				},
				_error: function(error) {
					_self.remoteError(error);
				}
			};
			
			var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
			responseHandler.setLabel(MSGS.MONITOR_LOAD_PLAN, this);		
			
			PlanningClient.fetchIterationPlan(responseHandler, iterationPlanId);

			this._currentAction= this.viewPlan;
			this._currentActionArgs= arguments;
		} else {
			console.debug("incomprehensible plan view parameters, redirecting to default view! " + dojo.toJson(args)); //$NON-NLS-1$
			this._showDefaultView();
		}
	},
 
	// ---- implementation -------------------------------------------------------------------------------------------------	

	_ensureSearchResultsAvailable: function(iterationPlanId, args) {
		var searchResults= this._currentSearchResults;
		if (searchResults != null) {
			var index= searchResults.getIndex(iterationPlanId);
			
			if (index > 0 && index < searchResults.getResultCount() - 1) {
				var loadStartIndex= -1;
				if (!searchResults.hasResult(index - 1)) {
					loadStartIndex= this._getPageStartForIndex(index - 1, searchResults);
				} else if (!searchResults.hasResult(index + 1)) {
					loadStartIndex= this._getPageStartForIndex(index + 1, searchResults);
				}
	
				if (loadStartIndex != -1) {
					Assert.isTrue(loadStartIndex != this._getPageStartForIndex(index, searchResults));
					
					var queryDescriptor= searchResults.getQueryDescriptor();
					Assert.isNotNull(queryDescriptor);
					
					var _self= this;
					this._doHandleSearchRequest(queryDescriptor, loadStartIndex, function(searchResult, resultIndex) {
						// wrong assertions -> if the server threw away our search results (token), we start with a new
						// search result and must probably do another roundtrip
						// Assert.isTrue(_self._currentSearchResults === searchResult);
						// Assert.isTrue(searchResult.hasResults(index - 1, index + 1));
						
						_self.viewPlan(args);
					});

					return false;
				}
			}
		}
		
		return true;
	},

	_getPageStartForIndex: function(index, searchResults) {
		Assert.isLegal(index >= 0);
		Assert.isLegal(index < searchResults.getResultCount());

		return Math.floor(index / searchResults.getResultPageSize()) * searchResults.getResultPageSize(); 
	},

	_doHandleSearchRequest: function(queryDescriptor, resultIndex, showResultPageFn, showSingleResultFn, showNoResultFn) {
		if (!this._view.showCachedSearchResult(queryDescriptor.getIdentifier(resultIndex))) {
			
			var _self= this;
			var serviceCallback= {
				_success: function(result) {
					_self._searchTokens[queryDescriptor.getIdentifier()]= result.token;

					if (result.totalPlans == 0 && dojo.isFunction(showNoResultFn)) {
						showNoResultFn();
					} else if (result.totalPlans == 1 && dojo.isFunction(showSingleResultFn)) {
						showSingleResultFn(result.results[0]);
					} else {
						var searchResult= _self._getSearchResult(queryDescriptor, result);

						try {
							_self._currentSearchResults= searchResult;
							showResultPageFn(searchResult, resultIndex);
						} finally {
							searchResult.release();
						}
					}
				},
				_error: function(error) {
					_self.remoteError(error);
				}
			}
			
			var searchToken= this._searchTokens[queryDescriptor.getIdentifier()] || null;
			
			var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$ 
			responseHandler.setLabel(MSGS.MONITOR_SEARCH_PLANS, this);		
			
			PlanningClient.searchIterationPlans(responseHandler, queryDescriptor, resultIndex, RESULTPAGE_SIZE, searchToken);
		}
	},

	_showResultPage: function(searchResult, resultIndex) {
		this._view.showSearchResult(searchResult, resultIndex);
	},

	_showSingleResult: function(result) {
		window.location.replace(PlanningUtil.getPlanViewURI(result.itemId));
	},
	
	// ---- search result registry -----------------------------------------------------------------------------------------
	
	_searchResults: null,

	_getSearchResult: function(queryDescriptor, searchResultDTO) {
		var queryId= queryDescriptor.getIdentifier();
		var result= this._searchResults[queryId];
		
		if (result != null && result._token == searchResultDTO.token) {
			result.__refCount++;
			result.addResultPage(searchResultDTO);
		} else {
			result= new SearchResult(searchResultDTO, RESULTPAGE_SIZE, queryDescriptor, this);
			result.__refCount= 1;
			this._searchResults[queryId]= result;
		}
		
		return result;
	},

	_incRefCount: function(searchResult){
		var queryId= searchResult.getQueryDescriptor().getIdentifier();
		Assert.isNotNull(this._searchResults[queryId]);
		++searchResult.__refCount;
	},

	_decRefCount: function(searchResult){
		var queryId= searchResult.getQueryDescriptor().getIdentifier();
		Assert.isNotNull(this._searchResults[queryId]);

		if (--searchResult.__refCount == 0) {
			this._searchResults[queryId]= null;
		}
	},

	__sentinel: null // terminates this class definition
});
})();
