//ScheduledAbsencesEditorPage.js

/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.users.ScheduledAbsencesEditorPage"); //$NON-NLS-1$

/*
 * Imports 
 */
dojo.require("dojox.data.dom"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require("dojox.uuid.generateRandomUuid"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ResourcePlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.users.model.AbsencesModel"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.users.model.Absence"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.viewers.TableViewer"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.users.AbsencesLabelProvider"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.users.AbsencesContentProvider"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.users.AbsencesColumnProvider"); //$NON-NLS-1$

(function() {

var ProfileEditorPage 		= com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage;
var ResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var ResourcePlanningClient	= com.ibm.team.apt.web.client.internal.ResourcePlanningClient;

var AbsencesModel			= com.ibm.team.apt.web.ui.internal.users.model.AbsencesModel;
var Absence					= com.ibm.team.apt.web.ui.internal.users.model.Absence;

var TableViewer				= com.ibm.team.apt.web.ui.internal.viewers.TableViewer;
var AbsencesLabelProvider	= com.ibm.team.apt.web.ui.internal.users.AbsencesLabelProvider;
var AbsencesContentProvider	= com.ibm.team.apt.web.ui.internal.users.AbsencesContentProvider;
var AbsencesColumnProvider	= com.ibm.team.apt.web.ui.internal.users.AbsencesColumnProvider;

var AbsenceEditDialog		= com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialog;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.users.ScheduledAbsencesEditorPage", //$NON-NLS-1$
	[ProfileEditorPage], {

	// Properties //----------------------------------------------------------->

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/templates/ScheduledAbsencesEditorPage.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	/* tbody of absence table */
	__absenceTable: null,
	/* externalized messages for the template */
	__messages: MSGS,
	/* absences table viewer */
	__tableViewer: null,
	/* the absences model */
	__model: null,
	/* src to icon add */
	__addIconSrc: null,
	
	// Public Methods //------------------------------------------------------->
	
	constructor: function(args) {
		this.__addIconSrc= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/images/add.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		this.inherited(arguments);
	
		this.__tableViewer= new TableViewer(TableViewer.constructorArgs(
				["aptAbsenceTable"],  //$NON-NLS-1$
				new AbsencesLabelProvider(), 
				new AbsencesContentProvider(), 
				new AbsencesColumnProvider()
		));
		this.__absenceTable.appendChild(this.__tableViewer.domNode);
		
		this.connect(this.__newAbsenceAction, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
			dojo.stopEvent(event);
			
			var absence= new Absence();
			absence.description= ""; //$NON-NLS-1$
			absence.startDate= new Date();
			absence.endDate= new Date();
			absence.itemId= dojox.uuid.generateRandomUuid();
			absence.isDraft= true;
			
			var _self= this;
			new AbsenceEditDialog({
				absence: absence,
				onOk: function() {
					_self.__model.addAbsence(absence);
				},
				onCancel: function() {
				}
			});
		}));
		
		this.__populateTable(this.itemUUID);
	},
		
	destroy: function() {
		if (this.__tableViewer) {
			this.__tableViewer.destroy();
			this.__tableViewer= null;
		}
		
		this.inherited(arguments);
	},
	
	doSave: function(callback) {
		
		var deletedAbsencesIds= new Array();
		var deletedAbsences= this.__model.getDeletedAbsences();
		if (deletedAbsences.length > 0) {
			for ( var i = 0; i < deletedAbsences.length; i++) {
				deletedAbsencesIds.push(deletedAbsences[i].itemId);
			}
		}
		
		var addedAbsencesDTOs= new Array();
		var addedAbsences= this.__model.getAddedAbsences();
		if (addedAbsences.length > 0) {
			for ( var i = 0; i < addedAbsences.length; i++) {
				var absence= addedAbsences[i];
				addedAbsencesDTOs.push(dojo.toJson({
					description: absence.description,
					startDate: dojo.date.stamp.toISOString(absence.startDate),
					endDate: dojo.date.stamp.toISOString(absence.endDate)
				}));
			}
		}
		
		var modifiedAbsencesDTOs= new Array();
		var modifiedAbsences= this.__model.getModifiedAbsences();
		if (modifiedAbsences.length > 0) {
			for ( var i = 0; i < modifiedAbsences.length; i++) {
				var absence= modifiedAbsences[i];
				modifiedAbsencesDTOs.push(dojo.toJson({
					description: absence.description,
					startDate: dojo.date.stamp.toISOString(absence.startDate),
					endDate: dojo.date.stamp.toISOString(absence.endDate),
					itemId: absence.itemId
				}));
			}
		}
		
		if (deletedAbsencesIds.length == 0 && addedAbsencesDTOs.length == 0 && modifiedAbsencesDTOs.length == 0) {
			callback(this.pageID);
			this.inherited(arguments);
			return;
		}
		
		var context = this;
		var serviceCallback= {
			_success: function(result) {
				callback(context.pageID);
				context.refresh();
			},
			_error: function(error) {
				callback(context.pageID, error);
			}
		};
		
		var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
		ResourcePlanningClient.postAbsences(responseHandler, this.itemUUID, addedAbsencesDTOs, deletedAbsencesIds, modifiedAbsencesDTOs);
		
		this.inherited(arguments);
	},

	refresh: function() {
		this.__populateTable(this.itemUUID);
	},

	// Private Methods //------------------------------------------------------>
	
	__populateTable: function(contributorId) {
		this.__tableViewer.setMessage(MSGS.USER_EDITOR_PENDING);
		dojo.addClass(this.__newAbsenceAction, "disabled"); //$NON-NLS-1$
		
		var _self= this;
		var serviceCallback= {
			_success: function(result) {
				dojo.removeClass(_self.__newAbsenceAction, "disabled"); //$NON-NLS-1$
				
				var absences= new Array();
				if (result) {
					for ( var i = 0; i < result.length; i++) {
						var absence= new Absence();
						
						absence.description= result[i].description;
						absence.startDate= dojo.date.stamp.fromISOString(result[i].startDate)
						absence.endDate= dojo.date.stamp.fromISOString(result[i].endDate)
						absence.itemId= result[i].itemId;
						
						absences.push(absence);
					}
				}

				_self.__model= new AbsencesModel(absences);
				_self.__tableViewer.setComparator(function(abs1, abs2) {
					return abs1.startDate == abs2.startDate ? 0 : abs1.startDate < abs2.startDate ? -1 : 1;
				});
				_self.__tableViewer.setInput(_self.__model);
				
				if (_self.__model.getAbsences().length == 0) {
					_self.__tableViewer.setMessage(MSGS.USER_EDITOR_ABSENCES_NONE);
				}
				_self.__model.addModelListener(function() {
					_self.setDirty(true);
					
					if (_self.__model.getAbsences().length == 0) {
						_self.__tableViewer.setMessage(MSGS.USER_EDITOR_ABSENCES_NONE);
					} else {
						_self.__tableViewer.refresh();
					}
				});
			},
			_error: function(error) {
				_self.tableViewer.setMessage(error.message);
			}
		};
		
		var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$		
		ResourcePlanningClient.fetchAbsences(responseHandler, contributorId);
	},
		
	__sentinel: null // terminates this class definition
	
});

})();
