/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */

dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.StreamBrowse");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit.form.CheckBox");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() {
	var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;
	var checkNonNegative	= net.jazz.ajax.toolbox.lang.checkNonNegative;
	var ScmUtil				= com.ibm.team.scm.web.client.util;
	var ScmClient			= com.ibm.team.scm.web.client.internal.ScmClient;

	/**
	 * Widget to display stream links. Choice of display may be limited to only "My Team Areas".
	 */
	dojo.declare(
		"com.ibm.team.scm.web.ui.internal.view.browse.StreamBrowse",
		[dijit._Widget, dijit._Templated],
		{
			templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/StreamBrowse.html"),
			checkBoxChecked: true, //To store the status of the checkbox (since in IE it is lost when the view is updated)
			
			constructor: function() {
				this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
				this.controller= null;
				this._streamList= [];
				this._myTeamsStreamList = [];
				this._myTeamsSelected = false;
			},
			
			postCreate: function() {
				this._selectedStreamId = "-1";	//No stream currently selected.
				
				//"My Team Areas" filter checkbox event connection.
				var self= this;
				dojo.connect(
					this.myTeamsOnly, 
					"onclick", 
					null,
					function(e) {
						if(e.target.checked) {
							self.checkBoxChecked = true;
							self._showMyTeamsStreamList();
						}
						else {
							self.checkBoxChecked = false;
							self._showStreamList();
						}
					});
				
				var serviceObject = {
					success: function(streams) {
						if(streams && streams.items) {
							dojo.forEach(streams.items, function(x) {
								self.addStream({text: x.workspace.name, id: x.workspace.itemId, href: ScmUtil.getBrowseStreamURI(x.workspace.itemId), inTeamArea: x.inTeamArea});
							});
						}
						self._showMyTeamsStreamList();
					},
				
					failure: function(errorObject) {
						self._displayError(this.messages.StreamBrowse_1);
					}
				};
				
				var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
				
				var contrib = com.ibm.team.repository.web.client.session.getAuthenticatedContributor();
				var ownerItemId = contrib.itemId;
				
				var OWNER_PREFIX = "itemOid/com.ibm.team.repository.Contributor/";
				var PROJECT_AREA_PREFIX = "itemOid/com.ibm.team.process.ProjectArea/";
				//A REST call to retrieve the streams in the logged in contributor's project area.
				ScmClient.getWorkspaces(new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceObject, "success", "failure"), {projectArea: PROJECT_AREA_PREFIX + currentProjectArea.itemId, owner: OWNER_PREFIX + ownerItemId});
			},

			/*
			 * Highlight/Unhighlight streams according to stream selection.
			 */
			refreshSelection: function(streamId) {
				if(this.checkBoxChecked == true) {
					document.getElementById("teamFilter").checked = true;
				}
				
				//If there is a stream previously selected, unhighlight that stream
				if(this._selectedStreamId!="-1") {
					var div = document.getElementById(this._selectedStreamId);
					if (div) {
						div.className = "stream-div-unselected";
					}
				}
				//Highlight the new selected stream
				if(streamId!="-1") {
					var div = document.getElementById(streamId);
					if (div) {
						div.className = "stream-div-selected";
					}
				}
				//Update the selection
				this._selectedStreamId = streamId;
			},
			
			/*
			 * Display an error message if failed to retrieve streams.
			 */
			_displayError: function(message) {
				var div = document.createElement("div");
				div.className = "errorMessage";
				var text= document.createTextNode(message);
				div.appendChild(text);
				this.streamListDiv.appendChild(div);
				this.myTeamsOnly.setAttribute("disabled", "true");
			},
			
			/*
			 * Display that no streams exist in the given project area.
			 */
			addNoStreams: function() {
				var div = document.createElement("div");
				div.className = "emptyStreamList";
				var text = document.createTextNode(this.messages.StreamBrowse_2);
				div.appendChild(text);
				this.streamListDiv.appendChild(div);
			},

			/*
			 * Add a stream <div> for the corresponding stream.
			 */
			addStream: function(stream) {
				this._streamList.push(stream);
				
				if(stream.inTeamArea == true || stream.inTeamArea == "true") {
					this._addToMyTeams(stream);
				}
			},
	
			/*
			 * Add a stream link to the list of streams.
			 */
			_addStreamLink: function (parentDiv, stream) {
				var div= document.createElement("div");
				
				div.className = "stream-div";
				div.id = stream.id;
				var streamText= stream.text;
				var href= stream.href;
				
				var img = document.createElement("img");
				img.src = ScmUtil.getIconSrc("stream");
				img.className = "streamIcon";
	
				var anchor= document.createElement("a");
				anchor.href= (href ? href : "#");
				var text= document.createTextNode(streamText);
				
				anchor.appendChild(text);
				div.appendChild(img);
				div.appendChild(anchor);
				
				parentDiv.appendChild(div);
			},
			
			/*
			 * Add the corresponding stream to the array containing the streams specific to the 
			 * contributor's team Areas.
			 */
			_addToMyTeams: function (stream) {
				this._myTeamsStreamList.push(stream);
			},
			
			/*
			 * Display the list of all streams without filtering only those in the 
			 * contributor's team areas.
			 */
			_showStreamList: function () {
				this._myTeamsSelected = false;
				dojox.data.dom.removeChildren(this.streamListDiv);
				if (this._streamList.length > 0) {
					for (var i = 0; i < this._streamList.length; i++) {
						var streamDiv = document.createElement("div");
						streamDiv.id = this._streamList[i].id;
						if (this._streamList[i].id == this._selectedStreamId) {
							streamDiv.className = "stream-div-selected";
						}
						else {
							streamDiv.className = "stream-div-unselected";
							
						}
						this._streamList[i].div = streamDiv;
						this._addStreamLink(streamDiv, this._streamList[i]);
						this.streamListDiv.appendChild(streamDiv);
					}
				}
				else {
					this.addNoStreams();
				}
			},
			
			/*
			 * Display the list of streams in the contributor's team areas.
			 */
			_showMyTeamsStreamList: function () {
				this._myTeamsSelected = true;
				dojox.data.dom.removeChildren(this.streamListDiv);
				if (this._myTeamsStreamList.length > 0) {
					for (var i = 0; i < this._myTeamsStreamList.length; i++) {
						var streamDiv = document.createElement("div");
						streamDiv.id = this._myTeamsStreamList[i].id;
						if (this._myTeamsStreamList[i].id == this._selectedStreamId) {
							streamDiv.className = "stream-div-selected";
						}
						else {
							streamDiv.className = "stream-div-unselected";
							
						}
						this._myTeamsStreamList[i].div = streamDiv;
						this._addStreamLink(streamDiv, this._myTeamsStreamList[i]);
						this.streamListDiv.appendChild(streamDiv);
					}
				}
				else {
					var div = document.createElement("div");
					div.className = "emptyStreamList";
					var text = document.createTextNode(this.messages.StreamBrowse_3);
					div.appendChild(text);
					this.streamListDiv.appendChild(div);
				}
			}
	}
);

})();
