/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewer"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDSource"); //$NON-NLS-1$

dojo.require("jazz.util.ResizeNotifier"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$

(function() {

var Assert					= com.ibm.team.apt.web.ui.internal.common.Assert;
var PlanItemDnDSource		= com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDSource;

var TAG_UNFILTERED_CHILDREN	= "unfilteredChildren"; //$NON-NLS-1$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewer", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_createViewerItem: null,
	
	_table: null,
	_tableBody: null,
	_nodes: null,
	_dndSource: null,
	_tabPlannedItems: null,

	__viewModel: null,

	// ---- api ------------------------------------------------------------------------------------------------------------

	setInput: function(viewModel) {
		this.__viewModel= viewModel;
		this.__rebuildTree();
	},

	refresh: function(viewEntry) {
		if (viewEntry && viewEntry._getParent()) {
			var expandedItems= {};

			var oldViewerItem= this.__getViewerItem(viewEntry);
			if (oldViewerItem) {
				// dispose the item and all of its children
				var toDispose= oldViewerItem._getChildViewerItems(false);
				toDispose.unshift(oldViewerItem);
				
				dojo.forEach(toDispose, function(viewerItem) {
					if (viewerItem.isExpanded()) {
						var path= viewerItem.getViewEntry().getTreePath();
						expandedItems[this.__convertTreePathToId(path)]= true;
					}

					viewerItem._dispose();
					delete this._nodes[viewerItem._getNodeId()];
				}, this);
			}

			var parentViewerItem= this.__getViewerItem(viewEntry._getParent());
			if (parentViewerItem) {
				var insertBefore= this.__findInsertionPosition(parentViewerItem, viewEntry);
				this.__addEntry(viewEntry, parentViewerItem, expandedItems, insertBefore);
			}
		} else {
			this.__rebuildTree();
		}
	},
	
	add: function(parentViewEntry, viewEntry) {
		var parentViewerItem= this.__getViewerItem(parentViewEntry);
		if (parentViewerItem && !parentViewerItem._isLazyChildCreationPending()) {
			var insertBefore= this.__findInsertionPosition(parentViewerItem, viewEntry);
			this.__addEntry(viewEntry, parentViewerItem, {}, insertBefore);
		}
	},

	remove: function(viewEntry) {
		var viewerItem= this.__getViewerItem(viewEntry);
		if (viewerItem) {
			var toRemove= viewerItem._getChildViewerItems(false);
			toRemove.unshift(viewerItem);
			
			dojo.forEach(toRemove, function(viewerItem) {
				viewerItem._dispose();
				delete this._nodes[viewerItem._getNodeId()];
			}, this);
		}
	},

	update: function(viewEntry) {
		var viewerItem= this.__getViewerItem(viewEntry);
		if (viewerItem) {
			viewerItem._updateContent();
		}
	},

	resort: function(viewEntry) {
		var viewerItem= this.__getViewerItem(viewEntry);
		if (viewerItem && !viewerItem._isLazyChildCreationPending()) {
			// this ViewerEntry and its children have already been created (-> this entry
			// was / is expanded and its children were visible)

			// we make use of the fact that during a resort no new ViewEntries are created, just the sort order
			// of the existing entries is updated. therefore we can simply move the existing ViewModelViewerEntry's DOM
			// node to the new position.

			// this leads to the assertion that all childEntries must be present in the viewer, 
			// otherwise the viewModel would have issued another command instead of a resort.

			// to achieve the actual resort, we iterate over the childEntries, and move all elements, whose sort 
			// order has changed (nodes that do not match currentNode), to the current position. we advance the 
			// current position to skip elements whose sort order has not changed.

			// when moving or skipping elements, we must always move the childEntries children iff they
			// belong to the same container

			var currentNode= viewerItem._getFirstChildNode();
			if (currentNode === undefined)
				currentNode= this._tableBody.firstChild;
			
			Assert.isNotNull(currentNode, "The resort request is invalid as the children are not yet created"); //$NON-NLS-1$
			var sortContainer= currentNode.parentNode;
			
			// the entries fetched from the ViewModel represent the new sort order
			var childEntries= this.__getChildren(viewEntry);
			
			dojo.forEach(childEntries, function(childEntry) {
				var childViewerItem= this.__getViewerItem(childEntry);
				
				//defect 85049: A draft item might not yet be part of the viewer
				if (!childViewerItem)
					return;
				
				if (currentNode != childViewerItem.domNode) {
					// sort order change detected. move this childViewerItem and all its 
					// children in front of currentNode
					
					var entriesToMove= childViewerItem._getChildViewerItems(false);
					entriesToMove.unshift(childViewerItem);
					
					dojo.forEach(entriesToMove, function(entryToMove) {
						if (sortContainer == entryToMove.domNode.parentNode) {
							
							var editorNode= null;
							if (entryToMove instanceof com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItem) {
								if (entryToMove.isEditorOpen()) {
									editorNode= entryToMove.domNode.nextSibling;		
								}
							}
							
							sortContainer.removeChild(entryToMove.domNode);
							sortContainer.insertBefore(entryToMove.domNode, currentNode);
							
							if (editorNode != null) {
								sortContainer.removeChild(editorNode);
								sortContainer.insertBefore(editorNode, currentNode);	
							}
						}
					}, this);
				} else {
					// the childEntry is still in the correct position in the viewer
					// -> skip this childViewerItem and its children

					var entriesToSkip= childViewerItem._getChildViewerItems(false);
					entriesToSkip.unshift(childViewerItem);
					
					dojo.forEach(entriesToSkip, function(entryToSkip) {
						if (sortContainer == entryToSkip.domNode.parentNode) {
							// only skip children that were created in the same container
							currentNode= currentNode.nextSibling;
							if (entryToSkip instanceof com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItem) {
								if (entryToSkip.isEditorOpen()) {
									//skip over the open editor
									currentNode= currentNode.nextSibling;		
								}
							}
							
							// currentNode might not be null unless it is the very last element.
							// therefore we need no null check
//							Assert.isNotNull(currentNode, "All created child entries must be present in the viewer"); //$NON-NLS-1$
						}
					}, this);
				}
			}, this);
		}
	},
	
	enableEditMode: function(enabled) {
		if (enabled) {
			dojo.removeClass(this._table, "viewMode"); //$NON-NLS-1$
			dojo.addClass(this._table, "editMode"); //$NON-NLS-1$
		} else {
			dojo.addClass(this._table, "viewMode"); //$NON-NLS-1$
			dojo.removeClass(this._table, "editMode"); //$NON-NLS-1$
		}
	},
	
	setExpandedState: function(entry, expanded) {
		var viewerItem= this.__getViewerItem(entry);
		if (viewerItem) {
			viewerItem.setExpansionState(expanded);
		}
	},
	
	setSelectionState: function(entry, selected) {
		var viewerItem= this.__getViewerItem(entry);
		if (viewerItem) {
			viewerItem.setSelectionState(selected);
		}
	},
	
	getSelectedEntries: function() {
		if (!this._nodes)
			return [];
		
		var result= new Array();
		for ( var node in this._nodes) {
			var viewerItem= this._nodes[node];
			if (viewerItem.isSelected()) {
				result.push(viewerItem.getViewEntry());
			}
		}
	
		return result;
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: "<table class=\"plannedItems\" dojoAttachPoint=\"_table\"></table>", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	postCreate: function() {
		dojo.addClass(this._table, "viewMode"); //$NON-NLS-1$
		if (dojo.isIE) {
			dojo.addClass(this._table, "isIE"); //$NON-NLS-1$
		} else {
			dojo.addClass(this._table, "isNotIE"); //$NON-NLS-1$
		}
	},
	
	destroy: function() {
		if (this._nodes != null) {
			this.__removeChildren();
		}
		
		if (this._dndSource != null) {
			this._dndSource.destroy();
			this._dndSource= null;
		}
	
		this._table= null;
		this.__viewModel= null;			

		this.inherited("destroy", arguments); //$NON-NLS-1$
	},
	
	// ---- SPI ------------------
	
	_insertBefore: function(viewerItem, anchor) {
		if (this.__isDraggableItem(viewerItem)) {
			this._dndSource.insertNodes(false, [viewerItem], true, anchor);
		} else {
			this._tableBody.insertBefore(viewerItem.domNode, anchor);
		}
	},
	
	_appendChild: function(viewerItem) {
		if (this.__isDraggableItem(viewerItem)) {
			this._dndSource.insertNodes(false, [viewerItem]);
		} else {
			this._tableBody.appendChild(viewerItem.domNode);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__isDraggableItem: function(viewerItem) {
		if (!(viewerItem instanceof com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItem))
			return false;
		
		return true;
	},

	__getViewerItem: function(viewEntry) {
		return this._nodes && this._nodes[viewEntry._getTreeNodeId()];
	},
	
	__removeChildren: function() {
		if (this._nodes != null) {
			for ( var node in this._nodes) {
				this._nodes[node]._dispose();
			}
			this._nodes= null;
		}

		dojox.data.dom.removeChildren(this._table);
	},
	
	__rebuildTree: function() {
		var expandedItems= {};
		
		if (this._nodes != null) {
			for ( var node in this._nodes) {
				var viewerItem= this._nodes[node];
				if (viewerItem.isExpanded()) {
					var path= viewerItem.getViewEntry().getTreePath();
					expandedItems[this.__convertTreePathToId(path)]= true;
				}
			}
			
			this.__removeChildren();
		}
		
		if (this._dndSource != null) {
			this._dndSource.destroy();
			this._dndSource= null;
		}

		this._nodes= new Object();
		
		dojox.data.dom.removeChildren(this._table);
		this._tableBody= null;
		
		if (this.__viewModel != null) {
			var modelTransformer=this.__viewModel.getModelTransformer();
			if (modelTransformer != null) {
				var elementsToExpandInitially= modelTransformer.getElementsToExpandInitially();

				// see task 80845
				var loggedInUserId= com.ibm.team.repository.web.client.session.getAuthenticatedContributor().itemId;

				var expandRequestCount= 0;
				var minimalElementsToExpandInitially= [];
				
				var PlanItem= com.ibm.team.apt.client.PlanItem;
				var RowElement= com.ibm.team.apt.ui.structure.RowElement;
				var ColumnElement= com.ibm.team.apt.ui.structure.ColumnElement;
				
				this.__viewModel.readModel(function(readAccessor) {
					for ( var i = 0; i < elementsToExpandInitially.length; i++) {
						dojo.forEach(readAccessor.getChildren(elementsToExpandInitially[i]), function(childEntry) {
							if (childEntry.getElement() instanceof RowElement) {
								// see defect 81537, we must count the plan items as row elements and column elements are expanded automatically
								readAccessor.accept(function(entry) {
									var element= entry.getElement();
									
									if (element instanceof PlanItem)
										expandRequestCount++;

									return element instanceof RowElement || element instanceof ColumnElement;
								}, childEntry, this);
							} else {
								expandRequestCount++;
							}
						}, this);

						var element= elementsToExpandInitially[i].getElement();
						if (element instanceof com.ibm.team.apt.ui.structure.OwnerElement) {
							// Hack to not force to collapsing of the owner group for the logged in user,
							// even on slow browsers such as IE
							if (element.getId() == loggedInUserId)
								minimalElementsToExpandInitially.push(elementsToExpandInitially[i]);
						}
					}
				}, this);

				if ((dojo.isIE && expandRequestCount > 50) || expandRequestCount > 500) {
					elementsToExpandInitially= minimalElementsToExpandInitially;
				}

				for ( var i = 0; i < elementsToExpandInitially.length; i++) {
					path= elementsToExpandInitially[i].getTreePath();
					expandedItems[this.__convertTreePathToId(path)]= true;
				}
			}
			this._tableBody= document.createElement("tbody"); //$NON-NLS-1$
		
			this._dndSource= new PlanItemDnDSource(this._table, PlanItemDnDSource.constructorArgs(null, this._tabPlannedItems.getItemMovePolicy(), this._tabPlannedItems, this._tableBody));
			
			this.__viewModel.readModel(function(readAccessor) {
				var rootEntry= readAccessor.getRootEntry();
				if (rootEntry != null) {
					this.__addEntry(readAccessor.getRootEntry(), null, expandedItems);
				}
			}, this);
			this._table.appendChild(this._tableBody);
		}
		
		jazz.util.ResizeNotifier.poke();
	},

	__addEntry: function(entry, parentItem, expandedItems, insertBefore) {
		var viewerItem= this._createViewerItem(entry, parentItem, this);
		
		viewerItem._createNode();
		this._nodes[viewerItem._entry._getTreeNodeId()]= viewerItem;

		if (!entry.isRootEntry()) {
			parentItem._insertChildItem(viewerItem, insertBefore);
		}

		var isExpanded= entry.isRootEntry() || viewerItem.isExpanded();
		if (!isExpanded)
			isExpanded= expandedItems[this.__convertTreePathToId(entry.getTreePath())] == true;
		
		
		dojo.toggleClass(viewerItem.domNode, "hidden", parentItem != null && !parentItem.isExpanded()); //$NON-NLS-1$
		dojo.toggleClass(viewerItem.domNode, "expanded", isExpanded); //$NON-NLS-1$
		dojo.toggleClass(viewerItem.domNode, "collapsed", !isExpanded); //$NON-NLS-1$

		
		var children= this.__getChildren(entry);
		if (children.length > 0) {			
			if (isExpanded) {
				// while refreshing, we create all nodes directly
				
				var successorNode;
				if (entry.getElement() instanceof com.ibm.team.apt.ui.structure.RowElement) {
					successorNode= null;
				} else {
					successorNode= viewerItem.domNode.nextSibling;
				}
				
				for (var i= 0; i < children.length; i++) {
					successorNode= this.__addEntry(children[i], viewerItem, expandedItems, successorNode);
				}
				
				if (entry.getElement() instanceof com.ibm.team.apt.ui.structure.RowElement) {
					return viewerItem.domNode.nextSibling;
				} else {
					return successorNode;
				}
			} else {
				var nodeCreator= dojo.hitch(this, function(entriesToCreate, yieldStartIndex, successorNode) {
					var yieldEndIndex= Math.min(entriesToCreate.length, yieldStartIndex + 10);
					for (var y= yieldStartIndex; y < yieldEndIndex; y++) {
							successorNode= this.__addEntry(entriesToCreate[y], viewerItem, expandedItems, successorNode);
					}
					
					if (entriesToCreate.length > yieldEndIndex) {
						window.setTimeout(function() { nodeCreator(entriesToCreate, yieldEndIndex, successorNode); }, 0);
					}
				});
					
				viewerItem._setLazyChildCreator(dojo.hitch(this, function() {
					nodeCreator(this.__getChildren(entry), 0, viewerItem.domNode.nextSibling);
				}));
			}
		}
		
		return viewerItem.domNode.nextSibling;
	},
	
	__convertTreePathToId: function(path) {
		var result= ""; //$NON-NLS-1$
		for ( var i = 0; i < path.length; i++) {
			result+= path[i];
		}
		return result;
	},
	
	__getElements: function() {
		return this.__viewModel.readModel(function(readAccessor) {
			var rootEntry= readAccessor.getRootEntry();
			return rootEntry != null ? this.__getVisibleChildren(rootEntry, readAccessor) : [];
		}, this);
	},

	__getChildren: function(parentEntry) {
		return this.__viewModel.readModel(function(readAccessor) {
			return this.__getVisibleChildren(parentEntry, readAccessor);
		}, this);
	},

	__getParent: function(entry) {
		return this.__viewModel.readModel(function(readAccessor) {
			return !entry.isDisposed() ? entry._getParent() : null;
		}, this);
	},

	__hasChildren: function(entry) {
		return this.__viewModel.readModel(function(readAccessor) {
			return this.__getVisibleChildren(entry, readAccessor).length > 0;
		}, this);
	},

	__getVisibleChildren: function(entry, readAccessor) {
		var children= readAccessor.getChildren(entry);

		var result= [];
		for (var i= 0; i < children.length; i++) {
			if (children[i].isVisible()) {
				result.push(children[i]);
			}
		}
		return result;
	},
	
	__findInsertionPosition: function(parentViewerItem, viewEntry) {
		var parentViewEntry= parentViewerItem.getViewEntry();
		var siblingEntries= this.__getChildren(parentViewEntry);
		var index= dojo.indexOf(siblingEntries, viewEntry);
		Assert.isTrue(index != -1, "viewEntry not included in parentViewEntry's children"); //$NON-NLS-1$

		// root entry returns undefined, normal viewer entries return next sibling dom node, taskboard overrides
		var result= parentViewerItem._getFirstChildNode();
		if (result === undefined)
			result= this._tableBody.firstChild;
		
		if (index > 0) {
			// predecessor entries must have been created by now
			Assert.isNotNull(result, "ViewModelView not correctly updated"); //$NON-NLS-1$

			var predecessorViewerItem= this.__getViewerItem(siblingEntries[index - 1]);
			
			var predecessorNodes= predecessorViewerItem._getChildViewerItems(false);
			predecessorNodes.unshift(predecessorViewerItem);
			
			// in a regular table based tree layout, we insert the new TR DOM node after the last child element's TR.
			// however, in the taskboard, the last can be a nested DOM element -> we therefore look for the last node 
			// with the same parent DOM node as the predecessorViewerItem 
			var predecessorViewerItemParentNode= predecessorViewerItem.domNode.parentNode;
			for (var i= predecessorNodes.length - 1; i >= 0; i--) {
				if (predecessorNodes[i].domNode.parentNode == predecessorViewerItemParentNode) {
					result= predecessorNodes[i].domNode.nextSibling;
					break;
				}
			}
		}
		
		return result;
	},


	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewer.constructorArgs= function(tabPlannedItems, createViewerItemFn) {
	return {
		_tabPlannedItems: tabPlannedItems,
		_createViewerItem: createViewerItemFn
	};
};

})();
