/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDTarget"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.apt.web.ui.internal.common.Assert;
var ViewModelViewerItemActionBar	= com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar;
var PlanItemDnDTarget				= com.ibm.team.apt.web.ui.internal.view.plan.dnd.PlanItemDnDTarget;

var doc= document;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem",  null, { //$NON-NLS-1$

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------
	
	_viewer: null,
	_entry: null,
	__treeLevel: -1,
	_plannedItemsTab: null,
	
	_parentItem: null,
	_children: null,

	_listeners: null,
	_expansionListeners: null,
	
	_selected: false,
	_isExpansionInProgress: false,
	_lazyChildCreator: null,
	
	__actionBar: null,

	constructor: function(entry, parentItem, plannedItemsTab, viewer) {
		this._viewer= viewer;
		this._entry= entry;
		this._plannedItemsTab= plannedItemsTab;
		this._parentItem= parentItem;

		this._children= [];
		this._listeners= [];
		this._expansionListeners= [];

		if (parentItem != null) {
			parentItem.__addChild(this);
		}
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	domNode: null,

	expand: function() {
		this._setExpansionState(true);
	},

	collapse: function() {
		this._setExpansionState(false);
	},
	
	isExpanded: function() {
		// a root node is always expanded
		return this._parentItem == null || dojo.hasClass(this.domNode, "expanded"); //$NON-NLS-1$
	},
	
	toggleExpansionState: function() {
		this._setExpansionState(!this.isExpanded());
	},

	setExpansionState: function(doExpand) {
		this._setExpansionState(doExpand);
	},
	
	setSelectionState: function(doSelect) {
		this._selected= doSelect;

		if (this.__actionBar) {
			this.__actionBar.setSelected(doSelect);
		}
	},
	
	isSelected: function() {
		return this._selected;
	},
	
	setHidden: function(hide) {
		if (hide) {
			dojo.addClass(this.domNode, "hidden"); //$NON-NLS-1$
		} else {
			dojo.removeClass(this.domNode, "hidden"); //$NON-NLS-1$
		}
	},

	// ---- questionable api -----------------------------------------------------------------------------------------------
	
	getViewModel: function() {
		return this._entry.getModel();
	},
	
	getViewEntry: function() {
		return this._entry;
	},

	getViewer: function() {
		return this._viewer;
	},
	
	// ---- ViewModelViewer API --------------------------------------------------------------------------------------------

	_setLazyChildCreator: function(listener) {
		this._lazyChildCreator= listener;
		if (dojo.hasClass(this.domNode, "nochildren")) { //$NON-NLS-1$
			dojo.removeClass(this.domNode, "nochildren"); //$NON-NLS-1$
			dojo.addClass(this.domNode, "collapsed"); //$NON-NLS-1$
		}
	},
	
	_isLazyChildCreationPending: function() {
		return this._lazyChildCreator != null;
	},

	_updateContent: function() {
		dojox.data.dom.removeChildren(this.domNode);
		
		this._createContent();
	},

	_getNodeId: function() {
		return this._entry._getTreeNodeId();
	},

	_createNode: function() {
		this.domNode= this._createContainer();
		
		if (!this._dndTarget) {
			this._dndTarget= this._createDropTarget();
		}

		this._createContent();
		return this.domNode;
	},
	
	_dispose: function() {
		if (this.domNode && this.domNode.parentNode) {
			this.domNode.parentNode.removeChild(this.domNode);
		}

		for ( var i= 0; i < this._listeners.length; i++) {
			dojo.disconnect(this._listeners[i]);
		}

		if (this._parentItem != null) {
			this._parentItem.__removeChild(this);
			this._parentItem= null;
		}
		
		if (this.__actionBar) {
			this.__actionBar.destroy();
			this.__actionBar= null;
		}

		if (this._dndTarget != null) {
			this._dndTarget.destroy();
			this._dndTarget= null;
		}
		
		if (this.isSelected()) {
			this.getViewModel().setEntrySelectionState(this._entry, false);
		}
	},
	
	_insertChildItem: function(viewerItem, insertBefore) {
		if (insertBefore) {
			this._viewer._insertBefore(viewerItem, insertBefore);
		} else {
			this._viewer._appendChild(viewerItem);
		}
	},
	
	_getFirstChildNode: function() {
		return !this._entry.isRootEntry() ? this.domNode.nextSibling : undefined;
	},

	// ---- ViewModelViewItem SPI ------------------------------------------------------------------------------------------
	
	_createContainer: function() {
		return doc.createElement("tr"); //$NON-NLS-1$
	},
	
	_createContent: function() {
	},

	_createDropTarget: function() {
		return new PlanItemDnDTarget(this.domNode, PlanItemDnDTarget.constructorArgs(this, this._plannedItemsTab.getItemMovePolicy(), this._plannedItemsTab, this.domNode));
	},
	
	_createActionBar: function() {
		this.__actionBar= new ViewModelViewerItemActionBar(ViewModelViewerItemActionBar.constructorArgs(this, this.getViewModel(), this._plannedItemsTab.getItemMovePolicy(), this._entry));
		return this.__actionBar;
	},

	_getTreeLayoutElement: function() {
		if (this.domNode.childNodes.length >= 2 && this.domNode.childNodes[1].tagName == "TD") //$NON-NLS-1$
			return this.domNode.childNodes[1];

		return null;
	},
	
	_getChildViewerItems: function(directChildrenOnly) {
		if (directChildrenOnly)
			return this._children;
		
		var result= [];
		this.__getChildViewerItemsRecursive(result);
		return result;
	},

	_getTreeLevel: function() {
		return this._parentItem != null ? this._parentItem._getChildTreeLevel(this._constructor) : 0;
	},

	_getChildTreeLevel: function(type) {
		return this._constructor == type ? this._getTreeLevel() + 1 : this._getTreeLevel();
	},

	// ---- subclass API ---------------------------------------------------------------------------------------------------
	
	_connect: function(element, event, fn) {
		var result= dojo.connect(element, event, this, fn);
		this._listeners.push(result);
		return result;
	},

	_disconnect: function(handle) {
		var handleIndex= dojo.indexOf(this._listeners, handle);
		if (handleIndex != -1)
			this._listeners.splice(handleIndex, 1);
		
		dojo.disconnect(handle);
	},
	
	_applyTreeLayout: function(expansionElement) {
		var firstCell= this._getTreeLayoutElement();

		dojo.style(firstCell, "paddingLeft",  (4 + this._getTreeLevel() * 16) + "px"); //$NON-NLS-1$ //$NON-NLS-2$

		dojo.toggleClass(this.domNode, "nochildren", this._lazyChildCreator == null && this._children.length == 0); //$NON-NLS-1$
		this.setHidden(this._parentItem != null && !this._parentItem.isExpanded()); //$NON-NLS-1$

		var twisty= doc.createElement("span"); //$NON-NLS-1$
		dojo.addClass(twisty, "twisty"); //$NON-NLS-1$
		twisty.tabIndex=0;
		firstCell.insertBefore(twisty, firstCell.firstChild);
		twisty.title = "+/-"; //$NON-NLS-1$

		var innerTwisty= doc.createElement("span"); //$NON-NLS-1$
		dojo.addClass(innerTwisty, "inner"); //$NON-NLS-1$
		twisty.appendChild(innerTwisty);


		if (!expansionElement)
			expansionElement= twisty;
		
		dojo.forEach(this._expansionListeners, function(handle) {
			this._disconnect(handle);
		}, this);
		this._expansionListeners= [];
		
		this._expansionListeners.push(this._connect(expansionElement, "onclick", function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			this.toggleExpansionState();
		})); 
		this._expansionListeners.push(this._connect(expansionElement, "onkeypress", function(e) { //$NON-NLS-1$
			var characterCode = e.keyCode;
			if(characterCode == dojo.keys.ENTER) { 
				this.toggleExpansionState();
			}	
		})); 
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_setExpansionState: function(doExpand) {
		if (!this._isExpansionInProgress) {
			this._isExpansionInProgress= true;
			
			if (doExpand && this._lazyChildCreator != null) {
				this._lazyChildCreator();
				this._lazyChildCreator= null;
			}

			dojo.toggleClass(this.domNode, "expanded", doExpand); //$NON-NLS-1$
			dojo.toggleClass(this.domNode, "collapsed", !doExpand); //$NON-NLS-1$

			var nodeToggler= dojo.hitch(this, function(nodesToToggle, yieldStartIndex) {
				var yieldEndIndex= doExpand ? Math.min(nodesToToggle.length, yieldStartIndex + 20) : nodesToToggle.length;
				
				for (var i= yieldStartIndex; i < yieldEndIndex; i++) {
					var node= nodesToToggle[i].domNode;
					if (doExpand) {
						nodesToToggle[i].setHidden(false);
						if (nodesToToggle[i].isExpanded()) {
							nodeToggler(nodesToToggle[i]._getChildViewerItems(true), 0);
						}
					} else {
						nodesToToggle[i].setHidden(true);
						// define the state of all child tree nodes as collapsed
						dojo.removeClass(node, "expanded"); //$NON-NLS-1$
						dojo.addClass(node, "collapsed"); //$NON-NLS-1$
					}
				}

				if (nodesToToggle.length > yieldEndIndex) {
					window.setTimeout(function() { nodeToggler(nodesToToggle, yieldEndIndex); }, 0);
				} else {
					this._isExpansionInProgress= false;
				}
			});
			
			nodeToggler(this._getChildViewerItems(doExpand), 0);
		}
	},

	__getChildViewerItemsRecursive: function(list) {
		for ( var i= 0; i < this._children.length; i++) {
			list.push(this._children[i]);
			this._children[i].__getChildViewerItemsRecursive(list);
			
		}
	},

	__addChild: function(childTreeItem) {
		this._children.push(childTreeItem);

		if (dojo.hasClass(this.domNode, "nochildren")) { //$NON-NLS-1$
			dojo.removeClass(this.domNode, "nochildren"); //$NON-NLS-1$
			dojo.addClass(this.domNode, "collapsed"); //$NON-NLS-1$
		}
	},
	
	__removeChild: function(childTreeItem) {
		var childIndex= dojo.indexOf(this._children, childTreeItem);
		
		if (childIndex != -1) {
			this._children.splice(childIndex, 1);
			if (this._children.length == 0) {
				dojo.addClass(this.domNode, "nochildren"); //$NON-NLS-1$
				dojo.removeClass(this.domNode, "collapsed"); //$NON-NLS-1$
			}
		}
	},
	

	__sentinel: null // terminates this class definition
});

})();
