/******************************************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.Dialog");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {
	
var KEY_RETURN			= 13;
var KEY_ENTER			= 14;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog", [dijit._Widget, dijit._Templated], {
	constructor: function(params) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		if (!dojo.isFunction(params.onOk)) throw new Error("Missing function for 'onOk' parameter");
	},
	postCreate: function() {
		this._initContentWidget();
		this._initFloatingPane();
	},
	_onClose: function() {
		this.floatingPane.close();
	},
	_initContentWidget: function() {
		var self = this;
		var onOk = function(parms /* isMove, projectAreaItemId */) {
			self.onOk(parms);
			dojo.hitch(self,self._onClose)();
		};
		var onCancel = dojo.hitch(this,this._onClose);
		var contentParams = {
			projectArea: this.projectArea,
			onOk: onOk,
			onCancel: onCancel,
			parentController: this.parentController
		};
		var content = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyContent(contentParams);
		this.dialogContent = content;
	},
	_initFloatingPane: function() {
		var floatingPaneParams = {
			width: "475px",
			onClose: this.onClose,
			contentNode: this.dialogContent.domNode
		};
		if (this.dialogTitle) {
			floatingPaneParams.primaryTitle = this.dialogTitle;
			floatingPaneParams.secondaryTitle = "";
		} else {
			floatingPaneParams.primaryTitle = dojo.string.substitute(this.messages.dialogTitleSelectMoveCopy, [this.parentController.workingCopy.getLinkLabel()]);
		}
		this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
		if(this.dialogContent) this.dialogContent.setFocus();
	},
	onClose: function () {
		
	}
});


/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyContent",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/SelectMoveCopyContent.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/SelectWorkItemContent.css"),
		constructor: function(params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.moveLabel = this.messages.labelMove;
			this.copyLabel = this.messages.labelCopy;
			this.buttonOK = this.messages.buttonOK;
			this.buttonCancel = this.messages.buttonCancel;
		},
		postCreate: function() {
			var nameValueLayout = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout();
			nameValueLayout.addRow( this.messages.labelMoveCopy, this.radioButtonSet);
			nameValueLayout.addRow(this.messages.labelProjectArea, this.projectAreaSelect);
			
			this._initProjectAreaSelector();
			this.projectNameValueSection.appendChild(nameValueLayout.domNode);
			this._connectEvents();
		},
		setFocus: function() {
			
		},
		reset: function() {
//			this.radioMove.checked = true;
//			this.radioCopy.checked = false;
			if(this.projectAreaSelect) this.projectAreaSelect.value= "";
			this.okCmd.disabled= false;
		},
		setKey: function(key) {
			this.key= key;
		},		
		_connectEvents: function() {
			this.connect(this.okCmd, "onclick", "_handleOk");
			this.connect(this.cancelCmd, "onclick",  "_handleCancel");
			this.connect(this.radioMove, "onchange",  "_handleChange");
			this.connect(this.radioCopy, "onchange",  "_handleChange");
			this.connect(this.projectAreaSelect, "onchange",  "_handleChange");
		},
		_initProjectAreaSelector: function() {
			this._currentProjectArea = this.projectArea ||  com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			this._currentProjectArea.itemId = this._currentProjectArea.itemId || this._currentProjectArea.id;
			this._projectSelect = document.createElement("select");

			this.projectAreaSelectDiv.appendChild(this._projectSelect);
			
			var srh= new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_setProjectAreaOptions", "_handleError");
			com.ibm.team.workitem.web.client.internal.WorkItemClient.getProcessAreaHierarchies(srh);
		},
		_selectMove: function(e) {
			this.radioMove.checked = true;
			this.radioCopy.checked = false;
		},
		_selectCopy: function(e) {
			this.radioCopy.checked = true;
			this.radioMove.checked = false;
		},
		_handleError: function(e) {
			console.debug("Error loading project areas. Option ignored."); // $NON-NLS-1
		},
		_setProjectAreaOptions: function(projectAreaTree) {
			var projectAreas = [];
			this._collectProcessAreas(projectAreaTree, projectAreas, "", false, false);
			if(projectAreas.length < 2) { // should never happen, be disabled by caller
			  	alert(this.messages.errorOnlyOneProjectArea);
				this._handleCancel();
				return;
			}
			projectAreas.sort(function(a, b){
				try{
					return a.label.toLowerCase() > b.label.toLowerCase() ? 1 : -1;
				} catch(e) { 
					return -1 
				}
			});
			for(var i=0;i<projectAreas.length;i++) {
				if (projectAreas[i].id != this._currentProjectArea.itemId) {
					if (!this.projectAreaItemId || this.projectAreaItemId == projectAreas[i].id) {
						com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil._addOption(this._projectSelect, new Option(projectAreas[i].label, projectAreas[i].id));
					}	
				}
			}
			
			if(this._projectSelect.childNodes.length === 0) this._projectSelect.style.visibility = "hidden";
			else if(this._projectSelect.childNodes.length === 1) this._projectSelect.disabled = true;
		},
		_collectProcessAreas: function(processAreas, collector, indent, includeChildren, disable){
			if (processAreas) {
				for (var i = 0; i < processAreas.length; i++) {
					var team = processAreas[i];
					var item = {
						id: team.itemId,
						label: indent + team.name,
						disabled: disable
					};
					collector.push(item);		
					if (includeChildren) 
						this._collectProcessAreas(team.children, collector, indent + "   ", true, false);
				}
			}
		},
		_handleChange: function(e) {
//			if(e.keyCode === KEY_ENTER || e.keyCode === KEY_RETURN ) {// enter
		},
		
		_handleOk: function() {
			this.onOk({
				isMove: this.radioMove.checked,
				projectAreaItemId: this._projectSelect.value
			});
			this.reset();			
		},
		
		_handleCancel: function() {
			this.onCancel();
			this.reset();
		}
	}
);

})();