/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItem"); //$NON-NLS-1$

dojo.require("dojo.fx"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Columns"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItemColumn"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItemDurationColumn"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory"); //$NON-NLS-1$

(function() {

var PlanningClient			= com.ibm.team.apt.web.client.internal.PlanningClient;
var PlanningUtil			= com.ibm.team.apt.web.client.util;
var ViewModelViewerItem		= com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem;
var InlineWorkItemEditor	= com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Columns					= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Columns;
var PlanItemViewerItemColumn= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItemColumn;
var PlanItemViewerItemDurationColumn= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItemDurationColumn;
var WorkItemProxyFactory	= com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory;

var colorInfo= new dojo.colorFromHex("#F2F2F2"); //$NON-NLS-1$
var colorBackground=  dojo.colorFromArray(dojo.Color.named.white);

var doc= document;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var bind= dojo.string.substitute;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItem", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItem], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	__planItem: null,
	__editor: null,
	
	__columns: null,
	__treeLayoutElement: null,
	
	__summaryCell: null,
	
	__colorInfo: null,
	
	constructor: function(entry, parentItem, plannedItemsTab, viewer) {
		this.__planItem= entry.getElement();
	},


	// ---- api ------------------------------------------------------------------------------------------------------------	

	getPlanItem: function() {
		return this.__planItem;
	},
	
	isPrimaryItem: function() {
		return this._entry.hasTag(com.ibm.team.apt.ui.model.PrimaryLocationTag.INSTANCE);
	},
	
	getOutplaceReasons: function() {
		if (this.__planItem.isAuxiliaryPlanItem()) {
			return this.__planItem.getAuxiliaryPlanItemReasons();
		}

		if (!this.isPrimaryItem()) {
			var locationMarker= this._entry.getModel().getModelTransformer().calculateLocationMarker(this._entry);
			if (locationMarker != null) 
				return [locationMarker.getLocationMessage()];
		}
		
		return null;
	},
	
	startSpinning: function() {
		dojo.toggleClass(this.domNode, "spinning", true); //$NON-NLS-1$
	},
	
	stopSpinning: function() {
		dojo.toggleClass(this.domNode, "spinning", false); //$NON-NLS-1$
	},
	
	isEditorOpen: function() {
		return this.__editor != null;
	},
	
	toggleEdit: function() {
		if (this.__editor != null) {
			this.__closeWorkItemEditor(false);
			return;
		}
		
		this._workItemEditorCell= doc.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(this._workItemEditorCell, "workItemEditor");//$NON-NLS-1$
		
		this.__createEditor(this._workItemEditorCell);
		
		var rowContainer= this.domNode;
		rowContainer.parentNode.insertBefore(this._workItemEditorCell, rowContainer.nextSibling);

		this.__editor.domNode.style.height = "1px"; //$NON-NLS-1$
		var args = {node: this.__editor.domNode, duration: 250};

		dojo.fx.wipeIn(args).play();
	},

	/**
	 * called by inline workitem editor!
	 */
	hidePreviewEditor: function() {
		this.__closeWorkItemEditor(true);
	},
	
	setHidden: function(hide) {
		this.inherited(arguments);
		
		if (this._workItemEditorCell) {
			if (hide) {
				dojo.addClass(this._workItemEditorCell, "hidden"); //$NON-NLS-1$
			} else {
				dojo.removeClass(this._workItemEditorCell, "hidden"); //$NON-NLS-1$
			}
		}
	},

	// ---- ViewModelViewerItem implementation -----------------------------------------------------------------------------

	_createContainer: function() {
		var result= doc.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(result, "workItem"); //$NON-NLS-1$
		return result;
	},
	
	_updateContent: function() {
		if (this.__editor != null) {			
			this.__editor.destroy();
			this.__editor= null;
		}
		
		this.__disposeColumns();
		this.inherited(arguments);
	},

	_createContent: function() {
		var localDoc= doc;
		this.__columns= [];
		
		this.__colorInfo= this.__calculateColorInfo();

		var rowContainer= this.domNode;
		
		var workItemId= this.__planItem.getId();
		if (workItemId < 0) {
			dojo.toggleClass(rowContainer, "placeholder", !this.isPrimaryItem()); //$NON-NLS-1$
			dojo.toggleClass(rowContainer, "auxiliary", this.__planItem.isAuxiliaryPlanItem()); //$NON-NLS-1$
			dojo.toggleClass(rowContainer, "resolved", this.__planItem.isResolved()); //$NON-NLS-1$
			dojo.toggleClass(rowContainer, "editorClosed", true); //$NON-NLS-1$
			
			this.__createEditor(rowContainer);

			this.__treeLayoutElement= this.domNode.childNodes[2];
			
			this._applyTreeLayout();
			
			return;
		}
		
		var actionBarCell=localDoc.createElement("td"); //$NON-NLS-1$
		dojo.addClass(actionBarCell, "attribute"); //$NON-NLS-1$
		dojo.addClass(actionBarCell, "actionBarCell"); //$NON-NLS-1$
		var actionBar= this._createActionBar();
		actionBarCell.appendChild(actionBar.domNode);
		this.domNode.appendChild(actionBarCell);
		
		var markerCell= Columns["marker"].createCell(this); //$NON-NLS-1$
		rowContainer.appendChild(markerCell);

		this.__summaryCell= Columns["summary"].createCell(this); //$NON-NLS-1$
		rowContainer.appendChild(this.__summaryCell);
		this.__treeLayoutElement= this.__summaryCell;
		this.__enabledDrag(true);
		
		dojo.forEach(this.__getFields(), dojo.hitch(this, function(fieldTag) {
			var attributeId= fieldTag.getValue();
			if ("com.ibm.team.apt.attribute.planitem.state" === attributeId) { //$NON-NLS-1$
				rowContainer.appendChild(Columns["state"].createCell(this)); //$NON-NLS-1$
			} else if("com.ibm.team.apt.attribute.planitem.complexity" === attributeId){ //$NON-NLS-1$
				var complexityAttribute= this.__planItem.getPlanModel().getPlanProgress().complexityAttribute;
				var column= new PlanItemViewerItemColumn(new com.ibm.team.apt.client.PlanningAttributeIdentifier(attributeId), this);
				// this has to go away, see defect 81281
				column._getLabel= function() { 
					var value= this._planItem.getAttributeValue(this._attribute);
					if(value == null)
						return null;
					if(value.label)
						value= value.label
					return isNaN(value) || complexityAttribute.unit == undefined ? String(value) : bind(MSGS.PLANNEDITEMS_COMPLEXITY_LABEL, [value, complexityAttribute.unit])
				};
				rowContainer.appendChild(column._createContainer());
				this.__columns.push(column);
			} else if (Columns[attributeId]) {
				var columnDefinition= Columns[attributeId];
				var cell= columnDefinition.createCell(this);
				rowContainer.appendChild(cell);
			} else {
				var attributeIdentifier= new com.ibm.team.apt.client.PlanningAttributeIdentifier(attributeId);
				var planItem= this.getPlanItem();
				var attribute= planItem.getPlanModel().findAttribute(attributeIdentifier);
				
				if (attribute.attributeType == com.ibm.team.apt.client.PlanningAttributeType.DURATION) {
					if (planItem.isTopLevelPlanItem() || planItem.isProgressPlanItem()) {
						var emptyColumn= localDoc.createElement("td"); //$NON-NLS-1$
						dojo.addClass(emptyColumn, "attribute"); //$NON-NLS-1$
						rowContainer.appendChild(emptyColumn);
					} else {
						var column= new PlanItemViewerItemDurationColumn(attributeIdentifier, this);
						rowContainer.appendChild(column._createContainer());
						this.__columns.push(column);
					}
				} else {
					var column= new PlanItemViewerItemColumn(attributeIdentifier, this);
					rowContainer.appendChild(column._createContainer());
					this.__columns.push(column);
				}
			}
		}), this);
		
		var actionCell= Columns["actions"].createCell(this); //$NON-NLS-1$
		rowContainer.appendChild(actionCell);
		
		var idCell= Columns["id"].createCell(this); //$NON-NLS-1$
		rowContainer.appendChild(idCell);
		
		dojo.toggleClass(rowContainer, "placeholder", !this.isPrimaryItem()); //$NON-NLS-1$
		dojo.toggleClass(rowContainer, "auxiliary", this.__planItem.isAuxiliaryPlanItem()); //$NON-NLS-1$
		dojo.toggleClass(rowContainer, "resolved", this.__planItem.isResolved()); //$NON-NLS-1$
		dojo.toggleClass(rowContainer, "editorClosed", true); //$NON-NLS-1$
		
		dojo.style(rowContainer, "backgroundColor", this.__colorInfo.rowBackgroundColor); //$NON-NLS-1$
		dojo.style(markerCell, "backgroundColor", this.__colorInfo.markerBackgroundColor); //$NON-NLS-1$
		dojo.style(actionBarCell, "backgroundColor", this.__colorInfo.markerBackgroundColor); //$NON-NLS-1$
		dojo.style(idCell, "backgroundColor", this.__colorInfo.idBackgroundColor); //$NON-NLS-1$

		this._applyTreeLayout();
		
		if (this._entry.hasTag(com.ibm.team.apt.ui.model.SpinningTag.INSTANCE)) {
			this.startSpinning();
		} else {
			this.stopSpinning();
		}

	},
		
	_createDropTarget: function() {
		return null;
	},
	
	_getTreeLayoutElement: function() {
		return this.__treeLayoutElement;
	},
	
	_setExpansionState: function(doExpand) {
		if (this.__editor != null && !this.__planItem.isDraftItem()) {
			this.__closeWorkItemEditor(false);
		}
		
		this.inherited(arguments);
	},

	_dispose: function() {
		if (this.__editor != null) {			
			this.__editor.destroy();
			this.__editor= null;
		}
		
		this.__disposeColumns();
		this.inherited(arguments);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__disposeColumns: function() {
		if (this.__columns != null) {
			for (var i= 0; i < this.__columns.length; i++) {
				this.__columns[i]._dispose();
			}
			this.__columns= null;
		}
		
		if (this._workItemEditorCell != null) {
			var rowContainer= this.domNode;
			rowContainer.parentNode.removeChild(this._workItemEditorCell);
			delete this._workItemEditorCell;
			this._workItemEditorCell= null;
			
			dojo.addClass(rowContainer, "editorClosed"); //$NON-NLS-1$
			dojo.removeClass(rowContainer, "editorOpen"); //$NON-NLS-1$
		}
	},
	
	__getFields: function() {
		return this._entry.getTags(com.ibm.team.apt.ui.model.FieldTag);
	},

	__createEditor: function(editorCell) {
		var localDoc= doc;
		var workingCopy= this.__getWorkingCopy();
		
		var workItemId= this.__planItem.getId();
		var rowContainer= this.domNode;
		this.__editor = new InlineWorkItemEditor({ 
			workItemId: workItemId, 
			parentWidget: this,
			presentation: "com.ibm.team.apt.ui.internal.previewPane", //$NON-NLS-1$
			parentParms: { 
				workItemId: workItemId, 
				parentRow: rowContainer
			}
		});
		
		var actionCell= localDoc.createElement("td"); //$NON-NLS-1$
		dojo.addClass(actionCell, "actionBarCell"); //$NON-NLS-1$
		dojo.addClass(actionCell, "attribute"); //$NON-NLS-1$
		editorCell.appendChild(actionCell);		
		
		var infoCell= localDoc.createElement("td"); //$NON-NLS-1$
		dojo.addClass(infoCell, "info");//$NON-NLS-1$
		editorCell.appendChild(infoCell);
		
		var cell= localDoc.createElement("td"); //$NON-NLS-1$
		cell.colSpan= 2 + this.__getFields().length;
		var padding= this._getTreeLevel() * 16 ;
		if (padding > 0)
			dojo.style(cell, "paddingLeft",  padding + "px"); //$NON-NLS-1$ //$NON-NLS-2$
		cell.appendChild(this.__editor.domNode);
		editorCell.appendChild(cell);
		
		var idCell= localDoc.createElement("td"); //$NON-NLS-1$
		dojo.addClass(idCell, "id");//$NON-NLS-1$
		editorCell.appendChild(idCell);
		
		dojo.toggleClass(rowContainer, "editorClosed"); //$NON-NLS-1$
		dojo.toggleClass(rowContainer, "editorOpen"); //$NON-NLS-1$
		
		dojo.style(rowContainer, "backgroundColor", this.__colorInfo.rowBackgroundColor); //$NON-NLS-1$
		dojo.style(infoCell, "backgroundColor", this.__colorInfo.markerBackgroundColor); //$NON-NLS-1$
		dojo.style(actionCell, "backgroundColor", this.__colorInfo.markerBackgroundColor); //$NON-NLS-1$
		dojo.style(idCell, "backgroundColor", this.__colorInfo.idBackgroundColor); //$NON-NLS-1$
		
		this.__enabledDrag(false);
	},

	__closeWorkItemEditor: function(synchronize) {
			this.__editor= null;
			
			var rowContainer= this.domNode;
			dojo.toggleClass(rowContainer, "editorClosed"); //$NON-NLS-1$
			dojo.toggleClass(rowContainer, "editorOpen"); //$NON-NLS-1$
			this.__enabledDrag(true);
			
			if (!this.__planItem.isDraftItem()) {
				rowContainer.parentNode.removeChild(this._workItemEditorCell);
				delete this._workItemEditorCell;
				
				if (synchronize)
					this.__planItem.synchronizeWithWorkItemWorkingCopy();
				
				var workingCopy= this.__getWorkingCopy();
				workingCopy.decrementUseCount();
			} else {
				var plan=this.__planItem.getPlanModel();
				
				var workingCopy= this.__getWorkingCopy();
				if (workingCopy.getId() < 0) {
					//unsaved draft throw it away
					this.__planItem.getParent().removeChild(this.__planItem);
					workingCopy.decrementUseCount(true);
					return;
				}
				
				//load and recreate
				this.getViewModel().updateModel(dojo.hitch(this, function(updateAccessor) {
					updateAccessor.setTag(this._entry, com.ibm.team.apt.ui.model.SpinningTag.INSTANCE);
				}));
				
				var _self= this;
				var serviceCallback= {
					_success: function(workItems) {
						// we have to wait until the scripts are loaded and this widget initialized before we can continue
						PlanningUtil.runWithCondition(function() {
							var planItem= _self.__planItem;
							var workItem= _self.__getFirst(workItems);
							if (workItem == null) {
								//the new item is not part of the plan -> remove it from the model 
								planItem.getParent().removeChild(planItem);
								workingCopy.decrementUseCount(true);
							} else {
								planItem.setWorkItem(workItem);
								
								workingCopy.decrementUseCount(true);
								
								_self.getViewModel().updateModel(function(updateAccessor) {
									updateAccessor.clearTag(_self._entry, com.ibm.team.apt.ui.model.SpinningTag.INSTANCE);
								});
							}
						}, function() { return _self.getViewModel(); });		
					},
					_error: function(operationReport) {
						workingCopy.decrementUseCount(true);
						
						_self.getViewModel().updateModel(function(updateAccessor) {
							updateAccessor.clearTag(_self._entry, com.ibm.team.apt.ui.model.SpinningTag.INSTANCE);
						});
						
						var description;
				   		if (operationReport.infos && operationReport.infos.length > 0)
				   			description= operationReport.infos[0].description;
				   		_self._plannedItemsTab.showErrorMessage(operationReport.message, description);
					}					
				};
				
				var responseHandler= new ServiceResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
				PlanningClient.loadPlanItems(responseHandler, plan.getPlanId(), [this.__planItem.getUuid()], this._plannedItemsTab._referencedItems);
			}
	},
	
	__getFirst: function(workItems) {
		for ( var workItemId in workItems) {
			return workItems[workItemId];
		}
		return null;
	},
	
	__getWorkingCopy: function() {
		return WorkItemProxyFactory.getWorkItemProxy({id: this.__planItem.getId(), createIfNeeded: true });
	},
	
	__enabledDrag: function(enabled) {
		if (!this.__summaryCell)
			return;
		
		if (enabled) {
			dojo.addClass(this.__summaryCell, "dojoDndHandle"); //$NON-NLS-1$
		} else {
			dojo.removeClass(this.__summaryCell, "dojoDndHandle"); //$NON-NLS-1$
		}
	},
	
	__calculateColorInfo: function() {
		var highestPriorityTag= null;
		dojo.forEach(this._entry.getTags(com.ibm.team.apt.ui.model.ColorizeTag), function(colorTag) {
			if (highestPriorityTag == null || highestPriorityTag.priority > colorTag.priority) {
				highestPriorityTag= colorTag;
			}
		});

		var rowBackgroundColor= colorBackground;
		var markerBackgroundColor= colorInfo;
		var idBackgroundColor= colorInfo;

		if (highestPriorityTag != null) {
			rowBackgroundColor= dojo.blendColors(highestPriorityTag.color, colorBackground, 192/256);
			markerBackgroundColor= dojo.blendColors(highestPriorityTag.color, colorInfo, 52/256);
			idBackgroundColor= dojo.blendColors(highestPriorityTag.color, colorInfo, 192/256);
		}
		
		return {
			rowBackgroundColor: rowBackgroundColor.toHex(),
			markerBackgroundColor: markerBackgroundColor.toHex(),
			idBackgroundColor: idBackgroundColor.toHex()
		};
	},

	__sentinel: null // terminates this class definition
});

})();
