/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.team.workitem.web.client.internal.query.Term");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeTree");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {
	
var Term= com.ibm.team.workitem.web.client.internal.query.Term;
var AttributeTree= com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeTree;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/AttributeChooser.html"),
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		if(!args.isOperationAttributeChooser) {
			this.textAddCondition = this.messages.labelAddCondition;
		} else {
			this.textAddCondition = this.messages.labelAddAttributeChange;
		}
		this.attributeTree= args.attributeTree;
		this.termSection= args.termSection;
		this._addIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/add-condition.gif");
		this._closeIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close.gif");
		this._closeIconHoverSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close-hov.gif"); 
	},
	
	postCreate: function() {
		this._createAddTermButton();
		this._createTree();
		this._createAddAttributeButton();
		if(this.isOperationAttributeChooser) {
			this._addTermButtonNode.style.display="none"; // operation attributes don't use that button
		}
	},
	
	destroy: function() {
		if(this._tree) this._tree.destroy();
		if(this._addAttributeButton) this._addAttributeButton.destroy();
		
		this.inherited(arguments);
	},
	
	onClose: function(e) {
		if(e) dojo.stopEvent(e);
		this._contentNode.style.display= "none";
	},
	
	onAdd: function(e) {
		if(e) dojo.stopEvent(e);
		this._contentNode.style.display= "block";
	},
	
	checkEnterForAdd: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key pressed was 'enter'
			this.onAdd();
		}		
	},
	
	checkEnterForClose: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key pressed was 'enter'
			this.onClose();
		}		
	},
	checkEnterForAddAttribute: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key pressed was 'enter'
			this._handleAddAttributeCondition();
		}			
	},
	addAttributeSection: function(attributeId) {
		this.onAdd(); // make visible
		if(this.termSection) {
			this.termSection.addSection(attributeId); // the attribute identifier
		}
	},
	_createTree: function() {
		var tree= new AttributeTree( { model: this._createModel(this.attributeTree.children) } );
		this._tree= tree;
		this._treeNode.appendChild(tree.domNode);
		this.connect(tree.domNode, "ondblclick", this._handleAddAttributeCondition);
		this.connect(tree.domNode, "onkeypress", this.checkEnterForAddAttribute);
	},

	_createAddTermButton: function() {
		var self= this;
		var operator= this.termSection.getOperator() == Term.AND ? Term.OR : Term.AND;
		var addTermButtonParameters= {
			label: operator == Term.AND ? self.messages.labelAddOperatorConditionAND : self.messages.labelAddOperatorConditionOR,
			onClick: function() {
				self._handleAddTermCondition(operator);
				self.onClose();
			}
		};
		var addTermButton= new dijit.form.Button(addTermButtonParameters);
		this._addTermButtonNode.appendChild(addTermButton.domNode);
	},
	
	_createAddAttributeButton: function() {
		var self= this;
		var addAttributeButtonParameters= {
			label: self.messages.labelAddAttributeCondition,
			onClick: function() {
				self._handleAddAttributeCondition();
			}
		};
		var addAttributeButton= new dijit.form.Button(addAttributeButtonParameters);
		this._addAttributeButton = addAttributeButton;
		this._addAttributeButtonNode.appendChild(addAttributeButton.domNode);
	},
	
	_handleAddAttributeCondition: function() {
		
		var nodes= this._tree.getSelection();
		var section= this.termSection;
		
		dojo.forEach(nodes, function(node) {
			if (node.item.isQueryable)
				section.addSection(node.item.id); // the attribute identifier
		});
	},
	
	_handleAddTermCondition: function(operator) {
		this.termSection.addTermSection({ operator: operator });
	},
	
	_createModel: function(/* TreeDTO */ dtos){
		var items= new Array();
		for (var i= 0; i < dtos.length; i++) {
			var attribute= dtos[i].item;
			if (dtos[i].children) {
				attribute.children= this._createModel(dtos[i].children);
			}
			if (attribute.isQueryable || attribute.children)
				items.push(attribute);
		}
		items.sort(this._sortAttributes);
		return items;
	},
	
	_sortAttributes: function(entry1, entry2){
		if (entry1.label < entry2.label) {
			return -1;
		}
		else if (entry1.label > entry2.label) {
			return 1;
		}
		return 0;
	},
	
	onCloseMouseOver: function(e) {
		this._closeImage.src = this._closeIconHoverSrc;	
	},
	
	onCloseMouseOut: function(e) {
		this._closeImage.src = this._closeIconSrc;
	}
});
	
})();
