/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007,2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.page.CqRecordController");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.page.Preference")
dojo.require("com.ibm.rational.cm.web.ui.page.RecordTypeManager");
dojo.require("net.jazz.ajax.ui.PlatformUI");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.utils.RecordChooser");

dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqStrings");
dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqRecordStrings");

(function(){

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";
var cqLoginController = com.ibm.rational.cm.cqLoginController;
var recTypeManager = com.ibm.rational.cm.web.ui.page.recTypeManager;

var RecordDetailRESTServiceURL = "cqartifactdetails.cq";
var workbench;

/* 
 * @description: CqRecordController is a custom dojo widget,
 * This widget will help handle all the events related to a CqRecord 
 * and manage the lifecycle of the CqRecord.
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.page.CqRecordController",
	dijit._Widget,
{
	/*
		CqView id with which this controller is associated
	*/
	connectWidgetId:"",
	tabContainerIds: {
		TOP_RIGHT: "top-rightContentPane",
		BOTTOM_RIGHT: "bottom-rightContentPane"
	},
	
	/*
	* holds all CqFormLayout data
	* key = recordType + "_" + state (VIEW,SUBMIT)
	*/
	_cqFormLayoutData:[], // map of form. 
	_isCustom:[], // true or false
	_tabIndexes:[],
	/*
	* List of CqRecords
	*/
	_cqRecords:[],
	/*
		List of FloatingPane  
	 */
	_floatingPane:[],
	/*
		List of all localized Strings required for this widget.
	*/	
	_messages:null,
	/*
		Store the current record displayed
	*/
	_cqRecordSelected:null,
	/*
		scale factor
	*/	
	xScaleFactor:1.9,		
	yScaleFactor:1.7,
	/*
	* Initial position of floating pane
	*/
	_initX: 100,
	_initY: 100,
	/*
	 * 
	 */
	_emailDialog:null,
	_errorDialog:null,
	/*
	 * Customize View
	 */
	_hiddenList:null,
	_prefVersion:"M3B",
	_prefCategory:"HIDDEN_FIELDS",
	_prefKey:"cqRecordHiddenList",

	/*
	 * Service URLs
	 */
	_GetModifyFormURL: RecordDetailRESTServiceURL + "?action=GetModifyForm",
	_GetRecordDetailURL: RecordDetailRESTServiceURL + "?action=GetCQRecordDetails",
	_GetSubmitFormURL: RecordDetailRESTServiceURL + "?action=GetSubmitForm",
	_GetNewRecordURL: RecordDetailRESTServiceURL + "?action=NewRecord",
	_RecordCommitURL: RecordDetailRESTServiceURL + "?action=RecordCommit",
	_BatchUpdateCommitURL: RecordDetailRESTServiceURL + "?action=BatchUpdateCommit",
	_RevertURL: RecordDetailRESTServiceURL + "?action=RecordRevert",
	_DeleteURL: RecordDetailRESTServiceURL + "?action=RecordDelete",
	_DuplicateURL: RecordDetailRESTServiceURL + "?action=RecordDuplicate",
	_UnduplicateURL: RecordDetailRESTServiceURL + "?action=RecordUnduplicate",
	_ScriptAliasURL: RecordDetailRESTServiceURL + "?action=FireRecordScriptAlias",
	_PrintURL: RecordDetailRESTServiceURL + "?action=PrintRecord",
	_PingURL: "cqlogin.cq?action=Ping",
	
	/*
	* Cache for XHR request
	*/
	_xhrReq: [], 
	/*
	*
	*/
	postMixInProperties:function(){
		var getLoc = dojo.i18n.getLocalization;
		var lang = this.lang;
		this._messages = getLoc(cqNS, "cqRecordStrings", lang);
		this.resources = getLoc(cqNS, "cqStrings", lang);
		workbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
		this.inherited(arguments);
	},
	/*
		Initialize the widget to subscribe to the events.
	*/
	postCreate: function(){
		this.inherited(arguments);
		var subscribe = dojo.subscribe;
		this._subscriptions=[
    		subscribe("CQ_RECORD_SUBMIT", this,"_onRecordSubmit"),	
//    		subscribe("CQ_RECORD_SELECTED", this,"_onRecordSelected"),	
//    		subscribe("CQ_BATCH_UPDATE_RECORD", this,"_onBatchUpdate"),
			// message from StackContainer	
    		subscribe(this.connectWidgetId+"-selectChild", this,"_onTabSelected"),
			// message from CqRecord	
    		subscribe("CQ_RECORD_ACTION", this,"_onRecordAction"),
    		subscribe("CQ_RECORD_INTERNAL_FAVORITES", this, "_onAddToFavoritesClicked"),
    		subscribe("CQ_RECORD_REFRESH", this, "_refreshRecord"),
    		subscribe("CQ_RECORD_PRINT", this, "_print"),
			// message from outside of CqRecord
    		subscribe("CQ_RE_AUTHENTICATED",this,"_handleReAuthenticated")
    	];
	},
	
	destroy: function(){
		dojo.forEach(this._subscriptions, dojo.unsubscribe);
		if(this._emailDialog && this._emailDialog.domNode){
			this._emailDialog.destroyRecursive();
		}
		if(this._errorDialog && this._errorDialog.domNode){
			this._errorDialog.destroyRecursive();
		}
		this.inherited(arguments);
	},
	
	_isRecord: function(widget){
		if(!widget){ return false; }
		return (widget.declaredClass == "com.ibm.rational.cm.web.ui.view.record.CqRecord");
	},
	
	_onAddToFavoritesClicked:function(message){
		var record = message.record;
		if (!record) {
			record = this._cqRecordSelected;
			if (!record) {
				return;
			}
		}
		var type = "RECORD";
		var title = record.getRecordTypeDisplayName() + "/" + record.getDisplayId();	
		var stableLocation = record.id;
		var masterReplica = null;
		if(record.cqRecordData && record.cqRecordData.MasterReplica){
			masterReplica = record.cqRecordData.MasterReplica.DisplayName;
		}
		dojo["require"]("com.ibm.rational.cm.web.ui.page.FavoritesController");
		com.ibm.rational.cm.favoritesController.addFavorite({
			type: type,
			title: title,
			stableLocation: stableLocation,
			masterReplica: masterReplica
		});
	},
	
	_onTabSelected: function(message){
		if(!(this._isRecord(message))){ return; }
		this._cqRecordSelected = message;
	},
		
	_getCQFormLayoutData:function(message){
//		console.debug("CqRecordController._getCQFormLayoutData:" + message.id);
//		this._fetchForm(recordId, "VIEW", null, true); // pre-fetch 
	},
	
	/*
	*
	*/
	_getRecordType:function(recordId, state) {
		if (state == "SUBMIT") {
			return recordId.substring(recordId.indexOf(":")+1,recordId.lastIndexOf("/"));
		}
		return recordId.substring(recordId.indexOf(":")+1,recordId.indexOf("/"));	
	},
	/*
		Handles all the record Actions and routes to appropriate handler
	*/
	_onRecordAction:function(message){		
		if ( message == null)
			return;
		var recordId = message.recordId;
		if ( message.action == "MODIFY"){
			var recs = this._filterEditableRecords([recordId]);
			if(recs.length == 0){
				alert(this._messages.cqRecord_error_startmodify);
				return;
			}
			this._displayForm(recordId, "MODIFY", message.actionId);
		}else if(message.action == "APPLY"){
			this._commitRecord(recordId, message.batchRecords);
		}else if(message.action == "CANCEL"){
			this._cancelEdit(recordId, message.batchRecords);
		}else if(message.action == "DELETE"){
			this._deleteRecord(recordId, message.actionId);
		}else if(message.action == "DUPLICATE"){
			this._duplicateRecord(recordId, message.actionId);
		}else if(message.action == "UNDUPLICATE"){
			this._unduplicateRecord(recordId, message.actionId);
		}else if(message.action == "SCRIPTALIAS"){
			this._runScriptAlias(recordId, message.actionId);
		}else if(message.action == "CLOSE"){
			this._displayError(null, message.view); // close view
		}else{
			console.log("cqRecordController._onRecordAction: unsupported action:"+message.action);
		}
	},
	/*
	
	*/
	_setLoading: function(view, loading){
		if(view && view.domNode){
			view.setLoading(loading);
		}else if(!loading && workbench){
			workbench.hideMessageArea();
		}
	},
	
	/*
		Fetch record form for the Submit
	*/
	newRecord: function(recordType, container, ts){
		return this._onRecordSubmit({recordType:recordType, container: container, ts: ts, openNewWindow: false});
	},
	
	_submitCount: 0,	
	_onRecordSubmit:function(message){
		if ( message == null)
			return;
		var openNewWindow = (message.openNewWindow !== undefined && !message.openNewWindow) ? false : true;
		if(message.openNewWindow == "auto"){
			openNewWindow = (this.connectWidgetId != this.tabContainerIds.TOP_RIGHT);
		}
		var recordType = message.recordType;
		// create dummy recordId ... see _getRecordType()
		var recordId = "rcm_new_record:" + recordType + "/" + (message.ts || this._submitCount); 
		this._submitCount++;	
		var view = this._createAndShowView(recordId, message.container, openNewWindow, message.displayName, null, true, message.referrer);
		this._displayForm(recordId, "SUBMIT");
		return view;
	},
	
	/*
		Fetch record details for the selected Record
	*/
	_onRecordSelected: function(message){
		// open record view
		var container = message.container || null;
		var openNewWindow = message.openNewWindow || false;
		dojo.forEach(message.selectedArtifactIds, function(r){
			var rec = r.recordId;
			var actionId = r.actionId || message.actionId || null;
			var hasMastership = r.hasMastership;
			if (dojo.isArray(rec)) {
				dojo.forEach(rec, function(rid) {
					this.openRecordView(rid, container, openNewWindow, actionId, hasMastership);
				}, this);
			} else {
				this.openRecordView(rec, container, openNewWindow, actionId, hasMastership);
			}
		}, this);
	},
	
	/*
		Fetch record details for the selected Record
	*/
	_batchCount: 0,
	_onBatchUpdate: function(message, container){
		var rs = message.selectedArtifactIds;
		if (!dojo.isArray(rs) || !rs[0]) {
			console.error("CqRecordController._onBatchUpdate needs array of record.");
			return;
		}
		var rs2 = this._filterEditableRecords(rs);
		if(rs2.length == 0){
			alert(this._messages.cqRecord_error_startbatchupdate);
			return;
		}else if(rs2.length != rs.length){
			var rs3 = dojo.filter(rs, function(r){
				return dojo.indexOf(rs2, r) < 0;
			});
			var dlg = this._openMessageDialog(this._messages.batchupdate_warning_body,
								  this._messages.batchupdate_warning_title,
								  [this._messages.loading]);
			cq.xhrPost({
				url:"cqproperties.cq",	
				handleAs: "json",
				content: {
					action:"GetDisplayName",
					resourceId: rs3.join(",")
				},
				load: function(data, ioArgs){
					if(data && data.STATUS=="true" && dojo.isArray(data.data) && dlg.domNode){
						dlg.setInvalidItems(data.data);
					}
				}		
			});
		}
		var actionId = message.actionId;
		var recId = rs2[0]+":BATCH_"+this._batchCount;
		this._createAndShowView(recId, container, false, this._messages.titleForBatchUpdate, rs2);
		this._fetchForm(recId, "BATCH", false, false, dojo.hitch(this, function(){
			this._fetchRecordData(recId, "BATCH", actionId);
		}));
		
		return recId;
	},
	
	/*
	*
	*/
	_filterEditableRecords: function(records){
		return dojo.filter(records, function(r){
			// try to find in editing records 
			for(var id in this._cqRecords){
				var rr = this._cqRecords[id];
				if(!rr){
					continue;
				}
				if(rr.batchRecords){
					// Batch Update
					for(var j=0, l2=rr.batchRecords.length; j<l2; j++){
						var rr2 = rr.batchRecords[j];
						if(rr2 == r){
							// found in Batch Update
							return false;
						}
					}
				}else{
					if(rr.cqState == "MODIFY" && id == r){
						// Under "MODIFY" state
						return false;
					}
				}
			} 
			// not found in editing records
			return true;
		}, this);
	},
	
	/*
		Fetch record details for the selected Record
	*/
	openRecordView: function(recordId, container, openNewWindow, actionId, hasMastership) {
		if (!dojo.isString(recordId)) {
			console.error("CqRecordController.openRecordView got invalid record id: " + recordId);
			return;
		}
		var newState = actionId ? "MODIFY" : "VIEW";
		var view = this._showExistingView(recordId, container);
		if(view && !(view.cqState == "VIEW" && newState == "MODIFY")){
			// if view is already exist and not (VIEW -> MODIFY) them just return.
			return view;
		}
		if(!view){
			// create new cqRecord
			view = this._createAndShowView(recordId, container, openNewWindow);
			if(!view){
				// fail to create cqRecord?
				return;
			}
		}
		// show view with new state
		this._displayForm(recordId, newState, actionId || null, hasMastership);
		return view;
	},
	
	_showExistingView: function(recordId, container){
		var recordView = this._cqRecords[recordId];
		if(!recordView){
			// record not found
			return false;
		}	
		if(container){
			container.addChild(recordView);
			return recordView;
		}
		var floatingPane = this._floatingPane[recordId];
		if (floatingPane) {
			floatingPane.show();
			floatingPane.bringToTop();
			dijit.focus(floatingPane.focusNode);
		}else{
			var mainContainer = dijit.byId(this.connectWidgetId);
			if(mainContainer.selectChild){
				mainContainer.selectChild(recordView);
			}
		}
		return recordView;
	},
	/*
		Create CqRecord widget
	 */
	_createAndShowView: function(recordId, container, openNewWindow, title, records, isSubmit, referrer){
		var view = this._showExistingView(recordId, container);
		if(view){
			// view for the target recordId has been opened already
			return view;
		}
		// load user's preference
		if(!this.userPref){
			this.userPref = dijit.byId("cqUserPreferences");
		}
		if(!this._hiddenList){
			this._getHiddenList();
		}
		// read from preferences
		var recordType = this._getRecordType(recordId);
		var customView = this._isCustom[recordType+"_MODIFY"];
		if(customView === undefined){
			customView = false;
			if(this.userPref){
				var prefVal = this.userPref.getUserPreference("resultSetview", "SHOW_CUSTOM_RECORD_VIEW");
				if(prefVal == "true"){
					customView = true;
				}
			}
		}
		// create cqRecord instance
		var cqFormProps={_hiddenList:this._hiddenList[recordType+"_MODIFY"],
						  id:recordId,		// use id for recordId except for BatchUpdate.
						  isContainer:true,
						  xScaleFactor:this.xScaleFactor,
						  yScaleFactor:this.yScaleFactor,
						  custom:customView,	// this.customView
						  region: "center",
						  _tabIndex: isSubmit ? 0 : this._tabIndexes[recordType+"_MODIFY"] || 0
						  };
		if(records){
			cqFormProps.batchRecords = records;
		}
		if(referrer){
			cqFormProps.referrer = referrer;
		}
		if(isSubmit){
			cqFormProps.cqStatus = "SUBMIT";
		}
		dojo["require"]("com.ibm.rational.cm.web.ui.view.record.CqRecord");
		view = new com.ibm.rational.cm.web.ui.view.record.CqRecord(cqFormProps);
		this._cqRecords[recordId] = view;
		// template 
		if(isSubmit){
			var w = new com.ibm.rational.cm.web.ui.page.CqRecordTemplatePane({view: view, recordType: recordType, region:"bottom", messages:this._messages});
			view.addChild(w);
		}
		// keep tab
		view.connect(view, "_tabSelected", dojo.hitch(this, function() {
			if(view.cqState != "SUBMIT"){
				this._tabIndexes[view.cqRecordType+"_MODIFY"] = view._tabIndex;
			}
		}));		
		// keep hidden list par record type
		view.connect(view, "updateHiddenList", dojo.hitch(this, function() {
			this._hiddenList[view.cqRecordType+"_MODIFY"] = view._hiddenList;
			if(this.userPref){
				var jsonData = this._createJSONfromHiddenList(this._hiddenList);
				if(jsonData){
					this.userPref.setUserPreference(this._prefCategory, this._prefKey, jsonData, {"ignoreError": true});
				}
			}
		}));
		// keep custom view or tab view par record type
		view.connect(view, "setCustom", dojo.hitch(this, function(custom) {
			this._isCustom[view.cqRecordType+"_MODIFY"] = custom;			
		}));
		// clean up from our cache when the tab page is destroyed
		var connId = this.connect(view, "destroy", dojo.hitch(this, function(){
			this.disconnect(connId);
			var viewId = view.id;
			if(this._cqRecords[viewId] == this._cqRecordSelected){
				this._cqRecordSelected = null;
			}
			var floatingPane = this._floatingPane[viewId]
			if (floatingPane && floatingPane.domNode) {
				floatingPane.destroy();
			}
			delete this._cqRecords[viewId];
		}));
		// add to container
		if(openNewWindow){
			this._openRecordInNewWindow(recordId, view, title);					  
			view.startup();
		}else{
			if(!container){
				container = dijit.byId(this.connectWidgetId);
			}
			container.addChild(view);

			if(container.setIcon) container.setIcon(view, "TreeIcon_RECORD");
			view.connect(view, "setLoading", function(loading){
				// should not refer to "container" since it might change by split view preference
				var parent = view.getParent();
				if(!parent || parent.declaredClass != "com.ibm.rational.cm.web.ui.widget.CqTabContainer"){ return; }
				parent.setLoading(view, loading);
			});
			if(container.selectChild){
				container.selectChild(view);
			}
			
			// RATLC01063622 : Sometimes tabs "spill" into the toolbar for the record being viewed
			if(container.getTabButton){
			view.connect(container.getTabButton(view), "setAbbreviatedLabel", function(){
				container.startup();
				container.layout();
			});
			}

			container.startup();
			container.layout();
		}
		view.setLoading(true); // after added to container
		return view;
	},
	
	_openRecordInNewWindow:function(recordId, recordView, title){
		var floatingPaneProperties={
		         id: "floatingPane"+recordId,
		         title: (title != null) ? title : recordId,
		         iconSrc: "images/clrqst.gif",
		         maxable: true,
		         resizable: true,
		         dockable: false,
		         taskBarId:"cqTaskBar",
		         parseOnLoad:false, // cqRecord was already created and added to manager. 
		         refreshOnShow:true,
		         executeScripts:false,
		         style: "position: absolute;"
		      };

		var div = dojo.doc.createElement("div");
		dojo.body().appendChild(div);
		var self = this;
		cq.loadModule("com.ibm.rational.cm.web.ui.view.common.FloatingPane", function(){
			var floatingPane = new com.ibm.rational.cm.web.ui.view.common.FloatingPane(floatingPaneProperties, div);
			floatingPane.resize({l:self._initX,t:self._initY,w:640,h:480});
			floatingPane.startup();
			floatingPane.show();
			recordView.domNode.style.width = "100%";
			recordView.domNode.style.height = "100%";
//			floatingPane.setContent(recordView.domNode);
			floatingPane.containerNode.appendChild(recordView.domNode);
			floatingPane.connect(floatingPane, "resize", function() {
				recordView.resize();
			});
			self._floatingPane[recordId]=floatingPane;
			var connId = self.connect(floatingPane, "destroy", dojo.hitch(self, function() {
				self.disconnect(connId);
				if(recordView){
					if(recordView.cqState == "SUBMIT"){
						self._cancelEdit(recordId);
					}
					if(recordView.domNode) {
						recordView.destroyRecursive();
					}
				}
				self._floatingPane[recordId]=null;
			}));
		
			floatingPane.bringToTop();
			dijit.focus(floatingPane.focusNode);
		});
	},
	
	/*
	*
	*/
	_displayForm: function(recordId, state, actionId, hasMastership) {
		var view = this._cqRecords[recordId];
		if (!view || !view.domNode) {
			// view might have been closed
			return;
		}
		if (!state) {
			state = "VIEW";
		} 
		// reload form and data
		view.setLoading(true);
		this._fetchForm(recordId, state, false, true, dojo.hitch(this, function(){
			this._fetchRecordData(recordId, state, actionId, false, null, hasMastership);
		})); 
	},

	/*
		Fetch the CQForm related to the RecordType associated with the Query;
	 */
	_fetchForm: function(recordId, state, cacheOnly, noUpdateWhenCached, postFunction){
		var recordType = this._getRecordType(recordId, state);
		var cacheKey = recordType+ "_" + ((state == "SUBMIT") ? "SUBMIT" : "VIEW"); // Form for "MODIFY" is same as form for "VIEW"
		var cachedForm = this._cqFormLayoutData[cacheKey]; 
		if(cachedForm){
			if(cacheOnly) {
				return;
			}
			var view = this._cqRecords[recordId];
			if(!view){
				return;
			}
			if(view.cqFormLayout == cachedForm && noUpdateWhenCached){
				postFunction();
				return;
			}
			if(!cachedForm.mForm || !cachedForm.mForm.mFormPage){
				if(!cacheOnly) this._displayError(this._messages.cqRecord_error_empty_form, view); // close view
				return;				
			}
			var floatingPane = this._floatingPane[recordId];
			if(floatingPane && cachedForm.mForm){
				var v = dijit.getViewport();
				var w = Math.max(Math.min(v.w, cachedForm.mForm.width*this.xScaleFactor+this._initX)-this._initX,100);
				var h = Math.max(Math.min(v.h, cachedForm.mForm.height*this.yScaleFactor+this._initY+100)-this._initY,100);
				var dim = {l:this._initX,t:this._initY,w:w,h:h};
				floatingPane.resize(dim);
			}
			view.isUcmEnabled = cachedForm.isUcmEnabled;
			view.isCCSupported=cachedForm.isCCSupported;
			view.updateCQForm(cachedForm, state);
			if(view.cqRecordData) {
				view.setLoading(false);
			}
			postFunction();
		}else{
			// fetch the CqFormLayoutData information and cache it.
			var url;
			var additions = null;
			if (state == "SUBMIT") {
				url = this._GetSubmitFormURL + "&recordType=" + encodeURIComponent(recordType);
			} else {
				var recId = recordId;
				if(state=="BATCH"){
					recId = recordId.substring(0,recordId.indexOf(":BATCH"));
				}
				url = this._GetModifyFormURL + "&resourceId=" + encodeURIComponent(recId);
			}
			this._callXHR(url, additions, cacheOnly ? null : this._messages.cqRecord_error_retrieval_form, this._cqRecords[recordId], true,  
				dojo.hitch(this, function(data) {
					// even if form is not defined, cache it to avoid GetCQRecordDetails is called again
					this._cqFormLayoutData[cacheKey] = data;
					
					var view = this._cqRecords[recordId];
					if(!data.mForm){
						console.error("CqRecordController._fetchForm: data.mForm is null");
						if(!cacheOnly) this._displayError(this._messages.cqRecord_error_retrieval_form, view);
						return false;
					}
					if(!data.mForm.mFormPage){
						console.log("CqRecordController._fetchForm: data.mForm.mFormPage is null");
						if(!cacheOnly) this._displayError(this._messages.cqRecord_error_empty_form, view);
						return false;
					}

					if(cacheOnly){
						return false;
					}
					
					if(view == null){
						console.log("CqRecordController._fetchForm: view has been closed");
						return false;
					}
					
					view.isUcmEnabled = data.isUcmEnabled;
					view.isCCSupported= data.isCCSupported;
					
					var floatingPane = this._floatingPane[recordId];
					if(floatingPane && data.mForm){
						var v = dijit.getViewport();
						var w = Math.max(Math.min(v.w, data.mForm.width*this.xScaleFactor+this._initX)-this._initX,100);
						var h = Math.max(Math.min(v.h, data.mForm.height*this.yScaleFactor+this._initY+100)-this._initY,100);
						var dim = {l:this._initX,t:this._initY,w:w,h:h};
						floatingPane.resize(dim);
					}
					view.updateCQForm(data, state);
					if(view.cqRecordData){
						view.setLoading(false);
					}
					postFunction();
					return false;
				}));
		}
	},
	
	/*
		Fetch field data for the selected Record
	 */
	_fetchRecordData: function(recordId, state, actionId, refresh, loadCallback, hasMastership){
		var view = this._cqRecords[recordId];
		if(!view){
			return; // view has been closed
		}
		this._setLoading(view, true);
		var tabIndex = -1;
		var fields = null;
		var url;
		if (state == "SUBMIT") {
			url = this._GetNewRecordURL + "&recordType=" + encodeURIComponent(this._getRecordType(recordId, "SUBMIT"));
		} else {
			var recId = recordId;
			var s = state;
			if(state=="BATCH"){
				recId = recordId.substring(0,recordId.indexOf(":BATCH"));
				s = "MODIFY";
			}
			url = this._GetRecordDetailURL + "&resourceId=" + encodeURIComponent(recId) + "&state=" + s;
		}
		if(refresh === true){
			url = url + "&refresh=true";
		}else if(state != "BATCH" /* && state != "SUBMIT" */){
			if(view.custom){
				if(state == "VIEW"){
					fields = view.calcDisplayFields();
					if(fields){
						url += "&fields=" + dojo.toJson(fields);
					}
				}
			}else{
				tabIndex = view._tabIndex; // Download data for opened page only
				url += "&tabIndex=" + tabIndex;
			}
		}
		if(actionId) {
			url = url + "&actionId=" + encodeURIComponent(actionId);
		}
		var self = this;
		cq.xhrGet({
			url:url,	
			handleAs: "json",
			preventCache: true,
			error:function(response, ioArgs){
				console.error("CqRecordController._fetchRecordData:" + response);
				var view = self._cqRecords[recordId];
				self._setLoading(view, false);
				if(view.cqFormLayout && view.cqRecordData){
					view = null; 
				}
				var errmsg = {message1: self._messages.cqRecord_error_retrieval_form_data};
				if(response.message){
					errmsg.message1 = errmsg.message1 + "\n\n" + response.message; 
				}
				if(djConfig.isDebug && ioArgs && ioArgs.xhr && ioArgs.xhr.responseText){
					errmsg.description = ioArgs.xhr.responseText;
				}
				self._displayError(errmsg, view);
				return;
			},
			load: function(data, ioArgs){
				var view = self._cqRecords[recordId];
				if (view == null) {
					console.log("CqRecordController._fetchRecordData: view has been closed");
					return false;
				}
				if (view.cqFormLayout) {
					view.setLoading(false);
				}
				var closeView = view.cqFormLayout && view.cqRecordData ? null : view; // data is already available
				var f = null;
				if(data){
					f = data.fields
					console.log("CqRecordController._fetchRecordData:" + data.status + "," + data.STATUS);
					if (data.status == "LOGIN_FAIL"){
						self._displayError(null, closeView);
						return false;
					}
					if (data.STATUS == "false"){
						if(data.ReasonCode == "not-found"){
							self._displayError(self._formatError(self._messages.recordDeletedSinceQueryRan, data.DetailedMessage), view);
						}else{
							self._displayError(self._formatError(data.ERROR_MESSAGE, data.DetailedMessage, data.ExceptionStackTrace), closeView);
						}
						return false;
					}
				}
				if (!f){
					console.error("CqRecordController._fetchRecordData: data is empty");
					self._displayError(self._messages.cqRecord_error_retrieval_form_data, closeView);
					return false;
				}
				// the following is a kind of workaround: GetRecordData returns duplicated fields for a FieldName
				var fs = [];
				for(var i=0,l=f.length; i<l; i++){
					var fname = f[i].FieldName;
					if(!fname) continue;
					if(dojo.indexOf(fs, fname) >= 0){
						f[i] = {}; // prevent duplicate Field
						continue;
					}
					fs.push(fname);
				}
				// set the FormLayout and field Data for CQForm control
				view.updateRecordData(data, state == "BATCH" ? "BATCH" : (data.State || state), tabIndex, fields != null);
				
				var newRecordType = data.recordType;
				newRecordType = newRecordType.substring(newRecordType.indexOf(":")+1,newRecordType.indexOf("@"));
				view.cqRecordType = newRecordType;
				view.isUcmEnabled = data.isUcmEnabled;
				
				if(state == "SUBMIT" || state == "MODIFY"){
					dojo.publish("CQ_MODIFY_FORM_SHOWN",[{"view":view, "id":recordId}]);
				}
				if (state != "SUBMIT" && state != "BATCH") {
					var title = view.getRecordTypeDisplayName() + "/" + view.getDisplayId();
					var masterReplica = "<local>";
					if(data.MasteredLocally !== undefined && data.MasteredLocally !== true && data.MasterReplica){
						masterReplica = data.MasterReplica.DisplayName;
					}
					dojo.publish("CQ_RECORD_SHOWN",[{"id":recordId, "title":title, "masterReplica":masterReplica}]);
					
					// change icon depending on mastership
					var parent = view.getParent();
					if(parent && parent.declaredClass == "com.ibm.rational.cm.web.ui.widget.CqTabContainer"){
						if(masterReplica == "<local>" || masterReplica == cqLoginController.getLoginInfo("localReplica")){
							parent.setIcon(view, "TreeIcon_RECORD");
						}else{
							parent.setIcon(view, "TreeIcon_RECORD_REMOT");
						}
					}
					
					var floatingPane = self._floatingPane[recordId];
					if (floatingPane) {
						dojo["require"]("com.ibm.rational.cm.web.ui.view.record.CqRecord");
						var displayName = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(view.getDisplayId());
						floatingPane.setTitle(displayName); // titlebar
						floatingPane.title = displayName; // dock title ... I'm not sure. Maybe bug of floatingpane.
					}
				}
				if(loadCallback){
					loadCallback();
				}
			}
		});
	},
	/*
		Deals with Committing the Record
	 */
	_commitRecord:function(recordId, batchRecords){
		var view = this._cqRecords[recordId];
		if(!view){
			return;
		}
		var msg = this._messages;
		if(view._sessionExpired){
			var error = msg.sessionExpired + "\n" + msg.sessionExpiredInstruction + "\n\n" + msg.sessionExpiredTips + "\n" + msg.sessionExpiredTipsBody;
			cq.openMessageDialog(error);
			return;
		}
		view.setLoading(true);
		view.updateModifiedList();
/* RATLC01170292 		
		var invalidFields = view.validateRequires();
		if (invalidFields.length > 0) {
			this._openMessageDialog(msg.value_required_dialog_body, 
									msg.error_dialog_title, 
									invalidFields, "FieldName");
			view.setLoading(false);			
			return;
		}
		invalidFields = view.validateFields();
		if (invalidFields.length > 0) {
			this._openMessageDialog(msg.invalid_fields_dialog_body, 
									msg.error_dialog_title, 
									invalidFields, "FieldName");
			view.setLoading(false);
			return;
		}
*/
		var url;
		var content = {Fields:dojo.toJson(view.getModifiedList()), state:view.cqState};
		if(view.cqState == "BATCH"){
			url = this._BatchUpdateCommitURL;
			content.resourceIds = dojo.toJson(batchRecords);
		}else{
			url = this._RecordCommitURL;
			if (view.cqState == "SUBMIT") {
				content.resourceId = view.cqRecordData.StableLocation;
			}else{
				content.resourceId = recordId;
			}
		}
		var self = this;	
		cq.xhrPost({
			url: url,	
			handleAs: "json",
			content: content,
			error: function(response, ioArgs){
				console.error("CqRecordController._commitRecord:" + response);
				var errmsg = {message1: msg.cqRecord_error_commit};
				if(djConfig.isDebug && ioArgs && ioArgs.xhr && ioArgs.xhr.responseText){
					errmsg.description = ioArgs.xhr.responseText;
				}
				self._displayError(errmsg); // not close view
				view.setLoading(false);
				return;
			},
			load: function(data, ioArgs){		
				var view = self._cqRecords[recordId];
				console.log("CqRecordController._commitRecord:" + data.status + "," + data.STATUS);		
				if (data.status == "LOGIN_FAIL"){
					view.setLoading(false);
					return;
				}
				if (data.STATUS == "false") {
					self._displayError(self._formatError(data.ERROR_MESSAGE, data.DetailedMessage, data.ExceptionStackTrace));
					if(data.UpdatedFields || data.Requiredness || data.InvalidFields){
						// RATLC01170908
						view._updateByHook(data.UpdatedFields, data.Requiredness, data.InvalidFields); 
					}
					view.setLoading(false);
					return;
				}
				// server data is successfully update
				view.clearModifiedList();
				
				if(view.cqState == "SUBMIT") {
					// add to recent
					var title = view.getRecordTypeDisplayName() + "/" + (data.DisplayName || view.getDisplayId()); // RATLC01264291
					var referrer = view.referrer || null;				
						
					var fields = view.cqRecordData.fields;
					var indexArray = recTypeManager.getDefaultFields(fields, function(field) {
						return field.FieldName;
					});
					var label = "";
					dojo.forEach(indexArray, function(item, idx){
						if(idx > 0){ label += ": "; }
						var next = fields[item];
						label += next.ControlValue || next.CurrentValue;
					});
					dojo.publish("CQ_RECORD_CREATED",[{"id":content.resourceId, "title":title, "referrer":referrer, "masterReplica":"<local>", "label":label}]);
					// close the record
					view.cqState = "DONE";  // hack: avoid calling _cancelEdit by floatingPane.destroy()
					// keep some info to open record again
					var inFloat = self._floatingPane[recordId];
					var recId = view.cqRecordData.StableLocation;
					// close view
					self._closeRecord(recordId);
					
					if (!referrer) { // 79680, open view only if it not a reference record
						dojo.publish("CQ_RECORD_SELECTED",[{ selectedArtifactIds: [{ recordId: recId }], openNewWindow: 0 }]);
					}	
					
// RCM					if(!data.closeRecordAfterSave){
//						self.openRecordView(recId, null, inFloat);
//					}
				}else if(view.cqState == "BATCH" /*|| data.closeRecordAfterSave */){ // RCM doesn't close view automatically
					// close the record
					view.cqState = "DONE";  // hack for cqRecord.onClose() not to call "CANCEL" again
					self._closeRecord(recordId);
					self._setLoading(view, false);
				}else{
					// reload data as VIEW mode
					self._displayForm(recordId, "VIEW");
				}
			}
		});
	},	
	/*
		Cancel editing a record
	 */
	_cancelEdit :function(recordId, batchRecords){
		var view = this._cqRecords[recordId];
		if(!view){
			console.error("CqRecordController._cancelEdit: cannot edit target view");
			return;
		}
		var state = view.cqState;
		var cleanup = dojo.hitch(this, function() {
			dojo.publish("CQ_EDIT_CANCELED",[{recordId: recordId, state: state}]);
			if(state == "SUBMIT" || state == "BATCH"){
				// close the record
				view.cqState = state == "SUBMIT" ? "CANCEL_SUBMIT" : "DONE"; // hack for cqRecord.onClose() not to call "CANCEL" again
				this._closeRecord(recordId);
				return true;
			}else{
				// reload data as VIEW mode
				this._displayForm(recordId, "VIEW");
				// successfully canceled
				view.clearModifiedList();
				return false;
			}		
		});
		if(view._sessionExpired){
			cleanup();
			delete view._sessionExpired;
			return;
		}
		var recId = recordId;
		var closeViewOnError = false;
		if(state == "SUBMIT"){
			recId = view.cqRecordData.StableLocation;
			closeViewOnError = true;
			view.cqState = "CANCEL_SUBMIT"; // hack: avoid calling _cancelEdit by floatingPane.destroy() twice
		}else if(state == "BATCH"){
			recId = batchRecords[0];
		}
		var url = this._RevertURL + "&resourceId="+encodeURIComponent(recId);
		this._callXHR(url, null, this._messages.cqRecord_error_revert, view, closeViewOnError, cleanup);
	},
	/*
		Delete a record
	 */
	_deleteRecord:function(recordId, actionId){
	if (confirm(this._messages.confirmDeleteRecord)) {
		var url = this._DeleteURL + "&resourceId="+encodeURIComponent(recordId)+"&actionId="+encodeURIComponent(actionId);
		this._callXHR(url, null, this._messages.cqRecord_error_delete, this._cqRecords[recordId], false, dojo.hitch(this, function() {
			// close view 
			this._closeRecord(recordId);
			dojo.publish("CQ_ITEM_DELETED", [{"stableLocation": recordId}]);								
			return true;							
		}));
		};
	},
	/*
		Duplicate a record
	 */
	_duplicateRecord:function(recordId, actionId){
		var recordType = null;
		var view = this._cqRecords[recordId];
		if(view && view.cqRecordData){
			recordType = view.cqRecordData.recordType;
		}
		
		// store columnInfo for search
		// RATLC01263995: Falcon: DCT schema: When searching for duplicate record, 
		// 				  search result set only shows headline, not ID.
		//  we don't need to check the existence of "id", which is always available 
		//  for stateful record 
		var columnInfo = [{fieldPath:"id"}];
		// we will consider again about the following code in post 2.0. see enhancement 82421
		var targetControls = view.findControlItemsFor("Headline"); 
		if(targetControls.length == 0) targetControls = view.findControlItemsFor("headline"); 
		if(targetControls.length == 0) targetControls = view.findControlItemsFor("Name"); 
		if(targetControls.length == 0) targetControls = view.findControlItemsFor("name"); 
		if(targetControls.length > 0){
			var headlineField = targetControls[0];
			columnInfo.push({ fieldPath: headlineField.fieldSource });
		}

//		dojo["require"]("com.ibm.rational.cm.web.ui.utils.RecordChooser");
		com.ibm.rational.cm.openRecordChooser({
			title: this._messages.duplicate_dialog_title,
			recordType: recordType,
			columnInfo: columnInfo,
			onOk: dojo.hitch(this, function(selected){
				if(selected.length > 0){
					var originalId = selected[0].recordId;
					var url = this._DuplicateURL + "&resourceId="+encodeURIComponent(recordId)+"&actionId="+encodeURIComponent(actionId)+"&originalRecord="+encodeURIComponent(originalId);
					this._callXHR(url, null, this._messages.cqRecord_error_duplicate, this._cqRecords[recordId], false, dojo.hitch(this, function() {
						// reload data as MODIFY mode
						this._displayForm(recordId, "MODIFY");
						return false;							
					}));
				}
			}),
			multiSelect: false
		});
	},
	/*
		Unduplicate a record
	 */
	_unduplicateRecord: function(recordId, actionId){
		var url = this._UnduplicateURL + "&resourceId="+encodeURIComponent(recordId)+"&actionId="+encodeURIComponent(actionId);
		this._callXHR(url, null, this._messages.cqRecord_error_unduplicate, this._cqRecords[recordId], false, dojo.hitch(this, function() {
			// reload data as MODIFY mode
			this._displayForm(recordId, "MODIFY");							
			return false;
		}));
	},
	/*
		Delete a record
	 */
	_runScriptAlias:function(recordId, actionId){
		var url = this._ScriptAliasURL + "&resourceId="+encodeURIComponent(recordId)+"&recordScriptAlias="+encodeURIComponent(actionId);
		this._callXHR(url, null, this._messages.cqRecord_error_scriptalias, this._cqRecords[recordId], false, dojo.hitch(this, function(data) {
			// reload data as VIEW mode
			this._displayForm(recordId, "VIEW");	
			// RATLC01279670 Returned result of record script is not visible in CQ Web
			// display the returned message from record script if have.  							
			if(data && data.message){
				alert(data.message);
			}
									
			return false;
		}));
	},
	/*
	* call XHR request
	*/
	_callXHR: function(url, additions, errorMessage, view, close, postCallback) {
		console.log("CqRecordController._callXHR:" + url);
		this._setLoading(view, true);
		var closeView = close ? view : null;
		var loadCallback = dojo.hitch(this, function(data, ioArgs){
			delete this._xhrReq[url];
			var stopLoading = true;
			if (data == null || data == ""){
				console.error("CqRecordController._callXHR.load: data is empty");
				this._displayError(errorMessage, closeView);
			}else{
				console.log("CqRecordController._callXHR.load:" + data.status + "," + data.STATUS);				
				if(data.status == "LOGIN_FAIL"){
					this._displayError(null, closeView);
				}else if (data.STATUS == "false") {
					this._displayError(this._formatError(data.ERROR_MESSAGE, data.DetailedMessage, data.ExceptionStackTrace), closeView);
				}else{
					stopLoading = postCallback(data, ioArgs);
				}
			}
			if(stopLoading){
				this._setLoading(view, false);
			}
			return data;
		});
		var errorCallback = dojo.hitch(this, function(response, ioArgs){
			this._xhrReq[url] = null;
			console.error("CqRecordController._callXHR.load:" + response);
			var errmsg = {message1:errorMessage};
			if(djConfig.isDebug && ioArgs && ioArgs.xhr && ioArgs.xhr.responseText){
				errmsg.description = ioArgs.xhr.responseText;
			}
			this._displayError(errmsg, closeView);
			this._setLoading(view, false);
			return;
		});
		var xhr = this._xhrReq[url];
		if(!xhr){
			xhr = cq.xhrGet({
				url: url + (additions || ""),	
				handleAs: "json",
				preventCache: true,
				load: loadCallback,
				error: errorCallback
			});
			this._xhrReq[url] = xhr;
		}else{
			xhr.addCallback(loadCallback);
			xhr.addErrback(errorCallback);
		}
	},
	/*
		Close a record
	 */
	_closeRecord:function(recordId){
		this._displayError(null, this._cqRecords[recordId]);
	},
	
	_displayError:function(message, recordView) {
		if(message){
			if(dojo.isString(message)){
				cq.openMessageDialog(message);
			}else{
				cq.openMessageDialog(message.message1, message.message2, message.description);
			}
		}
		if(recordView && recordView.domNode){
			var floatingPane = this._floatingPane[recordView.id];
			if(floatingPane && floatingPane.domNode){
				floatingPane.close();
			}else{
				var mainContainer = dijit.byId(this.connectWidgetId);
				if(mainContainer && mainContainer.closeChild){
					mainContainer.closeChild(recordView);
				}else if(recordView.domNode){
					recordView.destroyRecursive();
				}
			}
			dojo.publish("CQ_RECORD_CLOSED",[{recordId: recordView.id}]);
		}
	},
		
	_refreshRecord:function(message) {
		if (message && message.recordId) {
			this._fetchRecordData(message.recordId, message.state, null, true); 
		} else if (this._cqRecordSelected) { 
			this._fetchRecordData(this._cqRecordSelected.id, this._cqRecordSelected.cqState, null, true); 
		}
	},
	
	_print: function(message){
		var view = message.record;
		var resourceId = null;
		if(view){
			resourceId = view.id;
		}else{
			resourceId = message.stableLocation;
			view = this._cqRecordSelected;
		}
		if(!resourceId || resourceId.length === 0){ return; }

		//we must verify our session is valid before printing because
		//print records open in a new window and cannot handle the
		//session timeout error message so we must 'ping' the controller first:
		var url = this._PrintURL + "&resourceId=" + encodeURIComponent(resourceId); //set the url up while "this" is still in scope
		url = cq.convertUrl(url);
		var self = this;
		cq.xhrPost({
			url: this._PingURL,
			handleAs: "json",
			content: null,
			sync: true,
			error: function(response, ioArgs){
				console.error("CqRecordController._print:" + response);
				var errmsg = {message1: msg.cqRecord_error_print};
				if(djConfig.isDebug && ioArgs && ioArgs.xhr && ioArgs.xhr.responseText){
					errmsg.description = ioArgs.xhr.responseText;
				}
				self._displayError(errmsg); // not close view
				self._setLoading(view, false);
				return;
			},
			load: function(data, ioArgs){
//				console.log("CqRecordController._print:" + data.status + "," + data.STATUS);		
				if(data.status == "LOGIN_FAIL"){
					self._setLoading(view, false);
					return;
				}
				if(data.STATUS == "false"){
					self._displayError(self._formatError(data.ERROR_MESSAGE, data.DetailedMessage, data.ExceptionStackTrace));
					if(view){
						if(data.UpdatedFields || data.Requiredness || data.InvalidFields){
							// RATLC01170908
							view._updateByHook(data.UpdatedFields, data.Requiredness, data.InvalidFields); 
						}
						view.setLoading(false);
					}
					return;
				}
				//here is our normal printing behaviour after checking session validity
				//var url = this._PrintURL + "&resourceId=" + view.id; //this line has been moved prior to the xhrPost call for scoping reasons
				if(view && view.custom){
					var f = view.getDisplayedControls();
					var w = window.open("about:blank","print");
					var d = w.document; 
					d.writeln("<form name='form'></form>");
					d.close();
					var hidden = d.createElement("input");
					hidden.type = "hidden";
					hidden.name = "fields";
					hidden.value = dojo.toJson(f);
					d.form.appendChild(hidden);
					d.form.method = "post";
					d.form.action = url;
					d.form.submit();
				}else{
					window.open(url);
				}
			}
		});
	},
	
	_getHiddenList: function(){
		if(this._hiddenList == null){
			this._hiddenList = []; // initialize
			if(!this.userPref){
				this.userPref = dijit.byId("cqUserPreferences");
			}
			if(this.userPref){
				var hiddenList = this._createHiddenListFromJSON(this.userPref.getUserPreference(this._prefCategory, this._prefKey));
				if(hiddenList){
					this._hiddenList = hiddenList; 
				}
			}
		}
		return this._hiddenList;
	},
	
	_createJSONfromHiddenList:function(hiddenList) {
		// [recordType][tabName][fieldSource::typeid::name] -> 
		//	{
		//		version: this._prefVersion,
		//		data: [{recordType:recordType, hiddenList: [{tabName:tabName, controls:[fieldSource::typeid::name]]}]
		//	}
		var data = [];
		for (recordType in hiddenList) {
			var rt = {recordType:recordType, hiddenList: []};
			for (tabName in hiddenList[recordType]) {
				var tn = {tabName: tabName, controls: []};
				var controls = hiddenList[recordType][tabName];
				for (controlId in controls) {
					tn.controls.push(controlId);
				}
				if (tn.controls.length > 0) {
					rt.hiddenList.push(tn);
				}
			};
			data.push(rt);
		};
		var json = {version: this._prefVersion, data: data};
		return json;
//		return dojo.toJson(json);
	}, 

	_createHiddenListFromJSON:function(json) {
		//	{
		//		version: this._prefVersion,
		//		data: [{recordType:recordType, hiddenList: [{tabName:tabName, controls:[fieldSource::typeid::name]]}]
		//	}
		//	-> [recordType][tabName][fieldSource::typeid::name]
//		if (!jsonString) return;
//		var json = dojo.fromJson(jsonString);
		if (!json) return;
		if (json.version != this._prefVersion) return; // version check
		data = json.data;
		var hiddenList = [];
		dojo.forEach (data, function(recordType) {
			var tab = [];
			hiddenList[recordType.recordType] = tab;
			dojo.forEach(recordType.hiddenList, function(hl) {
				var con = [];
				tab[hl.tabName] = con;
				dojo.forEach(hl.controls, function(control) {
					con[control] = true;
				});
			});
		});
		return hiddenList;
	},
	
	_formatError: function(message1, message2, description){
		var message = {message1:message1, message2:message2, description:description};
		return message;
	},
	
	_openMessageDialog: function(message, title, invalidItems, fieldName) {
		if(this._errorDialog) {
			this._errorDialog.titleNode.innerHTML = title;
		}else{
			this._errorDialog = new dijit.Dialog({title:title,"class":"cqRecordControllerErrorDialog"}, dojo.doc.createElement("div"));
			this._errorDialog.setInvalidItems = function(invalidItems, fieldName){
				var area = dojo.byId("cqRecordControllerErrorDialogFieldArea");
				var content = "";
				dojo.forEach(invalidItems, function(f) {
					dojo["require"]("com.ibm.rational.cm.web.ui.view.record.CqRecord");
					var v = com.ibm.rational.cm.web.ui.view.record.CqRecord.escapeHTML(fieldName ? f[fieldName] : f);
					content = content + "<li>" + v + "</li>";
				});
				area.innerHTML = content;
			};
		}
		var body = 	"<p>" +	message + "</p><ul id='cqRecordControllerErrorDialogFieldArea'></ul>" +
					"<p align='center'><button dojoType='dijit.form.Button' type='submit'>" + this._messages.close + "</button></p>"; 
		this._errorDialog.setContent(body);
		this._errorDialog.setInvalidItems(invalidItems, fieldName);
		this._errorDialog.show();
		return this._errorDialog;
	},

	_reloadNewRecord: function(view){
		var fields = com.ibm.rational.cm.web.ui.page.CqRecordTemplatePane.prototype._getUpdatedFields(view);
		this._fetchRecordData(view.id, "SUBMIT", null, false, function(){
			view.updateControls(fields);
		});
	},

	_handleReAuthenticated: function(){
		// session has been expired...
		for (var i in this._cqRecords){
			var view = this._cqRecords[i];
			var s = view.cqState;
			if(s == "SUBMIT"){
				this._reloadNewRecord(view);
			}else if(s == "MODIFY" || s == "BATCH"){
				view._sessionExpired = true; // mark as expired
			}
		};
	},
	
	/*
	 * Find a record view for a stable location id
	 */
	findRecordView: function(stableLocationId){
		for(var id in this._cqRecords){
			var rr = this._cqRecords[id];
			if(!rr || rr.batchRecords){
				continue;
			}
			if(rr.stableLocation === stableLocationId){
				return rr;
			}
		}
		return null;		
	}
}
);

var preference = com.ibm.rational.cm.preference;
var _prefCategory = "HIDDEN_FIELDS";
var _prefKey = "cqRecordTemplate";

/*
* Record Template User Interface
*/
dojo.declare(
	"com.ibm.rational.cm.web.ui.page.CqRecordTemplatePane",
	[dijit._Widget, dijit._Templated],
{
	/*
	*  preference format = {recType1:[{name:template1,fields:[]},{name:template2,fields[]}], recType2: ... } 
	*/
		
	messages: {},
	widgetsInTemplate: true,
	view: null,
	recordType: null,
	templateString: "<div class='cqRecordTemplate'>${messages.templateLabel}<select dojoAttachPoint='select'></select>" +
					"<div dojoType='dijit.form.ComboButton' dojoAttachEvent='onClick:_loadTemplate'><span>${messages.loadTemplate}</span><div dojoType='dijit.Menu'>" +
					"<div dojoType='dijit.MenuItem' dojoAttachEvent='onClick:_loadTemplate'>${messages.loadTemplate}</div>" +
					"<div dojoType='dijit.MenuItem' dojoAttachEvent='onClick:_saveTemplate'>${messages.saveTemplate}</div>" +
					"<div dojoType='dijit.MenuItem' dojoAttachEvent='onClick:_removeTemplate'>${messages.removeTemplate}</div>" +
					"</div></div></div>",
	
	postCreate: function(){
		this._createTemplateSelect();
	},
	/*
	* Load a selected template
	*/
	_loadTemplate: function(){
		var pref = this._getPref();
		if(!pref){
			return;
		}	
		var list = this._getTemplateList(pref);
		var i = this._findTemplate(list, this.templateSelect.getValue());
		if(i>=0){
			this.view.updateControls(list[i].fields);
		}else{
			alert(this.messages.templateLoadError);
		}
	},
	/*
	* Save selected or new template
	*/
	_getUpdatedFields: function(view){
		var fields = [];
		view.updateModifiedList();
		var allModified = view.getAllModified(); 
		for(var name in allModified){
			var val = allModified[name];
			if(val && (!dojo.isArray(val) || val.length > 0)){ 
				// Use ControlValue instead of CurrentValue. 
				// -> Don't update model itself. Just update control value in CqRecord:updateControls()
				var fieldData = view.findFieldDataFor(name);
				fields.push({FieldName: name, ControlValue: val, DataType: fieldData.DataType});
			}				
		}
		return fields;
	},
	_saveTemplate: function(){
		var pref = this._getPref(true);
		if(!pref){
			return;
		}	
		var name = this.templateSelect.getValue();
		if(!name){
			alert(this.messages.templateNoNameForSave);
			return;
		}
		var fields = this._getUpdatedFields(this.view);
		if(fields.length == 0){
			alert(this.messages.templateNoUpdateFields);
			return;
		}
		var list = this._getTemplateList(pref);
		var i = this._findTemplate(list, name);
		if(i>=0){
			list[i].fields = fields;
		}else{
			list.push({name: name, fields: fields});
		}
		preference.setValue(_prefCategory, _prefKey, pref);
		
		this._updateStore();
	},
	/*
	* Remove a selected template
	*/
	_removeTemplate: function(){
		var pref = this._getPref();
		if(!pref){
			return;
		}	
		var list = this._getTemplateList(pref);
		var i = this._findTemplate(list, this.templateSelect.getValue());
		if(i<0){
			return;
		}
		list.splice(i,1);
		preference.setValue(_prefCategory, _prefKey, pref);
	
		this._updateStore();
		this.templateSelect.setValue("");
	},
	/*
	* Create data store from preference
	*/
	_createStore: function(){
		var pref = this._getPref();
		var templateList = (pref == null) ? {} : dojo.map(
				pref["rt_" + this.recordType] || [], 
				function(i){
					return {name: i.name};
				}
			);
		dojo["require"]("dojo.data.ItemFileReadStore");
		var store = new dojo.data.ItemFileReadStore({data: {identifier:"name", items:templateList}});
		var req = store.fetch();
		return store;
	},
	/*
	 * Create template select combobox
	 */
	 _createTemplateSelect: function(){
	 	preference.process({
			onLoad: dojo.hitch(this, function(){
				this.templateSelect = new dijit.form.ComboBox({
						autoComplete:false, 
						store: this._createStore(), 
						searchAttr:"name"
					}, this.select);
			})
		});
	 },
	/*
	* Update select box for template
	*/
	_updateStore: function(store){
		var store = this.templateSelect.store;
		if(store){
			store.close();
		}
		this.templateSelect.store = this._createStore();
	},
	/*
	* Get preference
	*/
	_getPref: function(create){
		var pref = preference.getValue(_prefCategory, _prefKey);
		if(!pref && create){
			pref = {};
			preference.setValue(_prefCategory, _prefKey, pref);
		}
		return pref;
	},
	/*
	* Get Template list
	*/
	_getTemplateList: function(pref){
		if(pref){
			var templateList = pref["rt_" + this.recordType];
			if(!templateList || !dojo.isArray(templateList)){
				templateList = pref["rt_" + this.recordType] = [];
			}
			return templateList;
		}
		return null;
	},
	/*
	* Find a template for specified name
	*/
	_findTemplate: function(list, name){
		if(name && list){
			for(var i=0,l=list.length; i<l; i++){
				if(list[i].name == name){
					return i;
				} 
			}
		}
		return -1;
	}
}
);

})();
