/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.viewlets.ui.utils.PresentationFieldsPreferenceProvider");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.RecordTypeManager");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");

dojo.requireLocalization("com.ibm.rational.cm.web.viewlets.ui", "ViewletMessages");

(function(){

var messages;
var cq = com.ibm.rational.cm.web;
var recTypeManager = com.ibm.rational.cm.web.ui.page.recTypeManager;
var PREF_ID_QUERY = "query";

/*
 * Converts array of presentation field names to comma-separated strings
 */
var _buildLabel = function(arr){
	var text = "";
	dojo.forEach(arr, function(item){
		if(text.length > 0){
			text += ", ";
		}
		text += item;
	});
	return "<input type='text' readonly='readonly' value='" + text.replace("'","&#39;") + "'/>";
}

/*
 * Converts colData to option label map
 */
var getOptionLabelMap = function(colData){
	var map = {};
	dojo.forEach(colData, function(item, idx){
		map[idx] = item.displayName;
	});	
	return map;
}

/*
 * Returns choice items from selected items (selected) and all selectable items (colData)
 */
var getChoiceList = function(selected, colData){
	var indexArray = dojo.map(colData, function(item, idx){
		return idx;
	});
	return choice = dojo.filter(indexArray, function(item){
		return (dojo.indexOf(selected, item) == -1);
	});
}

dojo.declare(
	"com.ibm.rational.cm.web.viewlets.ui.utils.PresentationFieldsPreferenceProvider",
	com.ibm.team.dashboard.web.ui.CustomPreferenceProvider,
{
	constructor: function(args){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.viewlets.ui", "ViewletMessages");
		this._prefId = args.prefId;
		this._resourceId = null;
		this._columnInfo = {};
	},
	
	// override CustomPreferenceProvider
	getLabel: function(prefValue){
		var val = [];
		if(prefValue && prefValue != "-1"){
			var prefObj = dojo.fromJson(prefValue);
			if(prefObj){
				// get an array of fields display name
				var colData = prefObj.colData;
				val = dojo.map(prefObj.value, function(item){
					return colData[item].displayName;
				});
			}
		}
		return _buildLabel(val);
	},
	
	getColumnData: function(/* String */ resourceId, /* Function */ onLoad){
		this._resourceId = resourceId;
		var prefObj = this._columnInfo[resourceId];
		if(prefObj){
			onLoad(prefObj);
		}else{
			var self = this;
			cq.xhrGet({
				url: "cqquerywizard.cq",
				content: {
					action: "GetDisplayFields",
					resourceId: resourceId
				},
				load: function(response, ioArgs){
					var data = dojo.filter(response.data, function(item){
						return (item[2] === true); // 3rd element is visibility of fields
					});
					
					colData = dojo.map(data, function(item){
						return {
							fieldPath: item[0], // 1st element is field path
							displayName: item[1] // 2nd element is display name
						};
					});
					
					// get default values
					prefObj = {
						resourceId: resourceId,
						value: recTypeManager.getDefaultFields(colData, function(item){
							return item.fieldPath;
						}),
						colData: colData
					}
					self._columnInfo[resourceId] = prefObj;
					onLoad(prefObj);
				},
				handleError: true,
				preventCache: true
			});
		}
		return prefObj;
	},

	// override CustomPreferenceProvider
	createChooser: function(onOk, settings, viewlet){
		var preferences = settings.preferences;
		// check if query has been specified or not
		var resourceId = this._resourceId;
		if(!resourceId || resourceId.length === 0 || resourceId == "-1"){
			alert(messages.SelectQuery);
			return null;
		}

		// create double list dialog
		var dlg = this._selectDialog;
		if(!dlg){
//			dojo["require"]("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");
			dlg = this._selectDialog = new com.ibm.rational.cm.web.ui.view.common.DoubleListBoxDialog({
				title: messages.SelectField,
				// don't allow selecting more than two fields
				validate: function(){
					if(dlg.getValueList().length > 2){
						alert(messages.SelectTwoFields);
						return false;
					}else{
						return true;
					}
				},
				allowUserValue: false,
				allowOrdering: true,
				addCSH: false
			});
			dlg.resize({ w:500, h:350 });
			dlg.startup();
		}
		
		// call onOk function when [OK] button is pressed
		var prefId = this._prefId;
		var self = this;
		dlg.execute = function(formContents){
			var obj = self._columnInfo[resourceId];
			obj.value = dlg.getValueList();
			onOk(dojo.toJson(obj), prefId);
		};

		var prefObj = this._columnInfo[resourceId];
		if(!prefObj){ // if no values are cached yet, get it from preferences
			var prefValue = preferences[prefId];
			if(prefValue && prefValue != "-1"){
				var tempObj = dojo.fromJson(prefValue);
				if(tempObj){
					var savedId = tempObj.resourceId;
					this._columnInfo[savedId] = tempObj;
					if(resourceId == savedId){
						prefObj = tempObj;
					}
				}
			}
		}
		
		var setDialog = function(obj){
			var selected = obj.value; // array of index
			var colData = obj.colData;
			dlg.setOptionLabelMap(getOptionLabelMap(colData));
			dlg.setChoiceList(getChoiceList(selected, colData));
			dlg.setValueList(selected);
		}
		if(prefObj){
			setDialog(prefObj);
		}else{ // no cache, no preference yet (select query, and then directly edit fields)
			this.getColumnData(resourceId, setDialog);
		}

		dlg.show();
		return dlg;
	}
});

})();
