/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");
dojo.provide("com.ibm.rational.cm.web.ui.view.common.DoubleListBoxDialog");

dojo.require("dojo.i18n");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");

dojo.requireLocalization("com.ibm.rational.cm.web.ui","DoubleListBox");

(function(){

var cq = com.ibm.rational.cm.web;

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.DoubleListBox",
	[dijit._Widget, dijit._Templated],
{
	choiceList: null,
	valueList: null,
	allowSearch: false, // if true, filters choice list when user types in text box
	allowOrdering: false, // shows top|up|down|bottom buttons to order selection list
	allowUserValue: true, // allows to add values not included in the choice list
	isSingleSelect: false, // if true, only one value can be added to value list
	
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/common/templates/DoubleListBox.html"),
	_messages: null, 
	
	_removedList: null,
	
	postMixInProperties:function(){
		this._messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "DoubleListBox", this.lang);
		this.inherited(arguments);
	},
	
	postCreate: function(){
		if(this.choiceList == null){ this.choiceList = []; }
		if(this.valueList == null){ this.valueList = []; }
		if(this._removedList == null){ this._removedList = []; }

		// check if nore more than one value is specified when isSingleSelect=true
		if(this.isSingleSelect && this.valueList.length > 1){
			this.valueList = this.valueList.slice(0, 1);
		}

		// show addValueArea only f allowUserValue=true or allowSearch=true
		this.setAllowUserValue(this.allowUserValue);
		
		if(!this.isSingleSelect){
			this.choiceSelect.multiple = true;
		}

		this._resetChoiceList();
		this.setLoading(false, true);
		this._resetValue();
		this.setLoading(true, true);

		// only show top/up/down/bottom buttons if allowOrdering=true
		if(this.allowOrdering){
			var topBtn = dojo.doc.createElement("BUTTON");
			topBtn.innerHTML = this._messages.top;
			this.connect(topBtn, "onclick", "topClicked");
			var orderingCell = this.orderingCell;
			orderingCell.appendChild(topBtn);
			orderingCell.appendChild(dojo.doc.createElement("BR"));
			var upBtn = dojo.doc.createElement("BUTTON");
			upBtn.innerHTML = this._messages.up;
			this.connect(upBtn, "onclick", "upClicked");
			orderingCell.appendChild(upBtn);
			orderingCell.appendChild(dojo.doc.createElement("BR"));
			var downBtn = dojo.doc.createElement("BUTTON");
			downBtn.innerHTML = this._messages.down;
			this.connect(downBtn, "onclick", "downClicked");
			orderingCell.appendChild(downBtn);
			orderingCell.appendChild(dojo.doc.createElement("BR"));
			var bottomBtn = dojo.doc.createElement("BUTTON");
			bottomBtn.innerHTML = this._messages.bottom;
			this.connect(bottomBtn, "onclick", "bottomClicked");
			orderingCell.appendChild(bottomBtn);
			orderingCell.style.display = "";
		}

		this.setSingleSelect(this.isSingleSelect);

		this.inherited(arguments);
	},
	
	resize: function(dim){
		if(!dim){ return; }
	
		// adjust width
		var btnCellWidth = dojo.marginBox(this.addBtn).w + dojo._getPadExtents(this.addBtn.parentNode).w;
		var w = (dim.w - btnCellWidth)/2;
		if(this.allowOrdering){
			w -= btnCellWidth/2; // ordering cell width is same as add/remove button cell
		}
		w = (w >= 0) ? Math.floor(w) : 0;
		this.choiceContainer.parentNode.style.width = w + "px";
		this.valueContainer.parentNode.style.width = w + "px";
		
		// adjust height
		var h = dim.h - dojo.marginBox(this.labelRow).h - dojo.marginBox(this.addValueArea).h - 20;
		h = (h >= 0) ? Math.floor(h) : 0;
		
		this.choiceContainer.style.height = h + "px";
		this.valueContainer.style.height = h + "px";
	},
	
	setAllowUserValue: function(isAllow){
		this.allowUserValue = ((isAllow + "") == "true"); // hack...
		this.addValueArea.style.display = (!this.allowUserValue && !this.allowSearch) ? "none" : "";
	},
	
	setSingleSelect: function(isSingle){
		this.isSingleSelect = isSingle;
		this.addAllBtn.style.display = (isSingle) ? "none" : "";
		this.removeAllBtn.style.display = (isSingle) ? "none" : "";
	},
	
	setLoading: function(isValueList, isLoading){
		var sel = (isValueList) ? this.valueSelect : this.choiceSelect;
		var div = (isValueList) ? this.valueContainer : this.choiceContainer;
		if(isLoading){
			sel.style.display = "none";
			if(dojo.isIE){
				div.style.borderWidth = "1px";
			}
		}else{
			sel.style.display = "";
			if(dojo.isIE){
				div.style.borderWidth = "0px";
			}
		}
	},
	
	setChoiceList: function(array){
		if(array){
			this.choiceList = array;
		}else{
			this.choiceList = [];
		}
		this._resetChoiceList();
		this.setLoading(false, false);
	},
	
	setValueList: function(array){
		if(array){
			this.valueList = array;
		}else{
			this.valueList = [];
		}
		this._resetValue();
		this.setLoading(true, false);
	},
	
		// sets object that stores a map of value:label pair of options in order to generate OPTION element having labels different from values
	// obj like this : { value1: label1, value2: label2, value3: label3 }
	// will generate <select><option value="value1">label1</option><option value="value2">label2</option><option value="value3">label3</option></select>
	setOptionLabelMap: function(obj){
		this._optionLabelMap = obj;
	},
	
	addClicked: function(evt){
		if(this.isSingleSelect && this.valueList.length >= 1){
			alert(this._messages.msgOneOnly);
			return;
		}
		// first check the number of selected options
		var arr = [];
		dojo.forEach(this.choiceSelect.options, function(op, idx){
			if(op.selected){
				arr.push(idx);
			}
		});
		
		if(this.allowUserValue == false || arr.length > 1){
			if(arr.length == 0 && this.allowSearch && this.choiceSelect.options.length == 1){
				var val = this.choiceList[0]
				this.valueList.push(val);
				if(this.allowOrdering){
					this._appendOption(this.valueSelect, val);
				}
				this.choiceList = []; // clear
				this._resetChoiceList();
			}else{
				for(var i = arr.length-1; i >= 0; i--){
					var idx = arr[i];
					var val = this.choiceList[idx];
					this.valueList.push(val);
					if(this.allowOrdering){
						this._appendOption(this.valueSelect, val);
					}
					this.choiceList.splice(idx, 1);
					this._removeOption(this.choiceSelect, idx);
				}
			}
		}else{
			var val = this.textBox.value;
			if(val == "" || dojo.indexOf(this.valueList, val) >= 0){
				// the value has been already added
				return;
			}
			var choiceIndex = dojo.indexOf(this.choiceList, val);
			if(choiceIndex >= 0){
				this.choiceList.splice(choiceIndex, 1);
				this._removeOption(this.choiceSelect, choiceIndex);
			}
			this.valueList.push(val);
			if(this.allowOrdering){
				this._appendOption(this.valueSelect, val);
			}
		}
		if(!this.allowOrdering){
			this._resetValue();
		}
		
		// RATLC01065064 : Edit List popup resets after each add
		if(arr.length > 0 && arr[0] < this.choiceSelect.options.length){
			this.choiceSelect.selectedIndex = arr[0];
			this.textBox.value = this.choiceSelect.options[arr[0]].text;
		}
	},
	
	_onKeyPress: function(evt){
		if(evt.altKey || evt.ctrlKey){ return; }
		if(evt.keyCode == dojo.keys.ENTER){
			this.addClicked(evt);
		}
	},
	
	removeClicked: function(evt) {
		var numVals = this.valueSelect.options.length;
		var idx = -1;
		for(var i = numVals - 1; i >= 0; i--){
			if(this.valueSelect.options[i].selected){
				var val = this.valueList[i];
				this.valueList.splice(i, 1);
				this._removeOption(this.valueSelect, i);
				this.choiceList.push(val);
				if(this.allowOrdering){
					this._appendOption(this.choiceSelect, val);
				}
				idx = i; // store smallest index
			}
		}
		if(!this.allowOrdering){
			this._resetChoiceList();
		}

		// RATLC01065064 : Edit List popup resets after each add
		if(idx >= 0 && idx < this.valueSelect.options.length){
			this.valueSelect.selectedIndex = idx;
		}
	},

	_onKeyPressOnSelectedBox: function(evt){
		if(evt.altKey || evt.ctrlKey){ return; }
		if(evt.keyCode == dojo.keys.ENTER){
			this.removeClicked(evt);
		}
	},
	
	addAllClicked: function(evt){
		this.valueList = this.valueList.concat(this.choiceList);
		this.choiceList = [];
		this._resetChoiceList();
		this._resetValue();
	},
	
	removeAllClicked: function(evt){
		this.choiceList = this.choiceList.concat(this.valueList);
		this.valueList = [];
		this._resetChoiceList();
		this._resetValue();
	},
	
	upClicked: function(evt){
		var index = this.valueSelect.selectedIndex;
		if(index <= 0){ return; }
		var val = this.valueList[index];
		this.valueList.splice(index, 1);
		this.valueList.splice(index-1, 0, val);
		this._orderValueSelect(index, index-1);
	},
	
	downClicked: function(evt){
		var index = this.valueSelect.selectedIndex;
		if(index == -1 || index >= this.valueSelect.options.length - 1){ return; }
		var val = this.valueList[index];
		this.valueList.splice(index, 1);
		this.valueList.splice(index+1, 0, val);
		this._orderValueSelect(index, index+1);
	},
	
	topClicked: function(evt){
		var index = this.valueSelect.selectedIndex;
		if(index <= 0){ return; }
		var val = this.valueList[index];
		this.valueList.splice(index, 1);
		this.valueList.splice(0, 0, val);
		this._orderValueSelect(index, 0);
	},
	
	bottomClicked: function(evt){
		var index = this.valueSelect.selectedIndex;
		var len = this.valueSelect.options.length;
		if(index == -1 || index >= len-1){ return; }
		var val = this.valueList[index];
		this.valueList.splice(index, 1);
		this.valueList.splice(len-1, 0, val);
		this._orderValueSelect(index, len-1);
	},
	
	_orderValueSelect: function(oldIndex, newIndex){
		if(oldIndex == newIndex){ return; }
		var valueSelect = this.valueSelect;
		var targetOp = valueSelect.options[oldIndex];
		var isSelected = targetOp.selected;
		var op = targetOp.cloneNode(true); // create a clone node since removeChild will shrink SELECT in IE7
		var refIndex = (newIndex < oldIndex) ? newIndex : newIndex+1;
		var refOp = valueSelect.options[refIndex];
		this._removeOption(valueSelect, oldIndex); // RATLC01271699
		if(newIndex >= valueSelect.length){
			valueSelect.appendChild(op);
		}else{
			valueSelect.insertBefore(op, refOp);
		}
		if(isSelected){
			op.selected = true;
		}
	},
	
	selectionChanged: function(evt){
		if(this.choiceSelect.selectedIndex >= 0){
			this.textBox.value = this.choiceList[this.choiceSelect.selectedIndex];
		}
	},
	
	// Re-create SELECT element by innerHTML in order to improve performance
	_resetSelect: function(isValueList){
		var select = isValueList ? this.valueSelect : this.choiceSelect;
		var list = isValueList ? this.valueList : this.choiceList;
		var size = list.length;
		var arr = new Array(size);
		for(var i=0; i<size; i++){
			if(this._optionLabelMap && this._optionLabelMap[list[i]]){
				 arr[i] = '<OPTION value="'+list[i]+'">' + this._optionLabelMap[list[i]] + '</OPTION>';
			}else{
				 arr[i] = "<OPTION>" + list[i] + "</OPTION>";
			}
		}
		var width = select.style.width, height = select.style.height;
		var parent = select.parentNode;
		
		parent.innerHTML = '<SELECT id="'+select.id+'" size="'+select.size+'" multiple="'+select.multiple+'" class="cqDoubleListSelect">' + arr.join() + '</SELECT>';
		var newSelect = parent.firstChild; 
		newSelect.style.width = width; newSelect.style.height = height;

		if(!this._selectConnects){
			this._selectConnects = { value: [], choice: [] };
		}
		if(isValueList){
			this.connect(newSelect, "onkeypress", "_onKeyPressOnSelectedBox");
			this.valueSelect = newSelect;
		}else{
			this.connect(newSelect, "onchange", "selectionChanged");
			this.connect(newSelect, "onkeypress", "_onKeyPress");
			this.choiceSelect = newSelect;
		}

		// RATLC01260556 : CQBeta7.1.2r: unable to double-click values in selection screens for queries or list view boxes
		var self = this;
		this.connect(newSelect, "ondblclick", function(evt){
			if(isValueList){
				self.removeClicked(evt);
			}else{
				self.addClicked(evt);
			}
		});
	},
	
	_resetChoiceList: function(){
		if(!this.choiceList){ return; } // not loaded yet
		if(!this.allowOrdering){
			// Further refine sort by calling caseInsensitiveSort [by using function pointer to the builtin array sort] so that
			// "aabb" appears before "ABC".
			dojo["require"]("com.ibm.rational.cm.web.ui.view.common.sort");
			this.choiceList.sort(cq.caseInsensitiveSort);	
		}
		this._resetSelect(false);
	},
	
	search: function(e){
		if(!this.allowSearch){ return; }
		var val = this.textBox.value.toLowerCase();
		var self = this;
		var needReset = false;
		// filter the values in _removedList
		for(var i = this._removedList.length-1; i >= 0; i--){
			var rem = this._removedList[i];
			if(rem.toLowerCase().indexOf(val) == 0){
				self.choiceList.push(rem);
				self._removedList.splice(i, 1);
				needReset = true;
			}
		}

		// filter the values in choiceSelect
		for(var j = this.choiceList.length-1; j >= 0; j--){
			var choice = this.choiceList[j];
			if(choice.toLowerCase().indexOf(val) != 0){
				self._removedList.push(choice);
				self.choiceList.splice(j, 1);
				needReset = true;
			}
		}
		if(needReset){
			this._resetChoiceList();
		}
	},
	
	_resetValue: function(){
		if(!this.allowOrdering){
			// Further refine sort by calling caseInsensitiveSort [by using function pointer to the builtin array sort] so that
			// "aabb" appears before "ABC".
			this.valueList.sort(cq.caseInsensitiveSort);
		}
		this._resetSelect(true);
	},
	
	// RATLC01271699 : IE7 - Dynamic query filter value choice list shrinks as the "Add" button is clicked
	// don't use node.removeChild since it will shrink SELECT box for IE7
	_removeOption: function(select, n) {
		select.remove(n);
	},
	
	_appendOption: function(select, value) {
		var newOption = dojo.doc.createElement('option');
		select.appendChild(newOption);
		var optionLabelMap = this._optionLabelMap;
		newOption.text = (optionLabelMap && optionLabelMap[value]) ? optionLabelMap[value] : value;
		newOption.value = value;
	} 
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.DoubleListBoxDialog",
	com.ibm.rational.cm.web.ui.view.common.ButtonDialog,
{
	allowSearch: false,
	allowOrdering: false,
	allowUserValue: true,
	isSingleSelect: false,
	addCSH: false,
	
	_doubleListBox: null,
	
	setAllowUserValue: function(isAllow){
		this._doubleListBox.setAllowUserValue(isAllow);
	},
	
	setSingleSelect: function(isSingle){
		this._doubleListBox.setSingleSelect(isSingle);
	},
	
	setChoiceList: function(array){
		this._doubleListBox.setChoiceList(array);
	},

	setValueList: function(array){
		this._doubleListBox.setValueList(array);
	},
	
	setOptionLabelMap: function(obj){
		this._doubleListBox.setOptionLabelMap(obj);
	},
	
	getChoiceList: function(){
		return this._doubleListBox.choiceList;
	},
	
	getValueList: function(){
		return this._doubleListBox.valueList;
	},
	
	show: function(){
		var dblList = this._doubleListBox;
		if(dblList.focusNode.value.length > 0){
			dblList.focusNode.value = "";
		}
		dblList.search();
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// set double list box
		this._doubleListBox = new com.ibm.rational.cm.web.ui.view.common.DoubleListBox({
			allowSearch: this.allowSearch,
			allowOrdering: this.allowOrdering,
			allowUserValue: this.allowUserValue,
			isSingleSelect: this.isSingleSelect
		});
		this.setContent(this._doubleListBox);
	},
	
	destroy: function(){
		var dblList = this._doubleListBox;
		if(dblList && dblList.domNode){
			dblList.destroyRecursive();
		}
		this.inherited(arguments);
	}
});

})();