/*
 * IBM Confidential
 * OCO Source Materials
 * Rational ClearQuest
 * (C) Copyright IBM Corp. 2009
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */


/********************************************************************
com.ibm.rational.cm.web.ui.view.filter.CqPromptDynamicFilters, is an extention of dojo.ContentPane to 
input values for dynamic filters of a query when running it.


********************************************************************/
dojo.provide("com.ibm.rational.cm.web.ui.view.filter.CqPromptDynamicFilters");

dojo.require("dijit._Widget");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("com.ibm.rational.cm.web.ui.view.filter.CqFilterInput");
dojo.require("com.ibm.rational.cm.web.ui.view.filter.CqFilterOPListBox");
dojo.require("com.ibm.rational.cm.web.ui.view.common.Tree");
dojo.require("com.ibm.rational.cm.web.ui.view.filter.CqTitlePane");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqQueryFilters");

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.filter.CqPromptDynamicFilters",
	[ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/filter/templates/CqPromptDynamicFilters.html"),
	/*
		could be used if we use Dojo Markup to classify this widget
	*/
	widgetType: "CqPromptDynamicFilters",
	
	/*
		filterData stores the JSON object structure for the filters
	*/
	filterData: [],
	
	/*
		inputData stores the JSON object structure for the values of filters
	*/	
	inputData: null,
	
	/*
		queryInfo the query information of filters
	*/	
	queryInfo: null,
	
	/*
		All i18n strings, can be used outside?
	*/	
	i18nStrings: null,
	
	inputWdgt: null,
	
	opwdgt: null,
	
	_getSelectedItem: function(){
		return this._selectedItem;
	},
	
	_fieldChanged: function(fldindex){
		var opwdgt = this.opwdgt;
		if(!opwdgt){
			opwdgt = this.opwdgt = this._createOPWidget();
		}
		opwdgt.setStore(this.filterData[fldindex].compops);
		opwdgt.setValue(this.inputData.filters[fldindex].op);
		
		// create the input and restore the value
		var inputWdgt = this.inputWdgt = this._createValueWidget(fldindex);
		opwdgt.connectInputWidget(inputWdgt);
				
		// change the selected after everything is done.
		this._changeInformation(false);
		
		// set the prompt information
		this.promptanchor.innerHTML = this.filterData[fldindex].prompt;
	},
	
	_createOPWidget: function(){
		var opwdgt = new com.ibm.rational.cm.web.ui.view.filter.CqFilterOPListBox({});
		this.operatoranchor.appendChild(opwdgt.domNode);
		opwdgt.connect(opwdgt, "onOpChanged", dojo.hitch(this, function(op){
			var item = this._getSelectedItem();
			if(item){
				this.inputData.filters[item.index].op = op;
			}
			this._changeInformation(true);
		}));
		this.opWidgetLabel.setAttribute("for", opwdgt.listwdgt.focusNode.id); // for accessibility
		return opwdgt;
	},
	
	_createValueWidget: function(fldindex){
		var inputWdgt = this.inputWdgt;
		if(inputWdgt){
			inputWdgt.destroy();
		}
	
		var localdata = this.inputData.filters[fldindex];
		var isSingle = (dojo.indexOf(this.filterData[fldindex].compops, 13) == -1);
		var choiceListInfo = null;
		if(this.filterData[fldindex].referencedRecordType ||
			(this.filterData[fldindex].hasDynamicChoiceList + "") == "true"){	// RATLC01262336 : Falcon: GetValueList is not called for field that has dynamic choice list during executing prompted query
																					// if(this.filterData[fldindex].hasDynamicChoiceList){ won't work
			choiceListInfo = {
				paramName: "fieldId",
				resourceId: this.filterData[fldindex].stableLocation
			};
		}
		inputWdgt = new com.ibm.rational.cm.web.ui.view.filter.CqFilterInput({
			op: localdata.op,
			fieldType: localdata.fieldType,
			valueList: this.filterData[fldindex].valueList,
			choiceListInfo: choiceListInfo,
			isSingleSelect: isSingle				
		});
		this.valuewidget.appendChild(inputWdgt.domNode);
		inputWdgt.connect(inputWdgt, "onValueChanged", dojo.hitch(this, function(values){
			var item = this._getSelectedItem();
			if(item){
				this.inputData.filters[item.index].valueList = values;
			}
			this._changeInformation(true);
		}));
		inputWdgt.setData(localdata.valueList);
		inputWdgt.connectOpWidget(this.opwdgt);
		
		// for accessibility
		var focusNode;
		if(inputWdgt.wdgt){
			this.valueWidgetLabel.setAttribute("for", inputWdgt.wdgt.focusNode.id);
		}
		return inputWdgt;
	},
	
	_onSubmit: function(){
		// convert DATE_TIME field to ISO-format date
		dojo.forEach(this.inputData.filters, function(filter){
			if(filter.fieldType == "DATE_TIME"){
				//RATLC01477492 - CQ 7.1 IHD: Problem running queries using history.action_timestamp
				dojo["require"]("com.ibm.rational.cm.web.ui.view.filter.CqFilterDateTextBox");
				filter.valueList = com.ibm.rational.cm.web.ui.view.filter.toISODate(filter.valueList);
			}
		});
		
		dojo.publish("CQ_QUERY_EXECUTE_WITH_DYNAMIC_FILTERS",[{
			filters: null,
			filtervalues: this.inputData,
			id: this.queryInfo.id,
			title: this.queryInfo.title,
			type: this.queryInfo.type
			
		}]);
	},	
	
	_changeInformation: function(/* boolean */ isFilterValueChanged){
		var item = this._getSelectedItem();
		var index = (item) ? item.index : 0;
		var fld = this.inputData.filters[index];
		var str = this.filterData[index].fieldPath; // get the field display name
		var op = fld.op;
		str += ' "' + this.i18nStrings.operator[op] + '"'; // get the op display name
		if(op != 0 && op != 11 && op != 12){ // Not "Null" or "Not Null"
			str += ' "';
			var dispDate = this.filterData[item.index].displayDate;
			if(dispDate){
				str += dispDate;
			}else if(fld.valueList){
				str += fld.valueList;
			}
			str += '"';
		}
		this.info.innerHTML = str; // change display in current filter pane
		
		if(isFilterValueChanged){
			var store = this.fieldsTree.store;
			if(store && item){
				store.setValue(item, "title", str); // set to tree title
			}
		}
	},

	postMixInProperties: function(){
		this.i18nStrings = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqQueryFilters", this.lang);
		this.inherited("postMixInProperties", arguments);
	},
	
	// bryanh (75767) - Corrected syntax errors
	_shouldDisable: function(wdgt1){
		if (wdgt1.state && wdgt1.state.length > 0)
		{	
			return true; 
		}
		else
		{
			return false; 
		}
	},

	/*
		overwrite the post create method to insert our filter Tab container with data
	*/	
	postCreate: function(args, frag){		
		this.inherited("postCreate", arguments);

		this.inputData = {
			resourceId: this.queryInfo.id,
			filters: new Array(this.filterData.length)
		};
		dojo.forEach(this.filterData, function(data, item){
			var choiceListInfo = null;
			// RATLC01235425 : Choice-list data should not be cached by referencedRecordType
			if(data.referencedRecordType || data.hasDynamicChoiceList){
				choiceListInfo = {
					resourceId: data.stableLocation
				};
			}

			var fieldType = data.fieldType;
			var path = data.fieldPath;
			this.inputData.filters[item] = {
				fieldPath: path,
				op: com.ibm.rational.cm.web.ui.view.filter.getDefaultOperator(fieldType, data.compops), 
				valueList: [],
				fieldType: fieldType,
				prompt: data.prompt,
				displayName: data.displayName,
				choiceListInfo: choiceListInfo
			};
			data.type = "FIELD";
			data.title = path;
			data.index = item;
			
			if(choiceListInfo){
				dojo["require"]("com.ibm.rational.cm.web.ui.view.filter.CqChoiceManager");
				com.ibm.rational.cm.web.ui.view.filter.choiceManager.fetchChoiceList(choiceListInfo);
			}
		}, this);
		
		var treeStore = new dojo.data.ItemFileWriteStore({ data: {
			identifier: 'index',
			label: 'title',
			type: 'type',
			items: this.filterData		
		}});
		var tree = new com.ibm.rational.cm.web.ui.view.common.Tree({
			store: treeStore,
			getIconClass: function(item, opened){return "TreeIcon_FIELD";}
			});
		this.fieldsTree = tree;
		this.field_tree_pane.appendChild(tree.domNode);
		// _fieldChanged
		this.connect(tree, "focusNode", dojo.hitch(this, function(node){
			this._selectedItem = node.item;
			this._fieldChanged(this._selectedItem.index);
		}));
		var children = tree.rootNode.getChildren();
		if(children.length > 0){
			this._selectedItem = children[0].item;
			this._fieldChanged(0);
		}
	},

	startup: function(){
		this.inherited("startup", arguments);
		
		// RATLC00745141 : RFE ClearQuest Web 2003.06.14 to have cursor in the value field of user-prompted filter type query
		var inputWdgt = this.inputWdgt;
		if(inputWdgt){
			var wdgt1 = inputWdgt.wdgt;
			if(wdgt1){
				wdgt1.focus();
			
				if(inputWdgt.fieldType == "DATE_TIME"){
					wdgt1._close();
				}

				// RATLC01266212 : Invalid query/report inputs still get processed, also problems with the calendar input (related)
				if(wdgt1.validate){
					wdgt1.connect(wdgt1, "validate", dojo.hitch(this, function(isFocused){
						this.runQryBtn.disabled = this._shouldDisable(wdgt1);  // disable [Ren Query] button if state is "Error" or "Warning"
					}));
				}
			}
		}


	},

	destroy: function(/*Boolean*/ finalize){
		if(this.opwdgt){ this.opwdgt.destroy(); }
		if(this.inputWdgt){ this.inputWdgt.destroy(); }
		this.inherited("destroy", arguments);
	}
});

com.ibm.rational.cm.web.ui.view.filter.prompt = {
	openPromptFilterPane: function(pageController, message, data){
		// destroy children
		// childTab.destroyContentWidget()

		// create content widget if it is not created yet
		// open a diaglog to input values for filters
		// var tabContainer = childTab.getParent();
		var contentWidget = new com.ibm.rational.cm.web.ui.view.filter.CqPromptDynamicFilters({
			title: message.title,
			queryInfo: message,
			filterData: data
		});

		pageController.setMainContent({
				id: message.id,
				type: message.type,
				content: contentWidget,
		// Don't cache the filter values prompt.
				preventCache: true,
				title: message.title});
		contentWidget.startup();
	}
}