/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.resultset.Pagination");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.NumberTextBox");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.Pagination",
	[ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/resultset/templates/Pagination.html"),

	startIndex: 1,
	count: 20,
	totalRows: 40,

	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages", this.lang);
		this.startIndex = parseInt(this.startIndex);
		this.count = parseInt(this.count);
		this._numOfItems = this.count;
		this.totalRows = parseInt(this.totalRows);
		this.inherited(arguments);
	},
	
	postCreate: function(){
		// don't format since it will add delimiter and indexColWidth cannot be used
		this.inherited(arguments);
		this.endCount.format = dijit.form.TextBox.prototype.format;
		this.update(this.startIndex, this._numOfItems, this.totalRows);
	},
	
	/*
	 * Called when "First" link is clicked
	 */
	first: function(){
		// if already at first record, do nothing
		if(this.startIndex == 1){ return; }
		
		var start = 1;
		this.startIndex = start;
		var numOfItems = this.count;
		this.onChange({
			startIndex: start,
			numOfItems: numOfItems
		});
	},
	
	/*
	 * Called when "Previous" link is clicked
	 */
	previous: function(){
		// if already at first record, do nothing
		if(this.startIndex == 1){ return; }

		var numOfItems = this.count;
		var start = this.startIndex - numOfItems;
		if(start < 1){
			start = 1;
			numOfItems = this.startIndex-1;
		}
		this.startIndex = start;
		this.onChange({
			startIndex: start,
			numOfItems: numOfItems
		});
	},
	
	/*
	 * Called when "Next" link is clicked
	 */
	next: function(){
		var start = this.startIndex + this._numOfItems;
		var totalRows = this.totalRows;
		if(start > totalRows){ return; }
		this.startIndex = start;
		var count = this.count;
		var numOfItems = (totalRows-start+1 < count) ? (totalRows-start+1) : count;
		this.onChange({
			startIndex: start,
			numOfItems: numOfItems
		});
	},
	
	/*
	 * Called when "Last" link is clicked
	 */
	last: function(){
		var totalRows = this.totalRows;
		var count = this.count;
		var numOfItems = totalRows % count;
		if(numOfItems === 0){
			numOfItems = count;
		}
		var start = totalRows - numOfItems + 1;
		if(start == this.startIndex){ return; }
		this.startIndex = start;
		this.onChange({
			startIndex: start,
			numOfItems: numOfItems
		});
	},

	countChanged: function(val){
		var newStart = parseInt(val) - this.count + 1;
		var numOfItems = this.count;
		if(newStart < 1){
			newStart = 1;
			numOfItems = this.startIndex;
		}
		if(this.startIndex == newStart){ return; }
		this.startIndex = newStart;
		this.onChange({
			startIndex: newStart,
			numOfItems: numOfItems
		});
	},
	
	/*
	 * When clicking "Enter", update the pagination if needed
	 */
	handleKeyPress: function(e){
		var endCount = this.endCount;
		if(e.keyCode == dojo.keys.ENTER && endCount.isValid()){
			this.countChanged(endCount.attr("value"));
		}
	},
	
	setCount: function(count){
		this.startIndex = 1;
		this.count = parseInt(count);
		this.onChange({
			startIndex: this.startIndex,
			count: this.count
		});
	},

	// a stub function that can be connected
	onChange: function(obj){
		this._updateLinks(obj.startIndex, obj.numOfItems, this.totalRows);
	},
	
	// enable/disable "Next" or "Previous" link depending on if there are more records to retrieve
	// the reason we are doing this complex way is because a:hover style is set in cqwebEx.css with !important
	_updateLinks: function(startIndex, numOfItems, totalRows){
		// enable/disable "Next" or "Previous" link depending on if there are more records to retrieve
		var CLS_DISABLED = "disabled";
		dojo.toggleClass(this.firstPrev, CLS_DISABLED, (startIndex <= 1));
		dojo.toggleClass(this.nextLast, CLS_DISABLED, (totalRows <= startIndex+numOfItems-1));
		
		// update displayed start/end count
		this.startCount.innerHTML = startIndex;
		this.endCount._setValueAttr(startIndex+numOfItems-1, false); // dojo 1.2 migration : second argument is no longer supported in setValue
		
		this._numOfItems = numOfItems;
	},
	
	update: function(startIndex, numOfItems, totalRows){
		this.startIndex = startIndex;
		this.totalRows = totalRows;
		
		// if no records were returned, hide pagination and return
		// RATLC01066731 : Display issues when refreshing queries that had a few hits, then no hits, then a few hits again
		// somehow end count is still shown even if it's parent node is visibility=hidden, so explicitly hide it here
		var noRecord = (numOfItems == 0);
		this.domNode.style.visibility = noRecord ? "hidden" : "visible";
		var endCount = this.endCount;
		endCount.textbox.style.display = noRecord ? "none" : "";
		if(noRecord){ return; }

		this._updateLinks(startIndex, numOfItems, totalRows);

		// update "n out of m" text
		endCount.invalidMessage = endCount.rangeMessage = dojo.string.substitute(this.messages.validationMsg, [1, totalRows]);
		endCount.constraints = {
			places: 0,
			min: 1,
			max: totalRows
		};
		this.outOfTotalCount.innerHTML = dojo.string.substitute(this.messages.nofmLabel, [totalRows]);
	}
});
