/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2008  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.resultset.ResultSetDialogContent");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.page.RecordTypeManager");
dojo.require("com.ibm.rational.cm.web.ui.view.resultset.Pagination");
dojo.require("com.ibm.rational.cm.web.ui.view.resultset.ResultSetGrid");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "common");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");

(function(){

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";
var recTypeManager = com.ibm.rational.cm.web.ui.page.recTypeManager;

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.ResultSetDialogContent",
	[ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/resultset/templates/ResultSetDialogContent.html"),
	
	recordType: null, // record type
	multiSelect: true,
	parentDialog: null,
	count: 20,

	postMixInProperties: function(){
		var getLoc = dojo.i18n.getLocalization;
		var lang = this.lang;
		this.commonStr = getLoc(cqNS, "common", lang);
		this.messages = getLoc(cqNS, "ResultSetMessages", lang);
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		var self = this;

		// pagination
		this.connect(this.pagination, "onChange", function(obj){
			self.executeQuery(obj);
		});
		
		// execute search when ENTER key is pressed
		this.connect(this.searchInputField, "onkeydown", function(e){
			if(e.keyCode == dojo.keys.ENTER){
				self.searchClicked(e);
			}
		});
		
		// Cleanup when parent dialog Cancel is invoked.
		this.connect(this.parentDialog, "onCancel", function(e){
			var grid = this.rsGrid;
			if(grid){
				grid.model.cancelFetch();
			}				
		});
		
		// initialize record type drop down
		this.setRecordType(this.recordType);
		recTypeManager.getRecordTypes({
			onBegin: function(){
				self.setLoading(true);
				self.disableSearch(true);
			},
			onLoad: function(allRecordTypes){
				// create drop down
				var dropDown = self.recTypeDropDown;
				dojo.forEach(dropDown.childNodes, function(child){
					dropDown.removeChild(child);
				});
				
				dojo.forEach(allRecordTypes, function(type){
					var opt = dojo.doc.createElement("OPTION");
					opt.value = type.stableLocation;
					opt.innerHTML = type.title;
					dropDown.appendChild(opt);
				});
				self._dropDownInit = true;
				
				// set specified recordType if specified, and if not set default record type
				self._setDropDownValue();
				self.disableSearch(false);
			},
			onHandle: function(){
				self.setLoading(false);
			}
		});
	},
	
	setRecordType: function(/* String */ recordType){
		this.recordType = recordType;
		
		if(this._dropDownInit){
			this._setDropDownValue();
		}
	},
	
	/*
	 * Sets dropdown value
	 * if no record type is specified, sets default record type
	 * enable/disable record type drop down
	 */
	_setDropDownValue: function(){
		var recordType = this.recordType;
		var dropDown = this.recTypeDropDown;
		if(recordType && recordType.length > 0){
			dropDown.value = recordType;
			dropDown.disabled = true;
		}else{
			var self = this;
			recTypeManager.getDefaultRecordType({
				onBegin: function(){
					self.setLoading(true);
					self.disableSearch(true);
				},
				onLoad: function(defaulRecordType){
					dropDown.value = defaulRecordType.RecordId;
					self._handleRecordTypeChange();
					self.disableSearch(false);
					self._dropDownInit = true;
				},
				onHandle: function(){
					self.setLoading(false);
				}
			});
			dropDown.disabled = false;
		}
	},
	
	_handleRecordTypeChange: function(){
		var self = this;
		recTypeManager.getFieldsForRecordType({
			recordType: this.recTypeDropDown.value,
			onBegin: function(){
				self.setLoading(true);
				self.disableSearch(true);
			},
			onLoad: function(fields){
				var arr = recTypeManager.getDefaultFields(fields, function(obj){
					return obj.FieldName;
				});
				self.columnInfo = dojo.map(arr, function(item, index){
					return { fieldPath: fields[item].FieldName };
				});
				self.disableSearch(false);
			},
			onHandle: function(){
				self.setLoading(false);
			}
		});
		this.updateDisplay(false);
	},

	executeQuery: function(/* Object */ queryParams, /* boolean */ refresh){
		var grid = this.rsGrid;
		if(!grid){
			// ExecuteUniqueKeyFilterQuery doesn't seem to support server-sort, so do client-sort
			// Falcon only do client-sort even if preference is server-sort
			queryParams.isServerSort = false;

			queryParams.elasticView = 0;
			this.setLoading(true);
			grid = this.rsGrid = new com.ibm.rational.cm.web.ui.view.resultset.ResultSetGrid(queryParams, this.gridNode);
		
			// grid
			var model = grid.model;
			var self = this;
			this.connect(model, "onBeforeFetch", function(){
				self.setLoading(true);
			});
			this.connect(model, "onComplete", "_onGridDataLoaded");
			this.connect(model, "onReturn", function(){
				self.setLoading(false);
			});
			this.connect(grid, "onSelectionChanged", function(){
				self.onSelectionChanged(grid.selection.getSelected());
			});
		}else{
			grid.sendRequest(queryParams, refresh);
		}
		grid.selection.multiSelect = this.multiSelect;
	},
	
	onSelectionChanged: function(/* Array */ selected){},
			
	// a stub function that can be connected
	setLoading: function(/* boolean */ loading){},
	
	disableSearch: function(disable){
		this.searchBtn.disabled = disable;
	},
	
	/*
	 * Called when [Search] button is clicked, and performs UniqueKeyFilterQuery or FullTextSearch
	 */
	searchClicked: function(e){
		var searchKey = dojo.trim(this.searchInputField.value);
		var queryParams = {
			resourceId: searchKey,
			startIndex: 1,
			count: this.count,
			searchRecordTypes: dojo.toJson([ this.recTypeDropDown.value ])
		};

		// RATLC01361542 : cannot duplicate record -- error from RPC server: full-text search not enabled
		// perform full text search only if if full text search is enabled
		// RATLC01229793 : Error returned when performing search with empty search key in duplicate record dialog
		if(!this.multiSelect && com.ibm.rational.cm.cqLoginController.getLoginInfo("isFullTextSearchEnabled") == true && searchKey.length > 0){ // when multiSelect=false, it is "Duplicate" record
			queryParams.action = "ExecuteFullTextSearch";
		}else{
			queryParams.action = "ExecuteUniqueKeyFilterQuery";
			var displayFields = dojo.map(this.columnInfo, function(col, idx){
				return {
					path: col.fieldPath,
					visible: true,
					label: col.fieldPath,
					sort: (idx === 0) ? "1" : "0", // RATLC01062313 : The results of searching costumer are not defaultly ordered by name
					order: 0
				};
			});
			queryParams.searchDisplayFields = dojo.toJson(displayFields);
		}
		this.executeQuery(queryParams, true);
	},
	
	/*
	 * Called when [Browse] button is clicked, and opens navigator dialog to select a query
	 */
	browseClicked: function(e){
//		dojo["require"]("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");
		var self = this;
		com.ibm.rational.cm.openNavigatorChooser({
			filter: {
				type: "LIST",
				recordType: self.recordType
			},
			onOk: function(arr){
				var dropDown = self.recTypeDropDown;
				if(!dropDown.disabled){
					dropDown.value = arr[0].recordType;
				}
				self.executeQuery({
					action: "ExecuteQuery",
					resourceId: arr[0].stableLocation,
					startIndex: 1,
					count: self.count
				}, true);
			},
			parentDialog: self.parentDialog
		});
	},
	
	/*
	 * Updates resultset toolbar when the data has been loaded
	 */
	_onGridDataLoaded: function(items, requestObject){
		var store = requestObject.store;
		var totalNumOfRows = 0;
		if(store){
			var dataObject = store.dataObject;
			if(dataObject){
				var resultSetData = dataObject.resultSetData;
				if(resultSetData){
					totalNumOfRows = resultSetData.totalNumOfRows;
				}
			}
		}

		var len = items.length;
		
		// update pagination
		this.pagination.update(this.rsGrid.startIndex, len, totalNumOfRows);
		
		this.updateDisplay((len > 0))

		if(len == 0){
			alert(this.messages.noRecordsMsg);
			return;
		}
	},
	
	/*
	 * Show/hide grid & pagination
	 */
	updateDisplay: function(/* boolean */ show){
		// update grid display
		var grid = this.rsGrid;
		if(grid){
			grid.domNode.style.visibility = (show) ? "visible" : "hidden";
			grid.selection.unselectAll(); // clear selection
		}

		// update pagination control
		this.pagination.domNode.style.display = show ? "" : "none";
	},
	
	/*
	 * Returns selected record information
	 * return value is an array containing object
	 */
	getValues: function(){
		var grid = this.rsGrid;
		if(!grid){
			return [];
		}
		var columnInfo = this.columnInfo;
		return dojo.map(grid.selection.getSelected(), function(sel){
			var obj = {};
			dojo.forEach(columnInfo, function(col){
				var field = col.fieldPath;
				obj[field] = grid.getCellData(sel, field);
			});
			obj.recordId = grid.getCellData(sel, "recordId");
			return obj;
		});
	}
});

})();