/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.schedule.ExclusionIterator"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.Timespan"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.Instant"); //$NON-NLS-1$

(function() {

var Timespan						= com.ibm.team.apt.client.internal.Timespan;
var Instant							= com.ibm.team.apt.client.internal.Instant;

var LOWEST							= new Instant(new Date(1));
var HIGHEST							= new Instant(new Date(Math.pow(2, 52)));

dojo.declare("com.ibm.team.apt.client.internal.schedule.ExclusionIterator", null, { //$NON-NLS-1$
	
	__current: null,
	__forward: true,
	
	constructor: function(wrapped) {
		this.__wrapped= wrapped;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	reset: function(earliestStart, forward) {
		this.__forward= forward;
		this.__wrapped.reset(earliestStart, forward);
		var wrappedCurrent= this.__wrapped.current();

		var begin= null;
		var end= null;

		while (wrappedCurrent != null && wrappedCurrent.contains(earliestStart)) {
			begin= forward ? wrappedCurrent.end : wrappedCurrent.start;
			wrappedCurrent= this.__wrapped.next();
		} 

		if (!begin) {
			begin= forward ? LOWEST : HIGHEST;
		}

		if (wrappedCurrent != null) {
			end= forward ? wrappedCurrent.start : wrappedCurrent.end;
		} else {
			end= forward ? HIGHEST : LOWEST;
		}

		if (forward) {
			this.__current= new Timespan(earliestStart.after(begin) ? earliestStart : begin, end);
		} else {
			this.__current= new Timespan(end, earliestStart.before(begin) ? earliestStart : begin);
		}
	},
	
	current: function() {
		return this.__current;
	},

	next: function() {
		if (this.__current != null) {
			if (this.__forward ? this.__current.end != HIGHEST : this.__current.start != LOWEST) {
				var wrappedLast= this.__wrapped.current();
				var wrappedNext= this.__wrapped.next();

				if (this.__forward) {
					this.__current= new Timespan(wrappedLast.end, wrappedNext != null ? wrappedNext.start : HIGHEST);
				} else {
					this.__current= new Timespan(wrappedNext != null ? wrappedNext.end : LOWEST, wrappedLast.start);
				}
			} else {
				this.__current= null;
			}
		}

		return this.__current;
	},

	__sentinel: null // terminates this class definition
});

})();
