/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.schedule.WorktimeIterator"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.Timespan"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.Instant"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.Duration"); //$NON-NLS-1$

(function() {

var Timespan						= com.ibm.team.apt.client.internal.Timespan;
var Instant							= com.ibm.team.apt.client.internal.Instant;
var Duration						= com.ibm.team.apt.client.Duration;

var DAY= new Duration(24 * 60 * 60 * 1000);

dojo.declare("com.ibm.team.apt.client.internal.schedule.WorktimeIterator", null, { //$NON-NLS-1$
	
	__current: null,
	__currentDay: null,

	constructor: function(workdays) {
		this.__workdays= [];
		var isValid= false;
		dojo.forEach(workdays, function(workday) {
			this.__workdays[workday.day - 1]= workday;
			isValid|= workday.workingTime > 0;
		}, this);
		
		if (!isValid) {
			var workDuration= 24*60*60*1000 - 1;
			dojo.forEach(workdays, function(workday) {
				workday.workingTime= workDuration;
				workday.endTime= workDuration;
			}, this);
		}
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	reset: function(earliestStart, forward) {
		this.__advanceFn= forward ? Instant.prototype.plus : Instant.prototype.minus;
		this.__currentDay= earliestStart;
		this.__validate();
		
		if (forward && earliestStart.after(this.__current.start)) {
			if (earliestStart.before(this.__current.end)) {
				this.__current= new Timespan(earliestStart, this.__current.end);
			} else {
				this.next();
			}
		} else if (!forward && earliestStart.before(this.__current.end)) {
			if (earliestStart.after(this.__current.start)) {
				this.__current= new Timespan(this.__current.start, earliestStart);
			} else {
				this.next();
			}
		}
	},
	
	current: function() {
		return this.__current;
	},

	next: function() {
		this.__currentDay= this.__advanceFn.call(this.__currentDay, DAY); //$NON-NLS-1$
		this.__validate();
		return this.__current;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__validate: function() {
		var date= this.__currentDay.getDate();
		while (true){
			var workDay= this.__workdays[date.getDay()];
			if (workDay.workingTime > 0) {
				var start= new Date(date.getFullYear(), date.getMonth(), date.getDate());
				start.setMilliseconds(workDay.endTime - workDay.workingTime);
				var end= new Date(date.getFullYear(), date.getMonth(), date.getDate());
				end.setMilliseconds(workDay.endTime);
				this.__current= new Timespan(new Instant(start), new Instant(end));
				break;
			}
			
			this.__currentDay= this.__advanceFn.call(this.__currentDay, DAY);
			date= this.__currentDay.getDate();
		};
	},
	
	__sentinel: null // terminates this class definition
});

})();
