/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.structure.ViewModeTransformer"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanModel"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.structure.GroupElement"); //$NON-NLS-1$

(function() {
	
var GroupElement		= com.ibm.team.apt.ui.structure.GroupElement;
var PlanModel			= com.ibm.team.apt.client.PlanModel;

var DeltaKind = {
	Added: "Added",  //$NON-NLS-1$
	Removed: "Removed",  //$NON-NLS-1$
	Changed: "Changed",  //$NON-NLS-1$
	Ignore: "Ignore",  //$NON-NLS-1$
	Refresh: "Refresh" //$NON-NLS-1$
};



var DefaultRootElement= null;

dojo.declare("com.ibm.team.apt.ui.structure.ViewModeTransformer", null, { //$NON-NLS-1$

	__viewModel: null,
	__planListener: null,

	constructor: function() {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	loadOptions: function(optionsStore) {
	},

	storeOptions: function(readAccessor) {
	},
	
	getEntryComparator: function() {
		return undefined;
	},

	getViewModel: function() {
		return this.__viewModel;
	},
	
	getElementsToExpandInitially: function() {
		return [];
	},

	calculateLocationMarker: function(entry) {
		return null;
	},
	
	// ---- spi ------------------------------------------------------------------------------------------------------------
	
	onBeginRebuild: function() {
	},

	rebuildModel: function(updateAccessor) {
	},

	onEndRebuild: function() {
	},

	computeDeltaKind: function(delta, readAccessor) {
		return DeltaKind.Ignore;
	},

	processPlanChanged: function(delta, updateAccessor) {
		return true;
	},

	processElementAdded: function(delta, updateAccessor) {
		return true;
	},

	processElementMoved: function(removeDelta, addDelta, updateAccessor) {
		return true;
	},

	processElementRemoved: function(delta, updateAccessor) {
		return true;
	},

	processElementChanged: function(delta, updateAccessor) {
		return true;
	},
	
	updatePlanElementData: function(delta) {
	},

	// ---- item move policy -----------------------------------------------------------------------------------------------
	
	canMove: function(request, readAccessor) {
		return null;
	},

	move: function(response, updateAccessor) {
	},
	
	canCreate: function(request, readAccessor) {
		return null;
	},
	
	create: function(response, updateAccessor) {
	},

	// ---- ModelTransformer implementation --------------------------------------------------------------------------------

	inputChanged: function(model, newInput) {
		this.__viewModel= model;
		if (this.__plan != null) {
			this.__plan.removeListener(this.__planListener);
		}

		this.__plan= newInput;

		if (this.__plan != null) {
			this.__planListener= dojo.hitch(this, this.__handlePlanChange);
			this.__plan.addListener(this.__planListener);
			// TODO from now on, event's might be sent to us, however, strictly speaking, we are not ready to process
			// events until the refreshModel method was invoked..
		}
	},

	refreshModel: function(rebuilder) {
		this.onBeginRebuild();
		try {
			rebuilder.createRootEntry(DefaultRootElement);
			this.rebuildModel(rebuilder);
		} finally {
			this.onEndRebuild();
		}
	},

	// ---- Implementation: delta dispatching ------------------------------------------------------------------------------

	__handlePlanChange: function(event) {
		this.__viewModel.updateModel(function(updateAccessor) {
			var refreshModel= false;

			event.getDelta().accept(dojo.hitch(this, function(delta) {
				if (refreshModel)
					return false;

				var visitChildren= true;
				try {
					if (delta.getPlanElement() instanceof PlanModel && delta.isChanged()) {
						return this.processPlanChanged(delta, updateAccessor);
					} else {
						var otherMoveDelta= delta.getOtherMoveDelta();
						var deltaKind= this.computeDeltaKind(delta, updateAccessor); 
						switch (deltaKind) {
							case DeltaKind.Added:
								if (otherMoveDelta == null) {
									visitChildren= this.processElementAdded(delta, updateAccessor);
								} else {
									visitChildren= this.processElementMoved(otherMoveDelta, delta, updateAccessor);
								}
								break;
							case DeltaKind.Removed:
								if (otherMoveDelta == null) {
									visitChildren= this.processElementRemoved(delta, updateAccessor);
								} else {
									// intentionally empty -> we call processElementMoved while processing the Added delta
								}
								break;
							case DeltaKind.Changed:
								visitChildren= this.processElementChanged(delta, updateAccessor);
								break;
							case DeltaKind.Refresh:
								refreshModel= true;
								visitChildren= false;
								break;
						}
					}
				} finally {
					if (!refreshModel) {
						if (visitChildren) {
							this.updatePlanElementData(delta);
						} else {
							delta.accept(dojo.hitch(this, function(delta) {
								this.updatePlanElementData(delta);
								return true;
							}));
						}
					}
				}
				return visitChildren;
			
			}));

			if (refreshModel) {
				this.storeOptions(updateAccessor);
				this.__viewModel.viewerPreservingRefresh();
			}
		}, this);
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.ui.structure.ViewModeTransformer.DeltaKind= DeltaKind;

dojo.declare("com.ibm.team.apt.ui.structure.ViewModeTransformer.__DefaultRootElement", GroupElement, { //$NON-NLS-1$

	constructor: function() {
	},

	// ---- GroupElement implementation ------------------------------------------------------------------------------------

	equals: function(other) {
		return other === this;
	},

	getId: function() {
		return "_YpOR48YpEd2DQP0MvE2Tkw"; //$NON-NLS-1$
	},

	compareTo: function(other) {
		this.equals(other) ? 0 : -1;
	},
	
	canAdopt: function() {
		return false;
	},

	__sentinel: null // terminates this class definition
});

DefaultRootElement= new com.ibm.team.apt.ui.structure.ViewModeTransformer.__DefaultRootElement();

})();
